/*
 Highmaps JS v5.0.8 (2017-03-08)

 (c) 2011-2016 Torstein Honsi

 License: www.highcharts.com/license
*/
(function(J,a){"object"===typeof module&&module.exports?module.exports=J.document?a(J):a:J.Highcharts=a(J)})("undefined"!==typeof window?window:this,function(J){J=function(){var a=window,y=a.document,E=a.navigator&&a.navigator.userAgent||"",A=y&&y.createElementNS&&!!y.createElementNS("http://www.w3.org/2000/svg","svg").createSVGRect,B=/(edge|msie|trident)/i.test(E)&&!window.opera,h=!A,g=/Firefox/.test(E),t=g&&4>parseInt(E.split("Firefox/")[1],10);return a.Highcharts?a.Highcharts.error(16,!0):{product:"Highmaps",
version:"5.0.8",deg2rad:2*Math.PI/360,doc:y,hasBidiBug:t,hasTouch:y&&void 0!==y.documentElement.ontouchstart,isMS:B,isWebKit:/AppleWebKit/.test(E),isFirefox:g,isTouchDevice:/(Mobile|Android|Windows Phone)/.test(E),SVG_NS:"http://www.w3.org/2000/svg",chartCount:0,seriesTypes:{},symbolSizes:{},svg:A,vml:h,win:a,charts:[],marginNames:["plotTop","marginRight","marginBottom","plotLeft"],noop:function(){}}}();(function(a){var y=[],E=a.charts,A=a.doc,B=a.win;a.error=function(h,g){h=a.isNumber(h)?"Highcharts error #"+
h+": www.highcharts.com/errors/"+h:h;if(g)throw Error(h);B.console&&console.log(h)};a.Fx=function(a,g,t){this.options=g;this.elem=a;this.prop=t};a.Fx.prototype={dSetter:function(){var a=this.paths[0],g=this.paths[1],t=[],r=this.now,l=a.length,f;if(1===r)t=this.toD;else if(l===g.length&&1>r)for(;l--;)f=parseFloat(a[l]),t[l]=isNaN(f)?a[l]:r*parseFloat(g[l]-f)+f;else t=g;this.elem.attr("d",t,null,!0)},update:function(){var a=this.elem,g=this.prop,t=this.now,r=this.options.step;if(this[g+"Setter"])this[g+
"Setter"]();else a.attr?a.element&&a.attr(g,t,null,!0):a.style[g]=t+this.unit;r&&r.call(a,t,this)},run:function(a,g,t){var h=this,l=function(a){return l.stopped?!1:h.step(a)},f;this.startTime=+new Date;this.start=a;this.end=g;this.unit=t;this.now=this.start;this.pos=0;l.elem=this.elem;l.prop=this.prop;l()&&1===y.push(l)&&(l.timerId=setInterval(function(){for(f=0;f<y.length;f++)y[f]()||y.splice(f--,1);y.length||clearInterval(l.timerId)},13))},step:function(a){var h=+new Date,t,r=this.options;t=this.elem;
var l=r.complete,f=r.duration,e=r.curAnim,n;if(t.attr&&!t.element)t=!1;else if(a||h>=f+this.startTime){this.now=this.end;this.pos=1;this.update();a=e[this.prop]=!0;for(n in e)!0!==e[n]&&(a=!1);a&&l&&l.call(t);t=!1}else this.pos=r.easing((h-this.startTime)/f),this.now=this.start+(this.end-this.start)*this.pos,this.update(),t=!0;return t},initPath:function(h,g,t){function r(a){var b,c;for(d=a.length;d--;)b="M"===a[d]||"L"===a[d],c=/[a-zA-Z]/.test(a[d+3]),b&&c&&a.splice(d+1,0,a[d+1],a[d+2],a[d+1],a[d+
2])}function l(a,c){for(;a.length<m;){a[0]=c[m-a.length];var q=a.slice(0,b);[].splice.apply(a,[0,0].concat(q));D&&(q=a.slice(a.length-b),[].splice.apply(a,[a.length,0].concat(q)),d--)}a[0]="M"}function f(a,d){for(var q=(m-a.length)/b;0<q&&q--;)c=a.slice().splice(a.length/u-b,b*u),c[0]=d[m-b-q*b],v&&(c[b-6]=c[b-2],c[b-5]=c[b-1]),[].splice.apply(a,[a.length/u,0].concat(c)),D&&q--}g=g||"";var e,n=h.startX,z=h.endX,v=-1<g.indexOf("C"),b=v?7:3,m,c,d;g=g.split(" ");t=t.slice();var D=h.isArea,u=D?2:1,q;
v&&(r(g),r(t));if(n&&z){for(d=0;d<n.length;d++)if(n[d]===z[0]){e=d;break}else if(n[0]===z[z.length-n.length+d]){e=d;q=!0;break}void 0===e&&(g=[])}g.length&&a.isNumber(e)&&(m=t.length+e*u*b,q?(l(g,t),f(t,g)):(l(t,g),f(g,t)));return[g,t]}};a.extend=function(a,g){var h;a||(a={});for(h in g)a[h]=g[h];return a};a.merge=function(){var h,g=arguments,t,r={},l=function(f,e){var n,h;"object"!==typeof f&&(f={});for(h in e)e.hasOwnProperty(h)&&(n=e[h],a.isObject(n,!0)&&"renderTo"!==h&&"number"!==typeof n.nodeType?
f[h]=l(f[h]||{},n):f[h]=e[h]);return f};!0===g[0]&&(r=g[1],g=Array.prototype.slice.call(g,2));t=g.length;for(h=0;h<t;h++)r=l(r,g[h]);return r};a.pInt=function(a,g){return parseInt(a,g||10)};a.isString=function(a){return"string"===typeof a};a.isArray=function(a){a=Object.prototype.toString.call(a);return"[object Array]"===a||"[object Array Iterator]"===a};a.isObject=function(h,g){return h&&"object"===typeof h&&(!g||!a.isArray(h))};a.isNumber=function(a){return"number"===typeof a&&!isNaN(a)};a.erase=
function(a,g){for(var h=a.length;h--;)if(a[h]===g){a.splice(h,1);break}};a.defined=function(a){return void 0!==a&&null!==a};a.attr=function(h,g,t){var r,l;if(a.isString(g))a.defined(t)?h.setAttribute(g,t):h&&h.getAttribute&&(l=h.getAttribute(g));else if(a.defined(g)&&a.isObject(g))for(r in g)h.setAttribute(r,g[r]);return l};a.splat=function(h){return a.isArray(h)?h:[h]};a.syncTimeout=function(a,g,t){if(g)return setTimeout(a,g,t);a.call(0,t)};a.pick=function(){var a=arguments,g,t,r=a.length;for(g=
0;g<r;g++)if(t=a[g],void 0!==t&&null!==t)return t};a.css=function(h,g){a.isMS&&!a.svg&&g&&void 0!==g.opacity&&(g.filter="alpha(opacity\x3d"+100*g.opacity+")");a.extend(h.style,g)};a.createElement=function(h,g,t,r,l){h=A.createElement(h);var f=a.css;g&&a.extend(h,g);l&&f(h,{padding:0,border:"none",margin:0});t&&f(h,t);r&&r.appendChild(h);return h};a.extendClass=function(h,g){var t=function(){};t.prototype=new h;a.extend(t.prototype,g);return t};a.pad=function(a,g,t){return Array((g||2)+1-String(a).length).join(t||
0)+a};a.relativeLength=function(a,g){return/%$/.test(a)?g*parseFloat(a)/100:parseFloat(a)};a.wrap=function(a,g,t){var h=a[g];a[g]=function(){var a=Array.prototype.slice.call(arguments),f=arguments,e=this;e.proceed=function(){h.apply(e,arguments.length?arguments:f)};a.unshift(h);a=t.apply(this,a);e.proceed=null;return a}};a.getTZOffset=function(h){var g=a.Date;return 6E4*(g.hcGetTimezoneOffset&&g.hcGetTimezoneOffset(h)||g.hcTimezoneOffset||0)};a.dateFormat=function(h,g,t){if(!a.defined(g)||isNaN(g))return a.defaultOptions.lang.invalidDate||
"";h=a.pick(h,"%Y-%m-%d %H:%M:%S");var r=a.Date,l=new r(g-a.getTZOffset(g)),f,e=l[r.hcGetHours](),n=l[r.hcGetDay](),z=l[r.hcGetDate](),v=l[r.hcGetMonth](),b=l[r.hcGetFullYear](),m=a.defaultOptions.lang,c=m.weekdays,d=m.shortWeekdays,D=a.pad,r=a.extend({a:d?d[n]:c[n].substr(0,3),A:c[n],d:D(z),e:D(z,2," "),w:n,b:m.shortMonths[v],B:m.months[v],m:D(v+1),y:b.toString().substr(2,2),Y:b,H:D(e),k:e,I:D(e%12||12),l:e%12||12,M:D(l[r.hcGetMinutes]()),p:12>e?"AM":"PM",P:12>e?"am":"pm",S:D(l.getSeconds()),L:D(Math.round(g%
1E3),3)},a.dateFormats);for(f in r)for(;-1!==h.indexOf("%"+f);)h=h.replace("%"+f,"function"===typeof r[f]?r[f](g):r[f]);return t?h.substr(0,1).toUpperCase()+h.substr(1):h};a.formatSingle=function(h,g){var t=/\.([0-9])/,r=a.defaultOptions.lang;/f$/.test(h)?(t=(t=h.match(t))?t[1]:-1,null!==g&&(g=a.numberFormat(g,t,r.decimalPoint,-1<h.indexOf(",")?r.thousandsSep:""))):g=a.dateFormat(h,g);return g};a.format=function(h,g){for(var t="{",r=!1,l,f,e,n,z=[],v;h;){t=h.indexOf(t);if(-1===t)break;l=h.slice(0,
t);if(r){l=l.split(":");f=l.shift().split(".");n=f.length;v=g;for(e=0;e<n;e++)v=v[f[e]];l.length&&(v=a.formatSingle(l.join(":"),v));z.push(v)}else z.push(l);h=h.slice(t+1);t=(r=!r)?"}":"{"}z.push(h);return z.join("")};a.getMagnitude=function(a){return Math.pow(10,Math.floor(Math.log(a)/Math.LN10))};a.normalizeTickInterval=function(h,g,t,r,l){var f,e=h;t=a.pick(t,1);f=h/t;g||(g=l?[1,1.2,1.5,2,2.5,3,4,5,6,8,10]:[1,2,2.5,5,10],!1===r&&(1===t?g=a.grep(g,function(a){return 0===a%1}):.1>=t&&(g=[1/t])));
for(r=0;r<g.length&&!(e=g[r],l&&e*t>=h||!l&&f<=(g[r]+(g[r+1]||g[r]))/2);r++);return e=a.correctFloat(e*t,-Math.round(Math.log(.001)/Math.LN10))};a.stableSort=function(a,g){var h=a.length,r,l;for(l=0;l<h;l++)a[l].safeI=l;a.sort(function(a,e){r=g(a,e);return 0===r?a.safeI-e.safeI:r});for(l=0;l<h;l++)delete a[l].safeI};a.arrayMin=function(a){for(var g=a.length,h=a[0];g--;)a[g]<h&&(h=a[g]);return h};a.arrayMax=function(a){for(var g=a.length,h=a[0];g--;)a[g]>h&&(h=a[g]);return h};a.destroyObjectProperties=
function(a,g){for(var h in a)a[h]&&a[h]!==g&&a[h].destroy&&a[h].destroy(),delete a[h]};a.discardElement=function(h){var g=a.garbageBin;g||(g=a.createElement("div"));h&&g.appendChild(h);g.innerHTML=""};a.correctFloat=function(a,g){return parseFloat(a.toPrecision(g||14))};a.setAnimation=function(h,g){g.renderer.globalAnimation=a.pick(h,g.options.chart.animation,!0)};a.animObject=function(h){return a.isObject(h)?a.merge(h):{duration:h?500:0}};a.timeUnits={millisecond:1,second:1E3,minute:6E4,hour:36E5,
day:864E5,week:6048E5,month:24192E5,year:314496E5};a.numberFormat=function(h,g,t,r){h=+h||0;g=+g;var l=a.defaultOptions.lang,f=(h.toString().split(".")[1]||"").length,e,n;-1===g?g=Math.min(f,20):a.isNumber(g)||(g=2);n=(Math.abs(h)+Math.pow(10,-Math.max(g,f)-1)).toFixed(g);f=String(a.pInt(n));e=3<f.length?f.length%3:0;t=a.pick(t,l.decimalPoint);r=a.pick(r,l.thousandsSep);h=(0>h?"-":"")+(e?f.substr(0,e)+r:"");h+=f.substr(e).replace(/(\d{3})(?=\d)/g,"$1"+r);g&&(h+=t+n.slice(-g));return h};Math.easeInOutSine=
function(a){return-.5*(Math.cos(Math.PI*a)-1)};a.getStyle=function(h,g){return"width"===g?Math.min(h.offsetWidth,h.scrollWidth)-a.getStyle(h,"padding-left")-a.getStyle(h,"padding-right"):"height"===g?Math.min(h.offsetHeight,h.scrollHeight)-a.getStyle(h,"padding-top")-a.getStyle(h,"padding-bottom"):(h=B.getComputedStyle(h,void 0))&&a.pInt(h.getPropertyValue(g))};a.inArray=function(a,g){return g.indexOf?g.indexOf(a):[].indexOf.call(g,a)};a.grep=function(a,g){return[].filter.call(a,g)};a.find=function(a,
g){return[].find.call(a,g)};a.map=function(a,g){for(var h=[],r=0,l=a.length;r<l;r++)h[r]=g.call(a[r],a[r],r,a);return h};a.offset=function(a){var g=A.documentElement;a=a.getBoundingClientRect();return{top:a.top+(B.pageYOffset||g.scrollTop)-(g.clientTop||0),left:a.left+(B.pageXOffset||g.scrollLeft)-(g.clientLeft||0)}};a.stop=function(a,g){for(var h=y.length;h--;)y[h].elem!==a||g&&g!==y[h].prop||(y[h].stopped=!0)};a.each=function(a,g,t){return Array.prototype.forEach.call(a,g,t)};a.addEvent=function(h,
g,t){function r(a){a.target=a.srcElement||B;t.call(h,a)}var l=h.hcEvents=h.hcEvents||{};h.addEventListener?h.addEventListener(g,t,!1):h.attachEvent&&(h.hcEventsIE||(h.hcEventsIE={}),h.hcEventsIE[t.toString()]=r,h.attachEvent("on"+g,r));l[g]||(l[g]=[]);l[g].push(t);return function(){a.removeEvent(h,g,t)}};a.removeEvent=function(h,g,t){function r(a,f){h.removeEventListener?h.removeEventListener(a,f,!1):h.attachEvent&&(f=h.hcEventsIE[f.toString()],h.detachEvent("on"+a,f))}function l(){var a,f;if(h.nodeName)for(f in g?
(a={},a[g]=!0):a=e,a)if(e[f])for(a=e[f].length;a--;)r(f,e[f][a])}var f,e=h.hcEvents,n;e&&(g?(f=e[g]||[],t?(n=a.inArray(t,f),-1<n&&(f.splice(n,1),e[g]=f),r(g,t)):(l(),e[g]=[])):(l(),h.hcEvents={}))};a.fireEvent=function(h,g,t,r){var l;l=h.hcEvents;var f,e;t=t||{};if(A.createEvent&&(h.dispatchEvent||h.fireEvent))l=A.createEvent("Events"),l.initEvent(g,!0,!0),a.extend(l,t),h.dispatchEvent?h.dispatchEvent(l):h.fireEvent(g,l);else if(l)for(l=l[g]||[],f=l.length,t.target||a.extend(t,{preventDefault:function(){t.defaultPrevented=
!0},target:h,type:g}),g=0;g<f;g++)(e=l[g])&&!1===e.call(h,t)&&t.preventDefault();r&&!t.defaultPrevented&&r(t)};a.animate=function(h,g,t){var r,l="",f,e,n;a.isObject(t)||(r=arguments,t={duration:r[2],easing:r[3],complete:r[4]});a.isNumber(t.duration)||(t.duration=400);t.easing="function"===typeof t.easing?t.easing:Math[t.easing]||Math.easeInOutSine;t.curAnim=a.merge(g);for(n in g)a.stop(h,n),e=new a.Fx(h,t,n),f=null,"d"===n?(e.paths=e.initPath(h,h.d,g.d),e.toD=g.d,r=0,f=1):h.attr?r=h.attr(n):(r=parseFloat(a.getStyle(h,
n))||0,"opacity"!==n&&(l="px")),f||(f=g[n]),f&&f.match&&f.match("px")&&(f=f.replace(/px/g,"")),e.run(r,f,l)};a.seriesType=function(h,g,t,r,l){var f=a.getOptions(),e=a.seriesTypes;f.plotOptions[h]=a.merge(f.plotOptions[g],t);e[h]=a.extendClass(e[g]||function(){},r);e[h].prototype.type=h;l&&(e[h].prototype.pointClass=a.extendClass(a.Point,l));return e[h]};a.uniqueKey=function(){var a=Math.random().toString(36).substring(2,9),g=0;return function(){return"highcharts-"+a+"-"+g++}}();B.jQuery&&(B.jQuery.fn.highcharts=
function(){var h=[].slice.call(arguments);if(this[0])return h[0]?(new (a[a.isString(h[0])?h.shift():"Chart"])(this[0],h[0],h[1]),this):E[a.attr(this[0],"data-highcharts-chart")]});A&&!A.defaultView&&(a.getStyle=function(h,g){var t={width:"clientWidth",height:"clientHeight"}[g];if(h.style[g])return a.pInt(h.style[g]);"opacity"===g&&(g="filter");if(t)return h.style.zoom=1,Math.max(h[t]-2*a.getStyle(h,"padding"),0);h=h.currentStyle[g.replace(/\-(\w)/g,function(a,l){return l.toUpperCase()})];"filter"===
g&&(h=h.replace(/alpha\(opacity=([0-9]+)\)/,function(a,l){return l/100}));return""===h?1:a.pInt(h)});Array.prototype.forEach||(a.each=function(a,g,t){for(var h=0,l=a.length;h<l;h++)if(!1===g.call(t,a[h],h,a))return h});Array.prototype.indexOf||(a.inArray=function(a,g){var h,r=0;if(g)for(h=g.length;r<h;r++)if(g[r]===a)return r;return-1});Array.prototype.filter||(a.grep=function(a,g){for(var h=[],r=0,l=a.length;r<l;r++)g(a[r],r)&&h.push(a[r]);return h});Array.prototype.find||(a.find=function(a,g){var h,
r=a.length;for(h=0;h<r;h++)if(g(a[h],h))return a[h]})})(J);(function(a){var y=a.each,E=a.isNumber,A=a.map,B=a.merge,h=a.pInt;a.Color=function(g){if(!(this instanceof a.Color))return new a.Color(g);this.init(g)};a.Color.prototype={parsers:[{regex:/rgba\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]?(?:\.[0-9]+)?)\s*\)/,parse:function(a){return[h(a[1]),h(a[2]),h(a[3]),parseFloat(a[4],10)]}},{regex:/rgb\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*\)/,parse:function(a){return[h(a[1]),
h(a[2]),h(a[3]),1]}}],names:{white:"#ffffff",black:"#000000"},init:function(g){var h,r,l,f;if((this.input=g=this.names[g]||g)&&g.stops)this.stops=A(g.stops,function(f){return new a.Color(f[1])});else if(g&&"#"===g[0]&&(h=g.length,g=parseInt(g.substr(1),16),7===h?r=[(g&16711680)>>16,(g&65280)>>8,g&255,1]:4===h&&(r=[(g&3840)>>4|(g&3840)>>8,(g&240)>>4|g&240,(g&15)<<4|g&15,1])),!r)for(l=this.parsers.length;l--&&!r;)f=this.parsers[l],(h=f.regex.exec(g))&&(r=f.parse(h));this.rgba=r||[]},get:function(a){var h=
this.input,g=this.rgba,l;this.stops?(l=B(h),l.stops=[].concat(l.stops),y(this.stops,function(f,e){l.stops[e]=[l.stops[e][0],f.get(a)]})):l=g&&E(g[0])?"rgb"===a||!a&&1===g[3]?"rgb("+g[0]+","+g[1]+","+g[2]+")":"a"===a?g[3]:"rgba("+g.join(",")+")":h;return l},brighten:function(a){var g,r=this.rgba;if(this.stops)y(this.stops,function(l){l.brighten(a)});else if(E(a)&&0!==a)for(g=0;3>g;g++)r[g]+=h(255*a),0>r[g]&&(r[g]=0),255<r[g]&&(r[g]=255);return this},setOpacity:function(a){this.rgba[3]=a;return this}};
a.color=function(g){return new a.Color(g)}})(J);(function(a){function y(){var l=a.defaultOptions.global,f=r.moment;if(l.timezone){if(f)return function(a){return-f.tz(a,l.timezone).utcOffset()};a.error(25)}return l.useUTC&&l.getTimezoneOffset}function E(){var l=a.defaultOptions.global,f,e=l.useUTC,n=e?"getUTC":"get",g=e?"setUTC":"set";a.Date=f=l.Date||r.Date;f.hcTimezoneOffset=e&&l.timezoneOffset;f.hcGetTimezoneOffset=y();f.hcMakeTime=function(a,b,m,c,d,n){var u;e?(u=f.UTC.apply(0,arguments),u+=h(u)):
u=(new f(a,b,t(m,1),t(c,0),t(d,0),t(n,0))).getTime();return u};B("Minutes Hours Day Date Month FullYear".split(" "),function(a){f["hcGet"+a]=n+a});B("Milliseconds Seconds Minutes Hours Date Month FullYear".split(" "),function(a){f["hcSet"+a]=g+a})}var A=a.color,B=a.each,h=a.getTZOffset,g=a.merge,t=a.pick,r=a.win;a.defaultOptions={colors:"#7cb5ec #434348 #90ed7d #f7a35c #8085e9 #f15c80 #e4d354 #2b908f #f45b5b #91e8e1".split(" "),symbols:["circle","diamond","square","triangle","triangle-down"],lang:{loading:"Loading...",
months:"January February March April May June July August September October November December".split(" "),shortMonths:"Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec".split(" "),weekdays:"Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),decimalPoint:".",numericSymbols:"kMGTPE".split(""),resetZoom:"Reset zoom",resetZoomTitle:"Reset zoom level 1:1",thousandsSep:" "},global:{useUTC:!0,VMLRadialGradientURL:"http://code.highcharts.com/5.0.8/gfx/vml-radial-gradient.png"},chart:{borderRadius:0,
defaultSeriesType:"line",ignoreHiddenSeries:!0,spacing:[10,10,15,10],resetZoomButton:{theme:{zIndex:20},position:{align:"right",x:-10,y:10}},width:null,height:null,borderColor:"#335cad",backgroundColor:"#ffffff",plotBorderColor:"#cccccc"},title:{text:"Chart title",align:"center",margin:15,widthAdjust:-44},subtitle:{text:"",align:"center",widthAdjust:-44},plotOptions:{},labels:{style:{position:"absolute",color:"#333333"}},legend:{enabled:!0,align:"center",layout:"horizontal",labelFormatter:function(){return this.name},
borderColor:"#999999",borderRadius:0,navigation:{activeColor:"#003399",inactiveColor:"#cccccc"},itemStyle:{color:"#333333",fontSize:"12px",fontWeight:"bold"},itemHoverStyle:{color:"#000000"},itemHiddenStyle:{color:"#cccccc"},shadow:!1,itemCheckboxStyle:{position:"absolute",width:"13px",height:"13px"},squareSymbol:!0,symbolPadding:5,verticalAlign:"bottom",x:0,y:0,title:{style:{fontWeight:"bold"}}},loading:{labelStyle:{fontWeight:"bold",position:"relative",top:"45%"},style:{position:"absolute",backgroundColor:"#ffffff",
opacity:.5,textAlign:"center"}},tooltip:{enabled:!0,animation:a.svg,borderRadius:3,dateTimeLabelFormats:{millisecond:"%A, %b %e, %H:%M:%S.%L",second:"%A, %b %e, %H:%M:%S",minute:"%A, %b %e, %H:%M",hour:"%A, %b %e, %H:%M",day:"%A, %b %e, %Y",week:"Week from %A, %b %e, %Y",month:"%B %Y",year:"%Y"},footerFormat:"",padding:8,snap:a.isTouchDevice?25:10,backgroundColor:A("#f7f7f7").setOpacity(.85).get(),borderWidth:1,headerFormat:'\x3cspan style\x3d"font-size: 10px"\x3e{point.key}\x3c/span\x3e\x3cbr/\x3e',
pointFormat:'\x3cspan style\x3d"color:{point.color}"\x3e\u25cf\x3c/span\x3e {series.name}: \x3cb\x3e{point.y}\x3c/b\x3e\x3cbr/\x3e',shadow:!0,style:{color:"#333333",cursor:"default",fontSize:"12px",pointerEvents:"none",whiteSpace:"nowrap"}},credits:{enabled:!0,href:"http://www.highcharts.com",position:{align:"right",x:-10,verticalAlign:"bottom",y:-5},style:{cursor:"pointer",color:"#999999",fontSize:"9px"},text:"Highcharts.com"}};a.setOptions=function(l){a.defaultOptions=g(!0,a.defaultOptions,l);E();
return a.defaultOptions};a.getOptions=function(){return a.defaultOptions};a.defaultPlotOptions=a.defaultOptions.plotOptions;E()})(J);(function(a){var y,E,A=a.addEvent,B=a.animate,h=a.attr,g=a.charts,t=a.color,r=a.css,l=a.createElement,f=a.defined,e=a.deg2rad,n=a.destroyObjectProperties,z=a.doc,v=a.each,b=a.extend,m=a.erase,c=a.grep,d=a.hasTouch,D=a.inArray,u=a.isArray,q=a.isFirefox,x=a.isMS,G=a.isObject,H=a.isString,k=a.isWebKit,C=a.merge,I=a.noop,L=a.pick,K=a.pInt,p=a.removeEvent,F=a.stop,M=a.svg,
N=a.SVG_NS,S=a.symbolSizes,O=a.win;y=a.SVGElement=function(){return this};y.prototype={opacity:1,SVG_NS:N,textProps:"direction fontSize fontWeight fontFamily fontStyle color lineHeight width textAlign textDecoration textOverflow textOutline".split(" "),init:function(a,b){this.element="span"===b?l(b):z.createElementNS(this.SVG_NS,b);this.renderer=a},animate:function(w,b,p){b=a.animObject(L(b,this.renderer.globalAnimation,!0));0!==b.duration?(p&&(b.complete=p),B(this,w,b)):(this.attr(w,null,p),b.step&&
b.step.call(this));return this},colorGradient:function(w,b,p){var c=this.renderer,d,q,k,F,x,m,Q,e,G,n,l,g=[],h;w.linearGradient?q="linearGradient":w.radialGradient&&(q="radialGradient");if(q){k=w[q];x=c.gradients;Q=w.stops;n=p.radialReference;u(k)&&(w[q]=k={x1:k[0],y1:k[1],x2:k[2],y2:k[3],gradientUnits:"userSpaceOnUse"});"radialGradient"===q&&n&&!f(k.gradientUnits)&&(F=k,k=C(k,c.getRadialAttr(n,F),{gradientUnits:"userSpaceOnUse"}));for(l in k)"id"!==l&&g.push(l,k[l]);for(l in Q)g.push(Q[l]);g=g.join(",");
x[g]?n=x[g].attr("id"):(k.id=n=a.uniqueKey(),x[g]=m=c.createElement(q).attr(k).add(c.defs),m.radAttr=F,m.stops=[],v(Q,function(w){0===w[1].indexOf("rgba")?(d=a.color(w[1]),e=d.get("rgb"),G=d.get("a")):(e=w[1],G=1);w=c.createElement("stop").attr({offset:w[0],"stop-color":e,"stop-opacity":G}).add(m);m.stops.push(w)}));h="url("+c.url+"#"+n+")";p.setAttribute(b,h);p.gradient=g;w.toString=function(){return h}}},applyTextOutline:function(a){var w=this.element,b,p,c,d;-1!==a.indexOf("contrast")&&(a=a.replace(/contrast/g,
this.renderer.getContrast(w.style.fill)));this.fakeTS=!0;this.ySetter=this.xSetter;b=[].slice.call(w.getElementsByTagName("tspan"));a=a.split(" ");p=a[a.length-1];(c=a[0])&&"none"!==c&&(c=c.replace(/(^[\d\.]+)(.*?)$/g,function(a,w,b){return 2*w+b}),v(b,function(a){"highcharts-text-outline"===a.getAttribute("class")&&m(b,w.removeChild(a))}),d=w.firstChild,v(b,function(a,b){0===b&&(a.setAttribute("x",w.getAttribute("x")),b=w.getAttribute("y"),a.setAttribute("y",b||0),null===b&&w.setAttribute("y",0));
a=a.cloneNode(1);h(a,{"class":"highcharts-text-outline",fill:p,stroke:p,"stroke-width":c,"stroke-linejoin":"round"});w.insertBefore(a,d)}))},attr:function(a,b,p,c){var w,d=this.element,q,k=this,x;"string"===typeof a&&void 0!==b&&(w=a,a={},a[w]=b);if("string"===typeof a)k=(this[a+"Getter"]||this._defaultGetter).call(this,a,d);else{for(w in a)b=a[w],x=!1,c||F(this,w),this.symbolName&&/^(x|y|width|height|r|start|end|innerR|anchorX|anchorY)/.test(w)&&(q||(this.symbolAttr(a),q=!0),x=!0),!this.rotation||
"x"!==w&&"y"!==w||(this.doTransform=!0),x||(x=this[w+"Setter"]||this._defaultSetter,x.call(this,b,w,d),this.shadows&&/^(width|height|visibility|x|y|d|transform|cx|cy|r)$/.test(w)&&this.updateShadows(w,b,x));this.doTransform&&(this.updateTransform(),this.doTransform=!1)}p&&p();return k},updateShadows:function(a,b,p){for(var w=this.shadows,c=w.length;c--;)p.call(w[c],"height"===a?Math.max(b-(w[c].cutHeight||0),0):"d"===a?this.d:b,a,w[c])},addClass:function(a,b){var w=this.attr("class")||"";-1===w.indexOf(a)&&
(b||(a=(w+(w?" ":"")+a).replace("  "," ")),this.attr("class",a));return this},hasClass:function(a){return-1!==h(this.element,"class").indexOf(a)},removeClass:function(a){h(this.element,"class",(h(this.element,"class")||"").replace(a,""));return this},symbolAttr:function(a){var w=this;v("x y r start end width height innerR anchorX anchorY".split(" "),function(b){w[b]=L(a[b],w[b])});w.attr({d:w.renderer.symbols[w.symbolName](w.x,w.y,w.width,w.height,w)})},clip:function(a){return this.attr("clip-path",
a?"url("+this.renderer.url+"#"+a.id+")":"none")},crisp:function(a,b){var w,p={},c;b=b||a.strokeWidth||0;c=Math.round(b)%2/2;a.x=Math.floor(a.x||this.x||0)+c;a.y=Math.floor(a.y||this.y||0)+c;a.width=Math.floor((a.width||this.width||0)-2*c);a.height=Math.floor((a.height||this.height||0)-2*c);f(a.strokeWidth)&&(a.strokeWidth=b);for(w in a)this[w]!==a[w]&&(this[w]=p[w]=a[w]);return p},css:function(a){var w=this.styles,p={},c=this.element,d,q="",k=!w,F=["textOverflow","width"];a&&a.color&&(a.fill=a.color);
if(w)for(d in a)a[d]!==w[d]&&(p[d]=a[d],k=!0);if(k){w&&(a=b(w,p));w=this.textWidth=a&&a.width&&"auto"!==a.width&&"text"===c.nodeName.toLowerCase()&&K(a.width);this.styles=a;w&&!M&&this.renderer.forExport&&delete a.width;if(x&&!M)r(this.element,a);else{w=function(a,w){return"-"+w.toLowerCase()};for(d in a)-1===D(d,F)&&(q+=d.replace(/([A-Z])/g,w)+":"+a[d]+";");q&&h(c,"style",q)}this.added&&("text"===this.element.nodeName&&this.renderer.buildText(this),a&&a.textOutline&&this.applyTextOutline(a.textOutline))}return this},
strokeWidth:function(){return this["stroke-width"]||0},on:function(a,b){var w=this,p=w.element;d&&"click"===a?(p.ontouchstart=function(a){w.touchEventFired=Date.now();a.preventDefault();b.call(p,a)},p.onclick=function(a){(-1===O.navigator.userAgent.indexOf("Android")||1100<Date.now()-(w.touchEventFired||0))&&b.call(p,a)}):p["on"+a]=b;return this},setRadialReference:function(a){var w=this.renderer.gradients[this.element.gradient];this.element.radialReference=a;w&&w.radAttr&&w.animate(this.renderer.getRadialAttr(a,
w.radAttr));return this},translate:function(a,b){return this.attr({translateX:a,translateY:b})},invert:function(a){this.inverted=a;this.updateTransform();return this},updateTransform:function(){var a=this.translateX||0,b=this.translateY||0,p=this.scaleX,c=this.scaleY,d=this.inverted,q=this.rotation,k=this.element;d&&(a+=this.width,b+=this.height);a=["translate("+a+","+b+")"];d?a.push("rotate(90) scale(-1,1)"):q&&a.push("rotate("+q+" "+(k.getAttribute("x")||0)+" "+(k.getAttribute("y")||0)+")");(f(p)||
f(c))&&a.push("scale("+L(p,1)+" "+L(c,1)+")");a.length&&k.setAttribute("transform",a.join(" "))},toFront:function(){var a=this.element;a.parentNode.appendChild(a);return this},align:function(a,b,p){var w,c,d,q,k={};c=this.renderer;d=c.alignedObjects;var F,x;if(a){if(this.alignOptions=a,this.alignByTranslate=b,!p||H(p))this.alignTo=w=p||"renderer",m(d,this),d.push(this),p=null}else a=this.alignOptions,b=this.alignByTranslate,w=this.alignTo;p=L(p,c[w],c);w=a.align;c=a.verticalAlign;d=(p.x||0)+(a.x||
0);q=(p.y||0)+(a.y||0);"right"===w?F=1:"center"===w&&(F=2);F&&(d+=(p.width-(a.width||0))/F);k[b?"translateX":"x"]=Math.round(d);"bottom"===c?x=1:"middle"===c&&(x=2);x&&(q+=(p.height-(a.height||0))/x);k[b?"translateY":"y"]=Math.round(q);this[this.placed?"animate":"attr"](k);this.placed=!0;this.alignAttr=k;return this},getBBox:function(a,p){var w,c=this.renderer,d,q=this.element,k=this.styles,F,x=this.textStr,m,f=c.cache,C=c.cacheKeys,u;p=L(p,this.rotation);d=p*e;F=k&&k.fontSize;void 0!==x&&(u=x.toString(),
-1===u.indexOf("\x3c")&&(u=u.replace(/[0-9]/g,"0")),u+=["",p||0,F,k&&k.width,k&&k.textOverflow].join());u&&!a&&(w=f[u]);if(!w){if(q.namespaceURI===this.SVG_NS||c.forExport){try{(m=this.fakeTS&&function(a){v(q.querySelectorAll(".highcharts-text-outline"),function(w){w.style.display=a})})&&m("none"),w=q.getBBox?b({},q.getBBox()):{width:q.offsetWidth,height:q.offsetHeight},m&&m("")}catch(V){}if(!w||0>w.width)w={width:0,height:0}}else w=this.htmlGetBBox();c.isSVG&&(a=w.width,c=w.height,k&&"11px"===k.fontSize&&
17===Math.round(c)&&(w.height=c=14),p&&(w.width=Math.abs(c*Math.sin(d))+Math.abs(a*Math.cos(d)),w.height=Math.abs(c*Math.cos(d))+Math.abs(a*Math.sin(d))));if(u&&0<w.height){for(;250<C.length;)delete f[C.shift()];f[u]||C.push(u);f[u]=w}}return w},show:function(a){return this.attr({visibility:a?"inherit":"visible"})},hide:function(){return this.attr({visibility:"hidden"})},fadeOut:function(a){var w=this;w.animate({opacity:0},{duration:a||150,complete:function(){w.attr({y:-9999})}})},add:function(a){var w=
this.renderer,b=this.element,p;a&&(this.parentGroup=a);this.parentInverted=a&&a.inverted;void 0!==this.textStr&&w.buildText(this);this.added=!0;if(!a||a.handleZ||this.zIndex)p=this.zIndexSetter();p||(a?a.element:w.box).appendChild(b);if(this.onAdd)this.onAdd();return this},safeRemoveChild:function(a){var w=a.parentNode;w&&w.removeChild(a)},destroy:function(){var a=this.element||{},b=this.renderer.isSVG&&"SPAN"===a.nodeName&&this.parentGroup,p,c;a.onclick=a.onmouseout=a.onmouseover=a.onmousemove=a.point=
null;F(this);this.clipPath&&(this.clipPath=this.clipPath.destroy());if(this.stops){for(c=0;c<this.stops.length;c++)this.stops[c]=this.stops[c].destroy();this.stops=null}this.safeRemoveChild(a);for(this.destroyShadows();b&&b.div&&0===b.div.childNodes.length;)a=b.parentGroup,this.safeRemoveChild(b.div),delete b.div,b=a;this.alignTo&&m(this.renderer.alignedObjects,this);for(p in this)delete this[p];return null},shadow:function(a,b,p){var w=[],c,d,q=this.element,k,F,x,m;if(!a)this.destroyShadows();else if(!this.shadows){F=
L(a.width,3);x=(a.opacity||.15)/F;m=this.parentInverted?"(-1,-1)":"("+L(a.offsetX,1)+", "+L(a.offsetY,1)+")";for(c=1;c<=F;c++)d=q.cloneNode(0),k=2*F+1-2*c,h(d,{isShadow:"true",stroke:a.color||"#000000","stroke-opacity":x*c,"stroke-width":k,transform:"translate"+m,fill:"none"}),p&&(h(d,"height",Math.max(h(d,"height")-k,0)),d.cutHeight=k),b?b.element.appendChild(d):q.parentNode.insertBefore(d,q),w.push(d);this.shadows=w}return this},destroyShadows:function(){v(this.shadows||[],function(a){this.safeRemoveChild(a)},
this);this.shadows=void 0},xGetter:function(a){"circle"===this.element.nodeName&&("x"===a?a="cx":"y"===a&&(a="cy"));return this._defaultGetter(a)},_defaultGetter:function(a){a=L(this[a],this.element?this.element.getAttribute(a):null,0);/^[\-0-9\.]+$/.test(a)&&(a=parseFloat(a));return a},dSetter:function(a,b,p){a&&a.join&&(a=a.join(" "));/(NaN| {2}|^$)/.test(a)&&(a="M 0 0");p.setAttribute(b,a);this[b]=a},dashstyleSetter:function(a){var b,w=this["stroke-width"];"inherit"===w&&(w=1);if(a=a&&a.toLowerCase()){a=
a.replace("shortdashdotdot","3,1,1,1,1,1,").replace("shortdashdot","3,1,1,1").replace("shortdot","1,1,").replace("shortdash","3,1,").replace("longdash","8,3,").replace(/dot/g,"1,3,").replace("dash","4,3,").replace(/,$/,"").split(",");for(b=a.length;b--;)a[b]=K(a[b])*w;a=a.join(",").replace(/NaN/g,"none");this.element.setAttribute("stroke-dasharray",a)}},alignSetter:function(a){this.element.setAttribute("text-anchor",{left:"start",center:"middle",right:"end"}[a])},opacitySetter:function(a,b,p){this[b]=
a;p.setAttribute(b,a)},titleSetter:function(a){var b=this.element.getElementsByTagName("title")[0];b||(b=z.createElementNS(this.SVG_NS,"title"),this.element.appendChild(b));b.firstChild&&b.removeChild(b.firstChild);b.appendChild(z.createTextNode(String(L(a),"").replace(/<[^>]*>/g,"")))},textSetter:function(a){a!==this.textStr&&(delete this.bBox,this.textStr=a,this.added&&this.renderer.buildText(this))},fillSetter:function(a,b,p){"string"===typeof a?p.setAttribute(b,a):a&&this.colorGradient(a,b,p)},
visibilitySetter:function(a,b,p){"inherit"===a?p.removeAttribute(b):p.setAttribute(b,a)},zIndexSetter:function(a,b){var w=this.renderer,p=this.parentGroup,c=(p||w).element||w.box,d,q=this.element,k;d=this.added;var F;f(a)&&(q.zIndex=a,a=+a,this[b]===a&&(d=!1),this[b]=a);if(d){(a=this.zIndex)&&p&&(p.handleZ=!0);b=c.childNodes;for(F=0;F<b.length&&!k;F++)p=b[F],d=p.zIndex,p!==q&&(K(d)>a||!f(a)&&f(d)||0>a&&!f(d)&&c!==w.box)&&(c.insertBefore(q,p),k=!0);k||c.appendChild(q)}return k},_defaultSetter:function(a,
b,p){p.setAttribute(b,a)}};y.prototype.yGetter=y.prototype.xGetter;y.prototype.translateXSetter=y.prototype.translateYSetter=y.prototype.rotationSetter=y.prototype.verticalAlignSetter=y.prototype.scaleXSetter=y.prototype.scaleYSetter=function(a,b){this[b]=a;this.doTransform=!0};y.prototype["stroke-widthSetter"]=y.prototype.strokeSetter=function(a,b,p){this[b]=a;this.stroke&&this["stroke-width"]?(y.prototype.fillSetter.call(this,this.stroke,"stroke",p),p.setAttribute("stroke-width",this["stroke-width"]),
this.hasStroke=!0):"stroke-width"===b&&0===a&&this.hasStroke&&(p.removeAttribute("stroke"),this.hasStroke=!1)};E=a.SVGRenderer=function(){this.init.apply(this,arguments)};E.prototype={Element:y,SVG_NS:N,init:function(a,b,p,c,d,F){var w;c=this.createElement("svg").attr({version:"1.1","class":"highcharts-root"}).css(this.getStyle(c));w=c.element;a.appendChild(w);-1===a.innerHTML.indexOf("xmlns")&&h(w,"xmlns",this.SVG_NS);this.isSVG=!0;this.box=w;this.boxWrapper=c;this.alignedObjects=[];this.url=(q||
k)&&z.getElementsByTagName("base").length?O.location.href.replace(/#.*?$/,"").replace(/<[^>]*>/g,"").replace(/([\('\)])/g,"\\$1").replace(/ /g,"%20"):"";this.createElement("desc").add().element.appendChild(z.createTextNode("Created with Highmaps 5.0.8"));this.defs=this.createElement("defs").add();this.allowHTML=F;this.forExport=d;this.gradients={};this.cache={};this.cacheKeys=[];this.imgCount=0;this.setSize(b,p,!1);var x;q&&a.getBoundingClientRect&&(b=function(){r(a,{left:0,top:0});x=a.getBoundingClientRect();
r(a,{left:Math.ceil(x.left)-x.left+"px",top:Math.ceil(x.top)-x.top+"px"})},b(),this.unSubPixelFix=A(O,"resize",b))},getStyle:function(a){return this.style=b({fontFamily:'"Lucida Grande", "Lucida Sans Unicode", Arial, Helvetica, sans-serif',fontSize:"12px"},a)},setStyle:function(a){this.boxWrapper.css(this.getStyle(a))},isHidden:function(){return!this.boxWrapper.getBBox().width},destroy:function(){var a=this.defs;this.box=null;this.boxWrapper=this.boxWrapper.destroy();n(this.gradients||{});this.gradients=
null;a&&(this.defs=a.destroy());this.unSubPixelFix&&this.unSubPixelFix();return this.alignedObjects=null},createElement:function(a){var b=new this.Element;b.init(this,a);return b},draw:I,getRadialAttr:function(a,b){return{cx:a[0]-a[2]/2+b.cx*a[2],cy:a[1]-a[2]/2+b.cy*a[2],r:b.r*a[2]}},buildText:function(a){var b=a.element,p=this,w=p.forExport,d=L(a.textStr,"").toString(),q=-1!==d.indexOf("\x3c"),k=b.childNodes,F,x,m,f,u=h(b,"x"),C=a.styles,e=a.textWidth,G=C&&C.lineHeight,n=C&&C.textOutline,l=C&&"ellipsis"===
C.textOverflow,g=C&&"nowrap"===C.whiteSpace,D=C&&C.fontSize,H,I=k.length,C=e&&!a.added&&this.box,t=function(a){var c;c=/(px|em)$/.test(a&&a.style.fontSize)?a.style.fontSize:D||p.style.fontSize||12;return G?K(G):p.fontMetrics(c,a.getAttribute("style")?a:b).h};H=[d,l,g,G,n,D,e].join();if(H!==a.textCache){for(a.textCache=H;I--;)b.removeChild(k[I]);q||n||l||e||-1!==d.indexOf(" ")?(F=/<.*class="([^"]+)".*>/,x=/<.*style="([^"]+)".*>/,m=/<.*href="(http[^"]+)".*>/,C&&C.appendChild(b),d=q?d.replace(/<(b|strong)>/g,
'\x3cspan style\x3d"font-weight:bold"\x3e').replace(/<(i|em)>/g,'\x3cspan style\x3d"font-style:italic"\x3e').replace(/<a/g,"\x3cspan").replace(/<\/(b|strong|i|em|a)>/g,"\x3c/span\x3e").split(/<br.*?>/g):[d],d=c(d,function(a){return""!==a}),v(d,function(c,d){var q,k=0;c=c.replace(/^\s+|\s+$/g,"").replace(/<span/g,"|||\x3cspan").replace(/<\/span>/g,"\x3c/span\x3e|||");q=c.split("|||");v(q,function(c){if(""!==c||1===q.length){var C={},G=z.createElementNS(p.SVG_NS,"tspan"),n,v;F.test(c)&&(n=c.match(F)[1],
h(G,"class",n));x.test(c)&&(v=c.match(x)[1].replace(/(;| |^)color([ :])/,"$1fill$2"),h(G,"style",v));m.test(c)&&!w&&(h(G,"onclick",'location.href\x3d"'+c.match(m)[1]+'"'),r(G,{cursor:"pointer"}));c=(c.replace(/<(.|\n)*?>/g,"")||" ").replace(/&lt;/g,"\x3c").replace(/&gt;/g,"\x3e");if(" "!==c){G.appendChild(z.createTextNode(c));k?C.dx=0:d&&null!==u&&(C.x=u);h(G,C);b.appendChild(G);!k&&d&&(!M&&w&&r(G,{display:"block"}),h(G,"dy",t(G)));if(e){C=c.replace(/([^\^])-/g,"$1- ").split(" ");n=1<q.length||d||
1<C.length&&!g;for(var D,H,I=[],Q=t(G),L=a.rotation,K=c,P=K.length;(n||l)&&(C.length||I.length);)a.rotation=0,D=a.getBBox(!0),H=D.width,!M&&p.forExport&&(H=p.measureSpanWidth(G.firstChild.data,a.styles)),D=H>e,void 0===f&&(f=D),l&&f?(P/=2,""===K||!D&&.5>P?C=[]:(K=c.substring(0,K.length+(D?-1:1)*Math.ceil(P)),C=[K+(3<e?"\u2026":"")],G.removeChild(G.firstChild))):D&&1!==C.length?(G.removeChild(G.firstChild),I.unshift(C.pop())):(C=I,I=[],C.length&&!g&&(G=z.createElementNS(N,"tspan"),h(G,{dy:Q,x:u}),
v&&h(G,"style",v),b.appendChild(G)),H>e&&(e=H)),C.length&&G.appendChild(z.createTextNode(C.join(" ").replace(/- /g,"-")));a.rotation=L}k++}}})}),f&&a.attr("title",a.textStr),C&&C.removeChild(b),n&&a.applyTextOutline&&a.applyTextOutline(n)):b.appendChild(z.createTextNode(d.replace(/&lt;/g,"\x3c").replace(/&gt;/g,"\x3e")))}},getContrast:function(a){a=t(a).rgba;return 510<a[0]+a[1]+a[2]?"#000000":"#FFFFFF"},button:function(a,p,c,d,q,k,F,m,f){var w=this.label(a,p,c,f,null,null,null,null,"button"),G=0;
w.attr(C({padding:8,r:2},q));var u,e,n,l;q=C({fill:"#f7f7f7",stroke:"#cccccc","stroke-width":1,style:{color:"#333333",cursor:"pointer",fontWeight:"normal"}},q);u=q.style;delete q.style;k=C(q,{fill:"#e6e6e6"},k);e=k.style;delete k.style;F=C(q,{fill:"#e6ebf5",style:{color:"#000000",fontWeight:"bold"}},F);n=F.style;delete F.style;m=C(q,{style:{color:"#cccccc"}},m);l=m.style;delete m.style;A(w.element,x?"mouseover":"mouseenter",function(){3!==G&&w.setState(1)});A(w.element,x?"mouseout":"mouseleave",function(){3!==
G&&w.setState(G)});w.setState=function(a){1!==a&&(w.state=G=a);w.removeClass(/highcharts-button-(normal|hover|pressed|disabled)/).addClass("highcharts-button-"+["normal","hover","pressed","disabled"][a||0]);w.attr([q,k,F,m][a||0]).css([u,e,n,l][a||0])};w.attr(q).css(b({cursor:"default"},u));return w.on("click",function(a){3!==G&&d.call(w,a)})},crispLine:function(a,b){a[1]===a[4]&&(a[1]=a[4]=Math.round(a[1])-b%2/2);a[2]===a[5]&&(a[2]=a[5]=Math.round(a[2])+b%2/2);return a},path:function(a){var p={fill:"none"};
u(a)?p.d=a:G(a)&&b(p,a);return this.createElement("path").attr(p)},circle:function(a,b,p){a=G(a)?a:{x:a,y:b,r:p};b=this.createElement("circle");b.xSetter=b.ySetter=function(a,b,p){p.setAttribute("c"+b,a)};return b.attr(a)},arc:function(a,b,p,c,d,q){G(a)?(c=a,b=c.y,p=c.r,a=c.x):c={innerR:c,start:d,end:q};a=this.symbol("arc",a,b,p,p,c);a.r=p;return a},rect:function(a,b,p,c,d,q){d=G(a)?a.r:d;var w=this.createElement("rect");a=G(a)?a:void 0===a?{}:{x:a,y:b,width:Math.max(p,0),height:Math.max(c,0)};void 0!==
q&&(a.strokeWidth=q,a=w.crisp(a));a.fill="none";d&&(a.r=d);w.rSetter=function(a,b,p){h(p,{rx:a,ry:a})};return w.attr(a)},setSize:function(a,b,p){var c=this.alignedObjects,d=c.length;this.width=a;this.height=b;for(this.boxWrapper.animate({width:a,height:b},{step:function(){this.attr({viewBox:"0 0 "+this.attr("width")+" "+this.attr("height")})},duration:L(p,!0)?void 0:0});d--;)c[d].align()},g:function(a){var b=this.createElement("g");return a?b.attr({"class":"highcharts-"+a}):b},image:function(a,p,
c,d,q){var w={preserveAspectRatio:"none"};1<arguments.length&&b(w,{x:p,y:c,width:d,height:q});w=this.createElement("image").attr(w);w.element.setAttributeNS?w.element.setAttributeNS("http://www.w3.org/1999/xlink","href",a):w.element.setAttribute("hc-svg-href",a);return w},symbol:function(a,p,c,d,q,k){var w=this,F,x=this.symbols[a],m=f(p)&&x&&this.symbols[a](Math.round(p),Math.round(c),d,q,k),C=/^url\((.*?)\)$/,G,u;x?(F=this.path(m),F.attr("fill","none"),b(F,{symbolName:a,x:p,y:c,width:d,height:q}),
k&&b(F,k)):C.test(a)&&(G=a.match(C)[1],F=this.image(G),F.imgwidth=L(S[G]&&S[G].width,k&&k.width),F.imgheight=L(S[G]&&S[G].height,k&&k.height),u=function(){F.attr({width:F.width,height:F.height})},v(["width","height"],function(a){F[a+"Setter"]=function(a,b){var p={},c=this["img"+b],d="width"===b?"translateX":"translateY";this[b]=a;f(c)&&(this.element&&this.element.setAttribute(b,c),this.alignByTranslate||(p[d]=((this[b]||0)-c)/2,this.attr(p)))}}),f(p)&&F.attr({x:p,y:c}),F.isImg=!0,f(F.imgwidth)&&f(F.imgheight)?
u():(F.attr({width:0,height:0}),l("img",{onload:function(){var a=g[w.chartIndex];0===this.width&&(r(this,{position:"absolute",top:"-999em"}),z.body.appendChild(this));S[G]={width:this.width,height:this.height};F.imgwidth=this.width;F.imgheight=this.height;F.element&&u();this.parentNode&&this.parentNode.removeChild(this);w.imgCount--;if(!w.imgCount&&a&&a.onload)a.onload()},src:G}),this.imgCount++));return F},symbols:{circle:function(a,b,p,c){return this.arc(a+p/2,b+c/2,p/2,c/2,{start:0,end:2*Math.PI,
open:!1})},square:function(a,b,p,c){return["M",a,b,"L",a+p,b,a+p,b+c,a,b+c,"Z"]},triangle:function(a,b,p,c){return["M",a+p/2,b,"L",a+p,b+c,a,b+c,"Z"]},"triangle-down":function(a,b,p,c){return["M",a,b,"L",a+p,b,a+p/2,b+c,"Z"]},diamond:function(a,b,p,c){return["M",a+p/2,b,"L",a+p,b+c/2,a+p/2,b+c,a,b+c/2,"Z"]},arc:function(a,b,p,c,d){var q=d.start,k=d.r||p,F=d.r||c||p,w=d.end-.001;p=d.innerR;c=d.open;var x=Math.cos(q),m=Math.sin(q),C=Math.cos(w),w=Math.sin(w);d=d.end-q<Math.PI?0:1;k=["M",a+k*x,b+F*m,
"A",k,F,0,d,1,a+k*C,b+F*w];f(p)&&k.push(c?"M":"L",a+p*C,b+p*w,"A",p,p,0,d,0,a+p*x,b+p*m);k.push(c?"":"Z");return k},callout:function(a,b,p,c,d){var q=Math.min(d&&d.r||0,p,c),k=q+6,F=d&&d.anchorX;d=d&&d.anchorY;var w;w=["M",a+q,b,"L",a+p-q,b,"C",a+p,b,a+p,b,a+p,b+q,"L",a+p,b+c-q,"C",a+p,b+c,a+p,b+c,a+p-q,b+c,"L",a+q,b+c,"C",a,b+c,a,b+c,a,b+c-q,"L",a,b+q,"C",a,b,a,b,a+q,b];F&&F>p?d>b+k&&d<b+c-k?w.splice(13,3,"L",a+p,d-6,a+p+6,d,a+p,d+6,a+p,b+c-q):w.splice(13,3,"L",a+p,c/2,F,d,a+p,c/2,a+p,b+c-q):F&&
0>F?d>b+k&&d<b+c-k?w.splice(33,3,"L",a,d+6,a-6,d,a,d-6,a,b+q):w.splice(33,3,"L",a,c/2,F,d,a,c/2,a,b+q):d&&d>c&&F>a+k&&F<a+p-k?w.splice(23,3,"L",F+6,b+c,F,b+c+6,F-6,b+c,a+q,b+c):d&&0>d&&F>a+k&&F<a+p-k&&w.splice(3,3,"L",F-6,b,F,b-6,F+6,b,p-q,b);return w}},clipRect:function(b,p,c,d){var q=a.uniqueKey(),k=this.createElement("clipPath").attr({id:q}).add(this.defs);b=this.rect(b,p,c,d,0).add(k);b.id=q;b.clipPath=k;b.count=0;return b},text:function(a,b,p,c){var d=!M&&this.forExport,q={};if(c&&(this.allowHTML||
!this.forExport))return this.html(a,b,p);q.x=Math.round(b||0);p&&(q.y=Math.round(p));if(a||0===a)q.text=a;a=this.createElement("text").attr(q);d&&a.css({position:"absolute"});c||(a.xSetter=function(a,b,p){var c=p.getElementsByTagName("tspan"),d,q=p.getAttribute(b),k;for(k=0;k<c.length;k++)d=c[k],d.getAttribute(b)===q&&d.setAttribute(b,a);p.setAttribute(b,a)});return a},fontMetrics:function(a,b){a=a||b&&b.style&&b.style.fontSize||this.style&&this.style.fontSize;a=/px/.test(a)?K(a):/em/.test(a)?parseFloat(a)*
(b?this.fontMetrics(null,b.parentNode).f:16):12;b=24>a?a+3:Math.round(1.2*a);return{h:b,b:Math.round(.8*b),f:a}},rotCorr:function(a,b,p){var c=a;b&&p&&(c=Math.max(c*Math.cos(b*e),4));return{x:-a/3*Math.sin(b*e),y:c}},label:function(c,d,q,k,F,x,m,G,u){var w=this,e=w.g("button"!==u&&"label"),n=e.text=w.text("",0,0,m).attr({zIndex:1}),l,g,D=0,h=3,H=0,z,I,M,N,r,K={},L,t,S=/^url\((.*?)\)$/.test(k),Q=S,P,R,O,U;u&&e.addClass("highcharts-"+u);Q=S;P=function(){return(L||0)%2/2};R=function(){var a=n.element.style,
p={};g=(void 0===z||void 0===I||r)&&f(n.textStr)&&n.getBBox();e.width=(z||g.width||0)+2*h+H;e.height=(I||g.height||0)+2*h;t=h+w.fontMetrics(a&&a.fontSize,n).b;Q&&(l||(e.box=l=w.symbols[k]||S?w.symbol(k):w.rect(),l.addClass(("button"===u?"":"highcharts-label-box")+(u?" highcharts-"+u+"-box":"")),l.add(e),a=P(),p.x=a,p.y=(G?-t:0)+a),p.width=Math.round(e.width),p.height=Math.round(e.height),l.attr(b(p,K)),K={})};O=function(){var a=H+h,b;b=G?0:t;f(z)&&g&&("center"===r||"right"===r)&&(a+={center:.5,right:1}[r]*
(z-g.width));if(a!==n.x||b!==n.y)n.attr("x",a),void 0!==b&&n.attr("y",b);n.x=a;n.y=b};U=function(a,b){l?l.attr(a,b):K[a]=b};e.onAdd=function(){n.add(e);e.attr({text:c||0===c?c:"",x:d,y:q});l&&f(F)&&e.attr({anchorX:F,anchorY:x})};e.widthSetter=function(b){z=a.isNumber(b)?b:null};e.heightSetter=function(a){I=a};e["text-alignSetter"]=function(a){r=a};e.paddingSetter=function(a){f(a)&&a!==h&&(h=e.padding=a,O())};e.paddingLeftSetter=function(a){f(a)&&a!==H&&(H=a,O())};e.alignSetter=function(a){a={left:0,
center:.5,right:1}[a];a!==D&&(D=a,g&&e.attr({x:M}))};e.textSetter=function(a){void 0!==a&&n.textSetter(a);R();O()};e["stroke-widthSetter"]=function(a,b){a&&(Q=!0);L=this["stroke-width"]=a;U(b,a)};e.strokeSetter=e.fillSetter=e.rSetter=function(a,b){"fill"===b&&a&&(Q=!0);U(b,a)};e.anchorXSetter=function(a,b){F=a;U(b,Math.round(a)-P()-M)};e.anchorYSetter=function(a,b){x=a;U(b,a-N)};e.xSetter=function(a){e.x=a;D&&(a-=D*((z||g.width)+2*h));M=Math.round(a);e.attr("translateX",M)};e.ySetter=function(a){N=
e.y=Math.round(a);e.attr("translateY",N)};var B=e.css;return b(e,{css:function(a){if(a){var b={};a=C(a);v(e.textProps,function(p){void 0!==a[p]&&(b[p]=a[p],delete a[p])});n.css(b)}return B.call(e,a)},getBBox:function(){return{width:g.width+2*h,height:g.height+2*h,x:g.x-h,y:g.y-h}},shadow:function(a){a&&(R(),l&&l.shadow(a));return e},destroy:function(){p(e.element,"mouseenter");p(e.element,"mouseleave");n&&(n=n.destroy());l&&(l=l.destroy());y.prototype.destroy.call(e);e=w=R=O=U=null}})}};a.Renderer=
E})(J);(function(a){var y=a.attr,E=a.createElement,A=a.css,B=a.defined,h=a.each,g=a.extend,t=a.isFirefox,r=a.isMS,l=a.isWebKit,f=a.pInt,e=a.SVGRenderer,n=a.win,z=a.wrap;g(a.SVGElement.prototype,{htmlCss:function(a){var b=this.element;if(b=a&&"SPAN"===b.tagName&&a.width)delete a.width,this.textWidth=b,this.updateTransform();a&&"ellipsis"===a.textOverflow&&(a.whiteSpace="nowrap",a.overflow="hidden");this.styles=g(this.styles,a);A(this.element,a);return this},htmlGetBBox:function(){var a=this.element;
"text"===a.nodeName&&(a.style.position="absolute");return{x:a.offsetLeft,y:a.offsetTop,width:a.offsetWidth,height:a.offsetHeight}},htmlUpdateTransform:function(){if(this.added){var a=this.renderer,b=this.element,m=this.translateX||0,c=this.translateY||0,d=this.x||0,e=this.y||0,u=this.textAlign||"left",q={left:0,center:.5,right:1}[u],x=this.styles;A(b,{marginLeft:m,marginTop:c});this.shadows&&h(this.shadows,function(a){A(a,{marginLeft:m+1,marginTop:c+1})});this.inverted&&h(b.childNodes,function(c){a.invertChild(c,
b)});if("SPAN"===b.tagName){var G=this.rotation,n=f(this.textWidth),k=x&&x.whiteSpace,C=[G,u,b.innerHTML,this.textWidth,this.textAlign].join();C!==this.cTT&&(x=a.fontMetrics(b.style.fontSize).b,B(G)&&this.setSpanRotation(G,q,x),A(b,{width:"",whiteSpace:k||"nowrap"}),b.offsetWidth>n&&/[ \-]/.test(b.textContent||b.innerText)&&A(b,{width:n+"px",display:"block",whiteSpace:k||"normal"}),this.getSpanCorrection(b.offsetWidth,x,q,G,u));A(b,{left:d+(this.xCorr||0)+"px",top:e+(this.yCorr||0)+"px"});l&&(x=b.offsetHeight);
this.cTT=C}}else this.alignOnAdd=!0},setSpanRotation:function(a,b,m){var c={},d=r?"-ms-transform":l?"-webkit-transform":t?"MozTransform":n.opera?"-o-transform":"";c[d]=c.transform="rotate("+a+"deg)";c[d+(t?"Origin":"-origin")]=c.transformOrigin=100*b+"% "+m+"px";A(this.element,c)},getSpanCorrection:function(a,b,m){this.xCorr=-a*m;this.yCorr=-b}});g(e.prototype,{html:function(a,b,m){var c=this.createElement("span"),d=c.element,e=c.renderer,f=e.isSVG,q=function(a,b){h(["opacity","visibility"],function(c){z(a,
c+"Setter",function(a,c,d,q){a.call(this,c,d,q);b[d]=c})})};c.textSetter=function(a){a!==d.innerHTML&&delete this.bBox;d.innerHTML=this.textStr=a;c.htmlUpdateTransform()};f&&q(c,c.element.style);c.xSetter=c.ySetter=c.alignSetter=c.rotationSetter=function(a,b){"align"===b&&(b="textAlign");c[b]=a;c.htmlUpdateTransform()};c.attr({text:a,x:Math.round(b),y:Math.round(m)}).css({fontFamily:this.style.fontFamily,fontSize:this.style.fontSize,position:"absolute"});d.style.whiteSpace="nowrap";c.css=c.htmlCss;
f&&(c.add=function(a){var b,m=e.box.parentNode,k=[];if(this.parentGroup=a){if(b=a.div,!b){for(;a;)k.push(a),a=a.parentGroup;h(k.reverse(),function(a){var d,x=y(a.element,"class");x&&(x={className:x});b=a.div=a.div||E("div",x,{position:"absolute",left:(a.translateX||0)+"px",top:(a.translateY||0)+"px",display:a.display,opacity:a.opacity,pointerEvents:a.styles&&a.styles.pointerEvents},b||m);d=b.style;g(a,{on:function(){c.on.apply({element:k[0].div},arguments);return a},translateXSetter:function(b,p){d.left=
b+"px";a[p]=b;a.doTransform=!0},translateYSetter:function(b,p){d.top=b+"px";a[p]=b;a.doTransform=!0}});q(a,d)})}}else b=m;b.appendChild(d);c.added=!0;c.alignOnAdd&&c.htmlUpdateTransform();return c});return c}})})(J);(function(a){var y,E,A=a.createElement,B=a.css,h=a.defined,g=a.deg2rad,t=a.discardElement,r=a.doc,l=a.each,f=a.erase,e=a.extend;y=a.extendClass;var n=a.isArray,z=a.isNumber,v=a.isObject,b=a.merge;E=a.noop;var m=a.pick,c=a.pInt,d=a.SVGElement,D=a.SVGRenderer,u=a.win;a.svg||(E={docMode8:r&&
8===r.documentMode,init:function(a,b){var c=["\x3c",b,' filled\x3d"f" stroked\x3d"f"'],d=["position: ","absolute",";"],q="div"===b;("shape"===b||q)&&d.push("left:0;top:0;width:1px;height:1px;");d.push("visibility: ",q?"hidden":"visible");c.push(' style\x3d"',d.join(""),'"/\x3e');b&&(c=q||"span"===b||"img"===b?c.join(""):a.prepVML(c),this.element=A(c));this.renderer=a},add:function(a){var b=this.renderer,c=this.element,d=b.box,q=a&&a.inverted,d=a?a.element||a:d;a&&(this.parentGroup=a);q&&b.invertChild(c,
d);d.appendChild(c);this.added=!0;this.alignOnAdd&&!this.deferUpdateTransform&&this.updateTransform();if(this.onAdd)this.onAdd();this.className&&this.attr("class",this.className);return this},updateTransform:d.prototype.htmlUpdateTransform,setSpanRotation:function(){var a=this.rotation,b=Math.cos(a*g),c=Math.sin(a*g);B(this.element,{filter:a?["progid:DXImageTransform.Microsoft.Matrix(M11\x3d",b,", M12\x3d",-c,", M21\x3d",c,", M22\x3d",b,", sizingMethod\x3d'auto expand')"].join(""):"none"})},getSpanCorrection:function(a,
b,c,d,k){var q=d?Math.cos(d*g):1,x=d?Math.sin(d*g):0,e=m(this.elemHeight,this.element.offsetHeight),f;this.xCorr=0>q&&-a;this.yCorr=0>x&&-e;f=0>q*x;this.xCorr+=x*b*(f?1-c:c);this.yCorr-=q*b*(d?f?c:1-c:1);k&&"left"!==k&&(this.xCorr-=a*c*(0>q?-1:1),d&&(this.yCorr-=e*c*(0>x?-1:1)),B(this.element,{textAlign:k}))},pathToVML:function(a){for(var b=a.length,c=[];b--;)z(a[b])?c[b]=Math.round(10*a[b])-5:"Z"===a[b]?c[b]="x":(c[b]=a[b],!a.isArc||"wa"!==a[b]&&"at"!==a[b]||(c[b+5]===c[b+7]&&(c[b+7]+=a[b+7]>a[b+
5]?1:-1),c[b+6]===c[b+8]&&(c[b+8]+=a[b+8]>a[b+6]?1:-1)));return c.join(" ")||"x"},clip:function(a){var b=this,c;a?(c=a.members,f(c,b),c.push(b),b.destroyClip=function(){f(c,b)},a=a.getCSS(b)):(b.destroyClip&&b.destroyClip(),a={clip:b.docMode8?"inherit":"rect(auto)"});return b.css(a)},css:d.prototype.htmlCss,safeRemoveChild:function(a){a.parentNode&&t(a)},destroy:function(){this.destroyClip&&this.destroyClip();return d.prototype.destroy.apply(this)},on:function(a,b){this.element["on"+a]=function(){var a=
u.event;a.target=a.srcElement;b(a)};return this},cutOffPath:function(a,b){var d;a=a.split(/[ ,]/);d=a.length;if(9===d||11===d)a[d-4]=a[d-2]=c(a[d-2])-10*b;return a.join(" ")},shadow:function(a,b,d){var q=[],k,e=this.element,x=this.renderer,f,u=e.style,p,F=e.path,n,l,g,G;F&&"string"!==typeof F.value&&(F="x");l=F;if(a){g=m(a.width,3);G=(a.opacity||.15)/g;for(k=1;3>=k;k++)n=2*g+1-2*k,d&&(l=this.cutOffPath(F.value,n+.5)),p=['\x3cshape isShadow\x3d"true" strokeweight\x3d"',n,'" filled\x3d"false" path\x3d"',
l,'" coordsize\x3d"10 10" style\x3d"',e.style.cssText,'" /\x3e'],f=A(x.prepVML(p),null,{left:c(u.left)+m(a.offsetX,1),top:c(u.top)+m(a.offsetY,1)}),d&&(f.cutOff=n+1),p=['\x3cstroke color\x3d"',a.color||"#000000",'" opacity\x3d"',G*k,'"/\x3e'],A(x.prepVML(p),null,null,f),b?b.element.appendChild(f):e.parentNode.insertBefore(f,e),q.push(f);this.shadows=q}return this},updateShadows:E,setAttr:function(a,b){this.docMode8?this.element[a]=b:this.element.setAttribute(a,b)},classSetter:function(a){(this.added?
this.element:this).className=a},dashstyleSetter:function(a,b,c){(c.getElementsByTagName("stroke")[0]||A(this.renderer.prepVML(["\x3cstroke/\x3e"]),null,null,c))[b]=a||"solid";this[b]=a},dSetter:function(a,b,c){var d=this.shadows;a=a||[];this.d=a.join&&a.join(" ");c.path=a=this.pathToVML(a);if(d)for(c=d.length;c--;)d[c].path=d[c].cutOff?this.cutOffPath(a,d[c].cutOff):a;this.setAttr(b,a)},fillSetter:function(a,b,c){var d=c.nodeName;"SPAN"===d?c.style.color=a:"IMG"!==d&&(c.filled="none"!==a,this.setAttr("fillcolor",
this.renderer.color(a,c,b,this)))},"fill-opacitySetter":function(a,b,c){A(this.renderer.prepVML(["\x3c",b.split("-")[0],' opacity\x3d"',a,'"/\x3e']),null,null,c)},opacitySetter:E,rotationSetter:function(a,b,c){c=c.style;this[b]=c[b]=a;c.left=-Math.round(Math.sin(a*g)+1)+"px";c.top=Math.round(Math.cos(a*g))+"px"},strokeSetter:function(a,b,c){this.setAttr("strokecolor",this.renderer.color(a,c,b,this))},"stroke-widthSetter":function(a,b,c){c.stroked=!!a;this[b]=a;z(a)&&(a+="px");this.setAttr("strokeweight",
a)},titleSetter:function(a,b){this.setAttr(b,a)},visibilitySetter:function(a,b,c){"inherit"===a&&(a="visible");this.shadows&&l(this.shadows,function(c){c.style[b]=a});"DIV"===c.nodeName&&(a="hidden"===a?"-999em":0,this.docMode8||(c.style[b]=a?"visible":"hidden"),b="top");c.style[b]=a},xSetter:function(a,b,c){this[b]=a;"x"===b?b="left":"y"===b&&(b="top");this.updateClipping?(this[b]=a,this.updateClipping()):c.style[b]=a},zIndexSetter:function(a,b,c){c.style[b]=a}},E["stroke-opacitySetter"]=E["fill-opacitySetter"],
a.VMLElement=E=y(d,E),E.prototype.ySetter=E.prototype.widthSetter=E.prototype.heightSetter=E.prototype.xSetter,E={Element:E,isIE8:-1<u.navigator.userAgent.indexOf("MSIE 8.0"),init:function(a,b,c){var d,k;this.alignedObjects=[];d=this.createElement("div").css({position:"relative"});k=d.element;a.appendChild(d.element);this.isVML=!0;this.box=k;this.boxWrapper=d;this.gradients={};this.cache={};this.cacheKeys=[];this.imgCount=0;this.setSize(b,c,!1);if(!r.namespaces.hcv){r.namespaces.add("hcv","urn:schemas-microsoft-com:vml");
try{r.createStyleSheet().cssText="hcv\\:fill, hcv\\:path, hcv\\:shape, hcv\\:stroke{ behavior:url(#default#VML); display: inline-block; } "}catch(C){r.styleSheets[0].cssText+="hcv\\:fill, hcv\\:path, hcv\\:shape, hcv\\:stroke{ behavior:url(#default#VML); display: inline-block; } "}}},isHidden:function(){return!this.box.offsetWidth},clipRect:function(a,b,c,d){var k=this.createElement(),q=v(a);return e(k,{members:[],count:0,left:(q?a.x:a)+1,top:(q?a.y:b)+1,width:(q?a.width:c)-1,height:(q?a.height:d)-
1,getCSS:function(a){var b=a.element,c=b.nodeName,p=a.inverted,d=this.top-("shape"===c?b.offsetTop:0),k=this.left,b=k+this.width,q=d+this.height,d={clip:"rect("+Math.round(p?k:d)+"px,"+Math.round(p?q:b)+"px,"+Math.round(p?b:q)+"px,"+Math.round(p?d:k)+"px)"};!p&&a.docMode8&&"DIV"===c&&e(d,{width:b+"px",height:q+"px"});return d},updateClipping:function(){l(k.members,function(a){a.element&&a.css(k.getCSS(a))})}})},color:function(b,c,d,m){var k=this,q,e=/^rgba/,f,u,p="none";b&&b.linearGradient?u="gradient":
b&&b.radialGradient&&(u="pattern");if(u){var F,n,x=b.linearGradient||b.radialGradient,g,h,w,D,z,G="";b=b.stops;var v,H=[],r=function(){f=['\x3cfill colors\x3d"'+H.join(",")+'" opacity\x3d"',w,'" o:opacity2\x3d"',h,'" type\x3d"',u,'" ',G,'focus\x3d"100%" method\x3d"any" /\x3e'];A(k.prepVML(f),null,null,c)};g=b[0];v=b[b.length-1];0<g[0]&&b.unshift([0,g[1]]);1>v[0]&&b.push([1,v[1]]);l(b,function(b,c){e.test(b[1])?(q=a.color(b[1]),F=q.get("rgb"),n=q.get("a")):(F=b[1],n=1);H.push(100*b[0]+"% "+F);c?(w=
n,D=F):(h=n,z=F)});if("fill"===d)if("gradient"===u)d=x.x1||x[0]||0,b=x.y1||x[1]||0,g=x.x2||x[2]||0,x=x.y2||x[3]||0,G='angle\x3d"'+(90-180*Math.atan((x-b)/(g-d))/Math.PI)+'"',r();else{var p=x.r,t=2*p,B=2*p,y=x.cx,E=x.cy,J=c.radialReference,T,p=function(){J&&(T=m.getBBox(),y+=(J[0]-T.x)/T.width-.5,E+=(J[1]-T.y)/T.height-.5,t*=J[2]/T.width,B*=J[2]/T.height);G='src\x3d"'+a.getOptions().global.VMLRadialGradientURL+'" size\x3d"'+t+","+B+'" origin\x3d"0.5,0.5" position\x3d"'+y+","+E+'" color2\x3d"'+z+'" ';
r()};m.added?p():m.onAdd=p;p=D}else p=F}else e.test(b)&&"IMG"!==c.tagName?(q=a.color(b),m[d+"-opacitySetter"](q.get("a"),d,c),p=q.get("rgb")):(p=c.getElementsByTagName(d),p.length&&(p[0].opacity=1,p[0].type="solid"),p=b);return p},prepVML:function(a){var b=this.isIE8;a=a.join("");b?(a=a.replace("/\x3e",' xmlns\x3d"urn:schemas-microsoft-com:vml" /\x3e'),a=-1===a.indexOf('style\x3d"')?a.replace("/\x3e",' style\x3d"display:inline-block;behavior:url(#default#VML);" /\x3e'):a.replace('style\x3d"','style\x3d"display:inline-block;behavior:url(#default#VML);')):
a=a.replace("\x3c","\x3chcv:");return a},text:D.prototype.html,path:function(a){var b={coordsize:"10 10"};n(a)?b.d=a:v(a)&&e(b,a);return this.createElement("shape").attr(b)},circle:function(a,b,c){var d=this.symbol("circle");v(a)&&(c=a.r,b=a.y,a=a.x);d.isCircle=!0;d.r=c;return d.attr({x:a,y:b})},g:function(a){var b;a&&(b={className:"highcharts-"+a,"class":"highcharts-"+a});return this.createElement("div").attr(b)},image:function(a,b,c,d,k){var m=this.createElement("img").attr({src:a});1<arguments.length&&
m.attr({x:b,y:c,width:d,height:k});return m},createElement:function(a){return"rect"===a?this.symbol(a):D.prototype.createElement.call(this,a)},invertChild:function(a,b){var d=this;b=b.style;var m="IMG"===a.tagName&&a.style;B(a,{flip:"x",left:c(b.width)-(m?c(m.top):1),top:c(b.height)-(m?c(m.left):1),rotation:-90});l(a.childNodes,function(b){d.invertChild(b,a)})},symbols:{arc:function(a,b,c,d,k){var m=k.start,e=k.end,q=k.r||c||d;c=k.innerR;d=Math.cos(m);var f=Math.sin(m),p=Math.cos(e),F=Math.sin(e);
if(0===e-m)return["x"];m=["wa",a-q,b-q,a+q,b+q,a+q*d,b+q*f,a+q*p,b+q*F];k.open&&!c&&m.push("e","M",a,b);m.push("at",a-c,b-c,a+c,b+c,a+c*p,b+c*F,a+c*d,b+c*f,"x","e");m.isArc=!0;return m},circle:function(a,b,c,d,k){k&&h(k.r)&&(c=d=2*k.r);k&&k.isCircle&&(a-=c/2,b-=d/2);return["wa",a,b,a+c,b+d,a+c,b+d/2,a+c,b+d/2,"e"]},rect:function(a,b,c,d,k){return D.prototype.symbols[h(k)&&k.r?"callout":"square"].call(0,a,b,c,d,k)}}},a.VMLRenderer=y=function(){this.init.apply(this,arguments)},y.prototype=b(D.prototype,
E),a.Renderer=y);D.prototype.measureSpanWidth=function(a,b){var c=r.createElement("span");a=r.createTextNode(a);c.appendChild(a);B(c,b);this.box.appendChild(c);b=c.offsetWidth;t(c);return b}})(J);(function(a){var y=a.correctFloat,E=a.defined,A=a.destroyObjectProperties,B=a.isNumber,h=a.merge,g=a.pick,t=a.deg2rad;a.Tick=function(a,l,f,e){this.axis=a;this.pos=l;this.type=f||"";this.isNew=!0;f||e||this.addLabel()};a.Tick.prototype={addLabel:function(){var a=this.axis,l=a.options,f=a.chart,e=a.categories,
n=a.names,z=this.pos,v=l.labels,b=a.tickPositions,m=z===b[0],c=z===b[b.length-1],n=e?g(e[z],n[z],z):z,e=this.label,b=b.info,d;a.isDatetimeAxis&&b&&(d=l.dateTimeLabelFormats[b.higherRanks[z]||b.unitName]);this.isFirst=m;this.isLast=c;l=a.labelFormatter.call({axis:a,chart:f,isFirst:m,isLast:c,dateTimeLabelFormat:d,value:a.isLog?y(a.lin2log(n)):n});E(e)?e&&e.attr({text:l}):(this.labelLength=(this.label=e=E(l)&&v.enabled?f.renderer.text(l,0,0,v.useHTML).css(h(v.style)).add(a.labelGroup):null)&&e.getBBox().width,
this.rotation=0)},getLabelSize:function(){return this.label?this.label.getBBox()[this.axis.horiz?"height":"width"]:0},handleOverflow:function(a){var l=this.axis,f=a.x,e=l.chart.chartWidth,n=l.chart.spacing,h=g(l.labelLeft,Math.min(l.pos,n[3])),n=g(l.labelRight,Math.max(l.pos+l.len,e-n[1])),v=this.label,b=this.rotation,m={left:0,center:.5,right:1}[l.labelAlign],c=v.getBBox().width,d=l.getSlotWidth(),D=d,u=1,q,x={};if(b)0>b&&f-m*c<h?q=Math.round(f/Math.cos(b*t)-h):0<b&&f+m*c>n&&(q=Math.round((e-f)/
Math.cos(b*t)));else if(e=f+(1-m)*c,f-m*c<h?D=a.x+D*(1-m)-h:e>n&&(D=n-a.x+D*m,u=-1),D=Math.min(d,D),D<d&&"center"===l.labelAlign&&(a.x+=u*(d-D-m*(d-Math.min(c,D)))),c>D||l.autoRotation&&(v.styles||{}).width)q=D;q&&(x.width=q,(l.options.labels.style||{}).textOverflow||(x.textOverflow="ellipsis"),v.css(x))},getPosition:function(a,l,f,e){var n=this.axis,g=n.chart,h=e&&g.oldChartHeight||g.chartHeight;return{x:a?n.translate(l+f,null,null,e)+n.transB:n.left+n.offset+(n.opposite?(e&&g.oldChartWidth||g.chartWidth)-
n.right-n.left:0),y:a?h-n.bottom+n.offset-(n.opposite?n.height:0):h-n.translate(l+f,null,null,e)-n.transB}},getLabelPosition:function(a,l,f,e,n,g,h,b){var m=this.axis,c=m.transA,d=m.reversed,D=m.staggerLines,u=m.tickRotCorr||{x:0,y:0},q=n.y;E(q)||(q=0===m.side?f.rotation?-8:-f.getBBox().height:2===m.side?u.y+8:Math.cos(f.rotation*t)*(u.y-f.getBBox(!1,0).height/2));a=a+n.x+u.x-(g&&e?g*c*(d?-1:1):0);l=l+q-(g&&!e?g*c*(d?1:-1):0);D&&(f=h/(b||1)%D,m.opposite&&(f=D-f-1),l+=m.labelOffset/D*f);return{x:a,
y:Math.round(l)}},getMarkPath:function(a,l,f,e,n,g){return g.crispLine(["M",a,l,"L",a+(n?0:-f),l+(n?f:0)],e)},renderGridLine:function(a,l,f){var e=this.axis,n=e.options,g=this.gridLine,h={},b=this.pos,m=this.type,c=e.tickmarkOffset,d=e.chart.renderer,D=m?m+"Grid":"grid",u=n[D+"LineWidth"],q=n[D+"LineColor"],n=n[D+"LineDashStyle"];g||(h.stroke=q,h["stroke-width"]=u,n&&(h.dashstyle=n),m||(h.zIndex=1),a&&(h.opacity=0),this.gridLine=g=d.path().attr(h).addClass("highcharts-"+(m?m+"-":"")+"grid-line").add(e.gridGroup));
if(!a&&g&&(a=e.getPlotLinePath(b+c,g.strokeWidth()*f,a,!0)))g[this.isNew?"attr":"animate"]({d:a,opacity:l})},renderMark:function(a,l,f){var e=this.axis,n=e.options,h=e.chart.renderer,v=this.type,b=v?v+"Tick":"tick",m=e.tickSize(b),c=this.mark,d=!c,D=a.x;a=a.y;var u=g(n[b+"Width"],!v&&e.isXAxis?1:0),n=n[b+"Color"];m&&(e.opposite&&(m[0]=-m[0]),d&&(this.mark=c=h.path().addClass("highcharts-"+(v?v+"-":"")+"tick").add(e.axisGroup),c.attr({stroke:n,"stroke-width":u})),c[d?"attr":"animate"]({d:this.getMarkPath(D,
a,m[0],c.strokeWidth()*f,e.horiz,h),opacity:l}))},renderLabel:function(a,l,f,e){var n=this.axis,h=n.horiz,v=n.options,b=this.label,m=v.labels,c=m.step,d=n.tickmarkOffset,D=!0,u=a.x;a=a.y;b&&B(u)&&(b.xy=a=this.getLabelPosition(u,a,b,h,m,d,e,c),this.isFirst&&!this.isLast&&!g(v.showFirstLabel,1)||this.isLast&&!this.isFirst&&!g(v.showLastLabel,1)?D=!1:!h||n.isRadial||m.step||m.rotation||l||0===f||this.handleOverflow(a),c&&e%c&&(D=!1),D&&B(a.y)?(a.opacity=f,b[this.isNew?"attr":"animate"](a)):b.attr("y",
-9999),this.isNew=!1)},render:function(a,l,f){var e=this.axis,n=e.horiz,h=this.getPosition(n,this.pos,e.tickmarkOffset,l),v=h.x,b=h.y,e=n&&v===e.pos+e.len||!n&&b===e.pos?-1:1;f=g(f,1);this.isActive=!0;this.renderGridLine(l,f,e);this.renderMark(h,f,e);this.renderLabel(h,l,f,a)},destroy:function(){A(this,this.axis)}}})(J);(function(a){var y=a.arrayMax,E=a.arrayMin,A=a.defined,B=a.destroyObjectProperties,h=a.each,g=a.erase,t=a.merge,r=a.pick;a.PlotLineOrBand=function(a,f){this.axis=a;f&&(this.options=
f,this.id=f.id)};a.PlotLineOrBand.prototype={render:function(){var a=this,f=a.axis,e=f.horiz,n=a.options,g=n.label,h=a.label,b=n.to,m=n.from,c=n.value,d=A(m)&&A(b),D=A(c),u=a.svgElem,q=!u,x=[],G,H=n.color,k=r(n.zIndex,0),C=n.events,x={"class":"highcharts-plot-"+(d?"band ":"line ")+(n.className||"")},I={},L=f.chart.renderer,K=d?"bands":"lines",p=f.log2lin;f.isLog&&(m=p(m),b=p(b),c=p(c));D?(x={stroke:H,"stroke-width":n.width},n.dashStyle&&(x.dashstyle=n.dashStyle)):d&&(H&&(x.fill=H),n.borderWidth&&
(x.stroke=n.borderColor,x["stroke-width"]=n.borderWidth));I.zIndex=k;K+="-"+k;(H=f[K])||(f[K]=H=L.g("plot-"+K).attr(I).add());q&&(a.svgElem=u=L.path().attr(x).add(H));if(D)x=f.getPlotLinePath(c,u.strokeWidth());else if(d)x=f.getPlotBandPath(m,b,n);else return;if(q&&x&&x.length){if(u.attr({d:x}),C)for(G in n=function(b){u.on(b,function(c){C[b].apply(a,[c])})},C)n(G)}else u&&(x?(u.show(),u.animate({d:x})):(u.hide(),h&&(a.label=h=h.destroy())));g&&A(g.text)&&x&&x.length&&0<f.width&&0<f.height&&!x.flat?
(g=t({align:e&&d&&"center",x:e?!d&&4:10,verticalAlign:!e&&d&&"middle",y:e?d?16:10:d?6:-4,rotation:e&&!d&&90},g),this.renderLabel(g,x,d,k)):h&&h.hide();return a},renderLabel:function(a,f,e,n){var g=this.label,h=this.axis.chart.renderer;g||(g={align:a.textAlign||a.align,rotation:a.rotation,"class":"highcharts-plot-"+(e?"band":"line")+"-label "+(a.className||"")},g.zIndex=n,this.label=g=h.text(a.text,0,0,a.useHTML).attr(g).add(),g.css(a.style));n=[f[1],f[4],e?f[6]:f[1]];f=[f[2],f[5],e?f[7]:f[2]];e=E(n);
h=E(f);g.align(a,!1,{x:e,y:h,width:y(n)-e,height:y(f)-h});g.show()},destroy:function(){g(this.axis.plotLinesAndBands,this);delete this.axis;B(this)}};a.AxisPlotLineOrBandExtension={getPlotBandPath:function(a,f){f=this.getPlotLinePath(f,null,null,!0);(a=this.getPlotLinePath(a,null,null,!0))&&f?(a.flat=a.toString()===f.toString(),a.push(f[4],f[5],f[1],f[2],"z")):a=null;return a},addPlotBand:function(a){return this.addPlotBandOrLine(a,"plotBands")},addPlotLine:function(a){return this.addPlotBandOrLine(a,
"plotLines")},addPlotBandOrLine:function(g,f){var e=(new a.PlotLineOrBand(this,g)).render(),n=this.userOptions;e&&(f&&(n[f]=n[f]||[],n[f].push(g)),this.plotLinesAndBands.push(e));return e},removePlotBandOrLine:function(a){for(var f=this.plotLinesAndBands,e=this.options,n=this.userOptions,l=f.length;l--;)f[l].id===a&&f[l].destroy();h([e.plotLines||[],n.plotLines||[],e.plotBands||[],n.plotBands||[]],function(e){for(l=e.length;l--;)e[l].id===a&&g(e,e[l])})}}})(J);(function(a){var y=a.addEvent,E=a.animObject,
A=a.arrayMax,B=a.arrayMin,h=a.AxisPlotLineOrBandExtension,g=a.color,t=a.correctFloat,r=a.defaultOptions,l=a.defined,f=a.deg2rad,e=a.destroyObjectProperties,n=a.each,z=a.extend,v=a.fireEvent,b=a.format,m=a.getMagnitude,c=a.grep,d=a.inArray,D=a.isArray,u=a.isNumber,q=a.isString,x=a.merge,G=a.normalizeTickInterval,H=a.pick,k=a.PlotLineOrBand,C=a.removeEvent,I=a.splat,L=a.syncTimeout,K=a.Tick;a.Axis=function(){this.init.apply(this,arguments)};a.Axis.prototype={defaultOptions:{dateTimeLabelFormats:{millisecond:"%H:%M:%S.%L",
second:"%H:%M:%S",minute:"%H:%M",hour:"%H:%M",day:"%e. %b",week:"%e. %b",month:"%b '%y",year:"%Y"},endOnTick:!1,labels:{enabled:!0,style:{color:"#666666",cursor:"default",fontSize:"11px"},x:0},minPadding:.01,maxPadding:.01,minorTickLength:2,minorTickPosition:"outside",startOfWeek:1,startOnTick:!1,tickLength:10,tickmarkPlacement:"between",tickPixelInterval:100,tickPosition:"outside",title:{align:"middle",style:{color:"#666666"}},type:"linear",minorGridLineColor:"#f2f2f2",minorGridLineWidth:1,minorTickColor:"#999999",
lineColor:"#ccd6eb",lineWidth:1,gridLineColor:"#e6e6e6",tickColor:"#ccd6eb"},defaultYAxisOptions:{endOnTick:!0,tickPixelInterval:72,showLastLabel:!0,labels:{x:-8},maxPadding:.05,minPadding:.05,startOnTick:!0,title:{rotation:270,text:"Values"},stackLabels:{enabled:!1,formatter:function(){return a.numberFormat(this.total,-1)},style:{fontSize:"11px",fontWeight:"bold",color:"#000000",textOutline:"1px contrast"}},gridLineWidth:1,lineWidth:0},defaultLeftAxisOptions:{labels:{x:-15},title:{rotation:270}},
defaultRightAxisOptions:{labels:{x:15},title:{rotation:90}},defaultBottomAxisOptions:{labels:{autoRotation:[-45],x:0},title:{rotation:0}},defaultTopAxisOptions:{labels:{autoRotation:[-45],x:0},title:{rotation:0}},init:function(a,b){var c=b.isX;this.chart=a;this.horiz=a.inverted?!c:c;this.isXAxis=c;this.coll=this.coll||(c?"xAxis":"yAxis");this.opposite=b.opposite;this.side=b.side||(this.horiz?this.opposite?0:2:this.opposite?1:3);this.setOptions(b);var p=this.options,k=p.type;this.labelFormatter=p.labels.formatter||
this.defaultLabelFormatter;this.userOptions=b;this.minPixelPadding=0;this.reversed=p.reversed;this.visible=!1!==p.visible;this.zoomEnabled=!1!==p.zoomEnabled;this.hasNames="category"===k||!0===p.categories;this.categories=p.categories||this.hasNames;this.names=this.names||[];this.isLog="logarithmic"===k;this.isDatetimeAxis="datetime"===k;this.positiveValuesOnly=this.isLog&&!this.allowNegativeLog;this.isLinked=l(p.linkedTo);this.ticks={};this.labelEdge=[];this.minorTicks={};this.plotLinesAndBands=
[];this.alternateBands={};this.len=0;this.minRange=this.userMinRange=p.minRange||p.maxZoom;this.range=p.range;this.offset=p.offset||0;this.stacks={};this.oldStacks={};this.stacksTouched=0;this.min=this.max=null;this.crosshair=H(p.crosshair,I(a.options.tooltip.crosshairs)[c?0:1],!1);var F;b=this.options.events;-1===d(this,a.axes)&&(c?a.axes.splice(a.xAxis.length,0,this):a.axes.push(this),a[this.coll].push(this));this.series=this.series||[];a.inverted&&c&&void 0===this.reversed&&(this.reversed=!0);
this.removePlotLine=this.removePlotBand=this.removePlotBandOrLine;for(F in b)y(this,F,b[F]);this.lin2log=p.linearToLogConverter||this.lin2log;this.isLog&&(this.val2lin=this.log2lin,this.lin2val=this.lin2log)},setOptions:function(a){this.options=x(this.defaultOptions,"yAxis"===this.coll&&this.defaultYAxisOptions,[this.defaultTopAxisOptions,this.defaultRightAxisOptions,this.defaultBottomAxisOptions,this.defaultLeftAxisOptions][this.side],x(r[this.coll],a))},defaultLabelFormatter:function(){var c=this.axis,
d=this.value,k=c.categories,m=this.dateTimeLabelFormat,e=r.lang,q=e.numericSymbols,e=e.numericSymbolMagnitude||1E3,w=q&&q.length,f,u=c.options.labels.format,c=c.isLog?Math.abs(d):c.tickInterval;if(u)f=b(u,this);else if(k)f=d;else if(m)f=a.dateFormat(m,d);else if(w&&1E3<=c)for(;w--&&void 0===f;)k=Math.pow(e,w+1),c>=k&&0===10*d%k&&null!==q[w]&&0!==d&&(f=a.numberFormat(d/k,-1)+q[w]);void 0===f&&(f=1E4<=Math.abs(d)?a.numberFormat(d,-1):a.numberFormat(d,-1,void 0,""));return f},getSeriesExtremes:function(){var a=
this,b=a.chart;a.hasVisibleSeries=!1;a.dataMin=a.dataMax=a.threshold=null;a.softThreshold=!a.isXAxis;a.buildStacks&&a.buildStacks();n(a.series,function(d){if(d.visible||!b.options.chart.ignoreHiddenSeries){var p=d.options,k=p.threshold,F;a.hasVisibleSeries=!0;a.positiveValuesOnly&&0>=k&&(k=null);if(a.isXAxis)p=d.xData,p.length&&(d=B(p),u(d)||d instanceof Date||(p=c(p,function(a){return u(a)}),d=B(p)),a.dataMin=Math.min(H(a.dataMin,p[0]),d),a.dataMax=Math.max(H(a.dataMax,p[0]),A(p)));else if(d.getExtremes(),
F=d.dataMax,d=d.dataMin,l(d)&&l(F)&&(a.dataMin=Math.min(H(a.dataMin,d),d),a.dataMax=Math.max(H(a.dataMax,F),F)),l(k)&&(a.threshold=k),!p.softThreshold||a.positiveValuesOnly)a.softThreshold=!1}})},translate:function(a,b,c,d,k,m){var p=this.linkedParent||this,F=1,e=0,q=d?p.oldTransA:p.transA;d=d?p.oldMin:p.min;var f=p.minPixelPadding;k=(p.isOrdinal||p.isBroken||p.isLog&&k)&&p.lin2val;q||(q=p.transA);c&&(F*=-1,e=p.len);p.reversed&&(F*=-1,e-=F*(p.sector||p.len));b?(a=(a*F+e-f)/q+d,k&&(a=p.lin2val(a))):
(k&&(a=p.val2lin(a)),a=F*(a-d)*q+e+F*f+(u(m)?q*m:0));return a},toPixels:function(a,b){return this.translate(a,!1,!this.horiz,null,!0)+(b?0:this.pos)},toValue:function(a,b){return this.translate(a-(b?0:this.pos),!0,!this.horiz,null,!0)},getPlotLinePath:function(a,b,c,d,k){var p=this.chart,F=this.left,m=this.top,e,q,f=c&&p.oldChartHeight||p.chartHeight,n=c&&p.oldChartWidth||p.chartWidth,g;e=this.transB;var h=function(a,b,c){if(a<b||a>c)d?a=Math.min(Math.max(b,a),c):g=!0;return a};k=H(k,this.translate(a,
null,null,c));a=c=Math.round(k+e);e=q=Math.round(f-k-e);u(k)?this.horiz?(e=m,q=f-this.bottom,a=c=h(a,F,F+this.width)):(a=F,c=n-this.right,e=q=h(e,m,m+this.height)):g=!0;return g&&!d?null:p.renderer.crispLine(["M",a,e,"L",c,q],b||1)},getLinearTickPositions:function(a,b,c){var d,p=t(Math.floor(b/a)*a);c=t(Math.ceil(c/a)*a);var k=[];if(this.single)return[b];for(b=p;b<=c;){k.push(b);b=t(b+a);if(b===d)break;d=b}return k},getMinorTickPositions:function(){var a=this,b=a.options,c=a.tickPositions,d=a.minorTickInterval,
k=[],m=a.pointRangePadding||0,e=a.min-m,m=a.max+m,q=m-e;if(q&&q/d<a.len/3)if(a.isLog)n(this.paddedTicks,function(b,c,p){c&&k.push.apply(k,a.getLogTickPositions(d,p[c-1],p[c],!0))});else if(a.isDatetimeAxis&&"auto"===b.minorTickInterval)k=k.concat(a.getTimeTicks(a.normalizeTimeTickInterval(d),e,m,b.startOfWeek));else for(b=e+(c[0]-e)%d;b<=m&&b!==k[0];b+=d)k.push(b);0!==k.length&&a.trimTicks(k);return k},adjustForMinRange:function(){var a=this.options,b=this.min,c=this.max,d,k=this.dataMax-this.dataMin>=
this.minRange,m,e,q,f,u,g;this.isXAxis&&void 0===this.minRange&&!this.isLog&&(l(a.min)||l(a.max)?this.minRange=null:(n(this.series,function(a){f=a.xData;for(e=u=a.xIncrement?1:f.length-1;0<e;e--)if(q=f[e]-f[e-1],void 0===m||q<m)m=q}),this.minRange=Math.min(5*m,this.dataMax-this.dataMin)));c-b<this.minRange&&(g=this.minRange,d=(g-c+b)/2,d=[b-d,H(a.min,b-d)],k&&(d[2]=this.isLog?this.log2lin(this.dataMin):this.dataMin),b=A(d),c=[b+g,H(a.max,b+g)],k&&(c[2]=this.isLog?this.log2lin(this.dataMax):this.dataMax),
c=B(c),c-b<g&&(d[0]=c-g,d[1]=H(a.min,c-g),b=A(d)));this.min=b;this.max=c},getClosest:function(){var a;this.categories?a=1:n(this.series,function(b){var c=b.closestPointRange,d=b.visible||!b.chart.options.chart.ignoreHiddenSeries;!b.noSharedTooltip&&l(c)&&d&&(a=l(a)?Math.min(a,c):c)});return a},nameToX:function(a){var b=D(this.categories),c=b?this.categories:this.names,p=a.options.x,k;a.series.requireSorting=!1;l(p)||(p=!1===this.options.uniqueNames?a.series.autoIncrement():d(a.name,c));-1===p?b||
(k=c.length):k=p;this.names[k]=a.name;return k},updateNames:function(){var a=this;0<this.names.length&&(this.names.length=0,this.minRange=void 0,n(this.series||[],function(b){b.xIncrement=null;if(!b.points||b.isDirtyData)b.processData(),b.generatePoints();n(b.points,function(c,d){var p;c.options&&(p=a.nameToX(c),p!==c.x&&(c.x=p,b.xData[d]=p))})}))},setAxisTranslation:function(a){var b=this,c=b.max-b.min,d=b.axisPointRange||0,p,k=0,m=0,e=b.linkedParent,f=!!b.categories,u=b.transA,g=b.isXAxis;if(g||
f||d)p=b.getClosest(),e?(k=e.minPointOffset,m=e.pointRangePadding):n(b.series,function(a){var c=f?1:g?H(a.options.pointRange,p,0):b.axisPointRange||0;a=a.options.pointPlacement;d=Math.max(d,c);b.single||(k=Math.max(k,q(a)?0:c/2),m=Math.max(m,"on"===a?0:c))}),e=b.ordinalSlope&&p?b.ordinalSlope/p:1,b.minPointOffset=k*=e,b.pointRangePadding=m*=e,b.pointRange=Math.min(d,c),g&&(b.closestPointRange=p);a&&(b.oldTransA=u);b.translationSlope=b.transA=u=b.options.staticScale||b.len/(c+m||1);b.transB=b.horiz?
b.left:b.bottom;b.minPixelPadding=u*k},minFromRange:function(){return this.max-this.range},setTickInterval:function(b){var c=this,d=c.chart,p=c.options,k=c.isLog,e=c.log2lin,q=c.isDatetimeAxis,f=c.isXAxis,g=c.isLinked,h=p.maxPadding,C=p.minPadding,x=p.tickInterval,D=p.tickPixelInterval,z=c.categories,I=c.threshold,K=c.softThreshold,r,L,B,y;q||z||g||this.getTickAmount();B=H(c.userMin,p.min);y=H(c.userMax,p.max);g?(c.linkedParent=d[c.coll][p.linkedTo],d=c.linkedParent.getExtremes(),c.min=H(d.min,d.dataMin),
c.max=H(d.max,d.dataMax),p.type!==c.linkedParent.options.type&&a.error(11,1)):(!K&&l(I)&&(c.dataMin>=I?(r=I,C=0):c.dataMax<=I&&(L=I,h=0)),c.min=H(B,r,c.dataMin),c.max=H(y,L,c.dataMax));k&&(c.positiveValuesOnly&&!b&&0>=Math.min(c.min,H(c.dataMin,c.min))&&a.error(10,1),c.min=t(e(c.min),15),c.max=t(e(c.max),15));c.range&&l(c.max)&&(c.userMin=c.min=B=Math.max(c.min,c.minFromRange()),c.userMax=y=c.max,c.range=null);v(c,"foundExtremes");c.beforePadding&&c.beforePadding();c.adjustForMinRange();!(z||c.axisPointRange||
c.usePercentage||g)&&l(c.min)&&l(c.max)&&(e=c.max-c.min)&&(!l(B)&&C&&(c.min-=e*C),!l(y)&&h&&(c.max+=e*h));u(p.floor)?c.min=Math.max(c.min,p.floor):u(p.softMin)&&(c.min=Math.min(c.min,p.softMin));u(p.ceiling)?c.max=Math.min(c.max,p.ceiling):u(p.softMax)&&(c.max=Math.max(c.max,p.softMax));K&&l(c.dataMin)&&(I=I||0,!l(B)&&c.min<I&&c.dataMin>=I?c.min=I:!l(y)&&c.max>I&&c.dataMax<=I&&(c.max=I));c.tickInterval=c.min===c.max||void 0===c.min||void 0===c.max?1:g&&!x&&D===c.linkedParent.options.tickPixelInterval?
x=c.linkedParent.tickInterval:H(x,this.tickAmount?(c.max-c.min)/Math.max(this.tickAmount-1,1):void 0,z?1:(c.max-c.min)*D/Math.max(c.len,D));f&&!b&&n(c.series,function(a){a.processData(c.min!==c.oldMin||c.max!==c.oldMax)});c.setAxisTranslation(!0);c.beforeSetTickPositions&&c.beforeSetTickPositions();c.postProcessTickInterval&&(c.tickInterval=c.postProcessTickInterval(c.tickInterval));c.pointRange&&!x&&(c.tickInterval=Math.max(c.pointRange,c.tickInterval));b=H(p.minTickInterval,c.isDatetimeAxis&&c.closestPointRange);
!x&&c.tickInterval<b&&(c.tickInterval=b);q||k||x||(c.tickInterval=G(c.tickInterval,null,m(c.tickInterval),H(p.allowDecimals,!(.5<c.tickInterval&&5>c.tickInterval&&1E3<c.max&&9999>c.max)),!!this.tickAmount));this.tickAmount||(c.tickInterval=c.unsquish());this.setTickPositions()},setTickPositions:function(){var a=this.options,b,c=a.tickPositions,d=a.tickPositioner,k=a.startOnTick,m=a.endOnTick;this.tickmarkOffset=this.categories&&"between"===a.tickmarkPlacement&&1===this.tickInterval?.5:0;this.minorTickInterval=
"auto"===a.minorTickInterval&&this.tickInterval?this.tickInterval/5:a.minorTickInterval;this.single=this.min===this.max&&l(this.min)&&!this.tickAmount&&!1!==a.allowDecimals;this.tickPositions=b=c&&c.slice();!b&&(b=this.isDatetimeAxis?this.getTimeTicks(this.normalizeTimeTickInterval(this.tickInterval,a.units),this.min,this.max,a.startOfWeek,this.ordinalPositions,this.closestPointRange,!0):this.isLog?this.getLogTickPositions(this.tickInterval,this.min,this.max):this.getLinearTickPositions(this.tickInterval,
this.min,this.max),b.length>this.len&&(b=[b[0],b.pop()]),this.tickPositions=b,d&&(d=d.apply(this,[this.min,this.max])))&&(this.tickPositions=b=d);this.paddedTicks=b.slice(0);this.trimTicks(b,k,m);this.isLinked||(this.single&&(this.min-=.5,this.max+=.5),c||d||this.adjustTickAmount())},trimTicks:function(a,b,c){var d=a[0],p=a[a.length-1],k=this.minPointOffset||0;if(!this.isLinked){if(b)this.min=d;else for(;this.min-k>a[0];)a.shift();if(c)this.max=p;else for(;this.max+k<a[a.length-1];)a.pop();0===a.length&&
l(d)&&a.push((p+d)/2)}},alignToOthers:function(){var a={},b,c=this.options;!1===this.chart.options.chart.alignTicks||!1===c.alignTicks||this.isLog||n(this.chart[this.coll],function(c){var d=c.options,d=[c.horiz?d.left:d.top,d.width,d.height,d.pane].join();c.series.length&&(a[d]?b=!0:a[d]=1)});return b},getTickAmount:function(){var a=this.options,b=a.tickAmount,c=a.tickPixelInterval;!l(a.tickInterval)&&this.len<c&&!this.isRadial&&!this.isLog&&a.startOnTick&&a.endOnTick&&(b=2);!b&&this.alignToOthers()&&
(b=Math.ceil(this.len/c)+1);4>b&&(this.finalTickAmt=b,b=5);this.tickAmount=b},adjustTickAmount:function(){var a=this.tickInterval,b=this.tickPositions,c=this.tickAmount,d=this.finalTickAmt,k=b&&b.length;if(k<c){for(;b.length<c;)b.push(t(b[b.length-1]+a));this.transA*=(k-1)/(c-1);this.max=b[b.length-1]}else k>c&&(this.tickInterval*=2,this.setTickPositions());if(l(d)){for(a=c=b.length;a--;)(3===d&&1===a%2||2>=d&&0<a&&a<c-1)&&b.splice(a,1);this.finalTickAmt=void 0}},setScale:function(){var a,b;this.oldMin=
this.min;this.oldMax=this.max;this.oldAxisLength=this.len;this.setAxisSize();b=this.len!==this.oldAxisLength;n(this.series,function(b){if(b.isDirtyData||b.isDirty||b.xAxis.isDirty)a=!0});b||a||this.isLinked||this.forceRedraw||this.userMin!==this.oldUserMin||this.userMax!==this.oldUserMax||this.alignToOthers()?(this.resetStacks&&this.resetStacks(),this.forceRedraw=!1,this.getSeriesExtremes(),this.setTickInterval(),this.oldUserMin=this.userMin,this.oldUserMax=this.userMax,this.isDirty||(this.isDirty=
b||this.min!==this.oldMin||this.max!==this.oldMax)):this.cleanStacks&&this.cleanStacks()},setExtremes:function(a,b,c,d,k){var p=this,m=p.chart;c=H(c,!0);n(p.series,function(a){delete a.kdTree});k=z(k,{min:a,max:b});v(p,"setExtremes",k,function(){p.userMin=a;p.userMax=b;p.eventArgs=k;c&&m.redraw(d)})},zoom:function(a,b){var c=this.dataMin,d=this.dataMax,p=this.options,k=Math.min(c,H(p.min,c)),p=Math.max(d,H(p.max,d));if(a!==this.min||b!==this.max)this.allowZoomOutside||(l(c)&&(a<k&&(a=k),a>p&&(a=p)),
l(d)&&(b<k&&(b=k),b>p&&(b=p))),this.displayBtn=void 0!==a||void 0!==b,this.setExtremes(a,b,!1,void 0,{trigger:"zoom"});return!0},setAxisSize:function(){var a=this.chart,b=this.options,c=b.offsets||[0,0,0,0],d=this.horiz,k=H(b.width,a.plotWidth-c[3]+c[1]),m=H(b.height,a.plotHeight-c[0]+c[2]),e=H(b.top,a.plotTop+c[0]),b=H(b.left,a.plotLeft+c[3]),c=/%$/;c.test(m)&&(m=Math.round(parseFloat(m)/100*a.plotHeight));c.test(e)&&(e=Math.round(parseFloat(e)/100*a.plotHeight+a.plotTop));this.left=b;this.top=e;
this.width=k;this.height=m;this.bottom=a.chartHeight-m-e;this.right=a.chartWidth-k-b;this.len=Math.max(d?k:m,0);this.pos=d?b:e},getExtremes:function(){var a=this.isLog,b=this.lin2log;return{min:a?t(b(this.min)):this.min,max:a?t(b(this.max)):this.max,dataMin:this.dataMin,dataMax:this.dataMax,userMin:this.userMin,userMax:this.userMax}},getThreshold:function(a){var b=this.isLog,c=this.lin2log,d=b?c(this.min):this.min,b=b?c(this.max):this.max;null===a?a=d:d>a?a=d:b<a&&(a=b);return this.translate(a,0,
1,0,1)},autoLabelAlign:function(a){a=(H(a,0)-90*this.side+720)%360;return 15<a&&165>a?"right":195<a&&345>a?"left":"center"},tickSize:function(a){var b=this.options,c=b[a+"Length"],d=H(b[a+"Width"],"tick"===a&&this.isXAxis?1:0);if(d&&c)return"inside"===b[a+"Position"]&&(c=-c),[c,d]},labelMetrics:function(){return this.chart.renderer.fontMetrics(this.options.labels.style&&this.options.labels.style.fontSize,this.ticks[0]&&this.ticks[0].label)},unsquish:function(){var a=this.options.labels,b=this.horiz,
c=this.tickInterval,d=c,k=this.len/(((this.categories?1:0)+this.max-this.min)/c),m,e=a.rotation,q=this.labelMetrics(),u,g=Number.MAX_VALUE,h,C=function(a){a/=k||1;a=1<a?Math.ceil(a):1;return a*c};b?(h=!a.staggerLines&&!a.step&&(l(e)?[e]:k<H(a.autoRotationLimit,80)&&a.autoRotation))&&n(h,function(a){var b;if(a===e||a&&-90<=a&&90>=a)u=C(Math.abs(q.h/Math.sin(f*a))),b=u+Math.abs(a/360),b<g&&(g=b,m=a,d=u)}):a.step||(d=C(q.h));this.autoRotation=h;this.labelRotation=H(m,e);return d},getSlotWidth:function(){var a=
this.chart,b=this.horiz,c=this.options.labels,d=Math.max(this.tickPositions.length-(this.categories?0:1),1),k=a.margin[3];return b&&2>(c.step||0)&&!c.rotation&&(this.staggerLines||1)*this.len/d||!b&&(k&&k-a.spacing[3]||.33*a.chartWidth)},renderUnsquish:function(){var a=this.chart,b=a.renderer,c=this.tickPositions,d=this.ticks,k=this.options.labels,m=this.horiz,e=this.getSlotWidth(),f=Math.max(1,Math.round(e-2*(k.padding||5))),u={},g=this.labelMetrics(),h=k.style&&k.style.textOverflow,C,l=0,D,v;q(k.rotation)||
(u.rotation=k.rotation||0);n(c,function(a){(a=d[a])&&a.labelLength>l&&(l=a.labelLength)});this.maxLabelLength=l;if(this.autoRotation)l>f&&l>g.h?u.rotation=this.labelRotation:this.labelRotation=0;else if(e&&(C={width:f+"px"},!h))for(C.textOverflow="clip",D=c.length;!m&&D--;)if(v=c[D],f=d[v].label)f.styles&&"ellipsis"===f.styles.textOverflow?f.css({textOverflow:"clip"}):d[v].labelLength>e&&f.css({width:e+"px"}),f.getBBox().height>this.len/c.length-(g.h-g.f)&&(f.specCss={textOverflow:"ellipsis"});u.rotation&&
(C={width:(l>.5*a.chartHeight?.33*a.chartHeight:a.chartHeight)+"px"},h||(C.textOverflow="ellipsis"));if(this.labelAlign=k.align||this.autoLabelAlign(this.labelRotation))u.align=this.labelAlign;n(c,function(a){var b=(a=d[a])&&a.label;b&&(b.attr(u),C&&b.css(x(C,b.specCss)),delete b.specCss,a.rotation=u.rotation)});this.tickRotCorr=b.rotCorr(g.b,this.labelRotation||0,0!==this.side)},hasData:function(){return this.hasVisibleSeries||l(this.min)&&l(this.max)&&!!this.tickPositions},addTitle:function(a){var b=
this.chart.renderer,c=this.horiz,d=this.opposite,k=this.options.title,p;this.axisTitle||((p=k.textAlign)||(p=(c?{low:"left",middle:"center",high:"right"}:{low:d?"right":"left",middle:"center",high:d?"left":"right"})[k.align]),this.axisTitle=b.text(k.text,0,0,k.useHTML).attr({zIndex:7,rotation:k.rotation||0,align:p}).addClass("highcharts-axis-title").css(k.style).add(this.axisGroup),this.axisTitle.isNew=!0);this.axisTitle[a?"show":"hide"](!0)},generateTick:function(a){var b=this.ticks;b[a]?b[a].addLabel():
b[a]=new K(this,a)},getOffset:function(){var a=this,b=a.chart,c=b.renderer,d=a.options,k=a.tickPositions,m=a.ticks,e=a.horiz,f=a.side,q=b.inverted?[1,0,3,2][f]:f,u,g,h=0,C,x=0,D=d.title,v=d.labels,G=0,I=b.axisOffset,b=b.clipOffset,z=[-1,1,1,-1][f],K,r=d.className,L=a.axisParent,t=this.tickSize("tick");u=a.hasData();a.showAxis=g=u||H(d.showEmpty,!0);a.staggerLines=a.horiz&&v.staggerLines;a.axisGroup||(a.gridGroup=c.g("grid").attr({zIndex:d.gridZIndex||1}).addClass("highcharts-"+this.coll.toLowerCase()+
"-grid "+(r||"")).add(L),a.axisGroup=c.g("axis").attr({zIndex:d.zIndex||2}).addClass("highcharts-"+this.coll.toLowerCase()+" "+(r||"")).add(L),a.labelGroup=c.g("axis-labels").attr({zIndex:v.zIndex||7}).addClass("highcharts-"+a.coll.toLowerCase()+"-labels "+(r||"")).add(L));if(u||a.isLinked)n(k,function(b,c){a.generateTick(b,c)}),a.renderUnsquish(),!1===v.reserveSpace||0!==f&&2!==f&&{1:"left",3:"right"}[f]!==a.labelAlign&&"center"!==a.labelAlign||n(k,function(a){G=Math.max(m[a].getLabelSize(),G)}),
a.staggerLines&&(G*=a.staggerLines,a.labelOffset=G*(a.opposite?-1:1));else for(K in m)m[K].destroy(),delete m[K];D&&D.text&&!1!==D.enabled&&(a.addTitle(g),g&&(h=a.axisTitle.getBBox()[e?"height":"width"],C=D.offset,x=l(C)?0:H(D.margin,e?5:10)));a.renderLine();a.offset=z*H(d.offset,I[f]);a.tickRotCorr=a.tickRotCorr||{x:0,y:0};c=0===f?-a.labelMetrics().h:2===f?a.tickRotCorr.y:0;x=Math.abs(G)+x;G&&(x=x-c+z*(e?H(v.y,a.tickRotCorr.y+8*z):v.x));a.axisTitleMargin=H(C,x);I[f]=Math.max(I[f],a.axisTitleMargin+
h+z*a.offset,x,u&&k.length&&t?t[0]+z*a.offset:0);d=d.offset?0:2*Math.floor(a.axisLine.strokeWidth()/2);b[q]=Math.max(b[q],d)},getLinePath:function(a){var b=this.chart,c=this.opposite,d=this.offset,k=this.horiz,p=this.left+(c?this.width:0)+d,d=b.chartHeight-this.bottom-(c?this.height:0)+d;c&&(a*=-1);return b.renderer.crispLine(["M",k?this.left:p,k?d:this.top,"L",k?b.chartWidth-this.right:p,k?d:b.chartHeight-this.bottom],a)},renderLine:function(){this.axisLine||(this.axisLine=this.chart.renderer.path().addClass("highcharts-axis-line").add(this.axisGroup),
this.axisLine.attr({stroke:this.options.lineColor,"stroke-width":this.options.lineWidth,zIndex:7}))},getTitlePosition:function(){var a=this.horiz,b=this.left,c=this.top,d=this.len,k=this.options.title,m=a?b:c,e=this.opposite,f=this.offset,q=k.x||0,u=k.y||0,g=this.chart.renderer.fontMetrics(k.style&&k.style.fontSize,this.axisTitle).f,d={low:m+(a?0:d),middle:m+d/2,high:m+(a?d:0)}[k.align],b=(a?c+this.height:b)+(a?1:-1)*(e?-1:1)*this.axisTitleMargin+(2===this.side?g:0);return{x:a?d+q:b+(e?this.width:
0)+f+q,y:a?b+u-(e?this.height:0)+f:d+u}},renderMinorTick:function(a){var b=this.chart.hasRendered&&u(this.oldMin),c=this.minorTicks;c[a]||(c[a]=new K(this,a,"minor"));b&&c[a].isNew&&c[a].render(null,!0);c[a].render(null,!1,1)},renderTick:function(a,b){var c=this.isLinked,d=this.ticks,k=this.chart.hasRendered&&u(this.oldMin);if(!c||a>=this.min&&a<=this.max)d[a]||(d[a]=new K(this,a)),k&&d[a].isNew&&d[a].render(b,!0,.1),d[a].render(b)},render:function(){var a=this,b=a.chart,c=a.options,d=a.isLog,m=a.lin2log,
e=a.isLinked,f=a.tickPositions,q=a.axisTitle,u=a.ticks,g=a.minorTicks,h=a.alternateBands,C=c.stackLabels,x=c.alternateGridColor,l=a.tickmarkOffset,D=a.axisLine,v=a.showAxis,G=E(b.renderer.globalAnimation),I,z;a.labelEdge.length=0;a.overlap=!1;n([u,g,h],function(a){for(var b in a)a[b].isActive=!1});if(a.hasData()||e)a.minorTickInterval&&!a.categories&&n(a.getMinorTickPositions(),function(b){a.renderMinorTick(b)}),f.length&&(n(f,function(b,c){a.renderTick(b,c)}),l&&(0===a.min||a.single)&&(u[-1]||(u[-1]=
new K(a,-1,null,!0)),u[-1].render(-1))),x&&n(f,function(c,e){z=void 0!==f[e+1]?f[e+1]+l:a.max-l;0===e%2&&c<a.max&&z<=a.max+(b.polar?-l:l)&&(h[c]||(h[c]=new k(a)),I=c+l,h[c].options={from:d?m(I):I,to:d?m(z):z,color:x},h[c].render(),h[c].isActive=!0)}),a._addedPlotLB||(n((c.plotLines||[]).concat(c.plotBands||[]),function(b){a.addPlotBandOrLine(b)}),a._addedPlotLB=!0);n([u,g,h],function(a){var c,d,k=[],m=G.duration;for(c in a)a[c].isActive||(a[c].render(c,!1,0),a[c].isActive=!1,k.push(c));L(function(){for(d=
k.length;d--;)a[k[d]]&&!a[k[d]].isActive&&(a[k[d]].destroy(),delete a[k[d]])},a!==h&&b.hasRendered&&m?m:0)});D&&(D[D.isPlaced?"animate":"attr"]({d:this.getLinePath(D.strokeWidth())}),D.isPlaced=!0,D[v?"show":"hide"](!0));q&&v&&(q[q.isNew?"attr":"animate"](a.getTitlePosition()),q.isNew=!1);C&&C.enabled&&a.renderStackTotals();a.isDirty=!1},redraw:function(){this.visible&&(this.render(),n(this.plotLinesAndBands,function(a){a.render()}));n(this.series,function(a){a.isDirty=!0})},keepProps:"extKey hcEvents names series userMax userMin".split(" "),
destroy:function(a){var b=this,c=b.stacks,k,m=b.plotLinesAndBands,p;a||C(b);for(k in c)e(c[k]),c[k]=null;n([b.ticks,b.minorTicks,b.alternateBands],function(a){e(a)});if(m)for(a=m.length;a--;)m[a].destroy();n("stackTotalGroup axisLine axisTitle axisGroup gridGroup labelGroup cross".split(" "),function(a){b[a]&&(b[a]=b[a].destroy())});for(p in b)b.hasOwnProperty(p)&&-1===d(p,b.keepProps)&&delete b[p]},drawCrosshair:function(a,b){var c,d=this.crosshair,k=H(d.snap,!0),m,e=this.cross;a||(a=this.cross&&
this.cross.e);this.crosshair&&!1!==(l(b)||!k)?(k?l(b)&&(m=this.isXAxis?b.plotX:this.len-b.plotY):m=a&&(this.horiz?a.chartX-this.pos:this.len-a.chartY+this.pos),l(m)&&(c=this.getPlotLinePath(b&&(this.isXAxis?b.x:H(b.stackY,b.y)),null,null,null,m)||null),l(c)?(b=this.categories&&!this.isRadial,e||(this.cross=e=this.chart.renderer.path().addClass("highcharts-crosshair highcharts-crosshair-"+(b?"category ":"thin ")+d.className).attr({zIndex:H(d.zIndex,2)}).add(),e.attr({stroke:d.color||(b?g("#ccd6eb").setOpacity(.25).get():
"#cccccc"),"stroke-width":H(d.width,1)}),d.dashStyle&&e.attr({dashstyle:d.dashStyle})),e.show().attr({d:c}),b&&!d.width&&e.attr({"stroke-width":this.transA}),this.cross.e=a):this.hideCrosshair()):this.hideCrosshair()},hideCrosshair:function(){this.cross&&this.cross.hide()}};z(a.Axis.prototype,h)})(J);(function(a){var y=a.Axis,E=a.getMagnitude,A=a.map,B=a.normalizeTickInterval,h=a.pick;y.prototype.getLogTickPositions=function(a,t,r,l){var f=this.options,e=this.len,g=this.lin2log,z=this.log2lin,v=[];
l||(this._minorAutoInterval=null);if(.5<=a)a=Math.round(a),v=this.getLinearTickPositions(a,t,r);else if(.08<=a)for(var e=Math.floor(t),b,m,c,d,D,f=.3<a?[1,2,4]:.15<a?[1,2,4,6,8]:[1,2,3,4,5,6,7,8,9];e<r+1&&!D;e++)for(m=f.length,b=0;b<m&&!D;b++)c=z(g(e)*f[b]),c>t&&(!l||d<=r)&&void 0!==d&&v.push(d),d>r&&(D=!0),d=c;else t=g(t),r=g(r),a=f[l?"minorTickInterval":"tickInterval"],a=h("auto"===a?null:a,this._minorAutoInterval,f.tickPixelInterval/(l?5:1)*(r-t)/((l?e/this.tickPositions.length:e)||1)),a=B(a,null,
E(a)),v=A(this.getLinearTickPositions(a,t,r),z),l||(this._minorAutoInterval=a/5);l||(this.tickInterval=a);return v};y.prototype.log2lin=function(a){return Math.log(a)/Math.LN10};y.prototype.lin2log=function(a){return Math.pow(10,a)}})(J);(function(a){var y=a.dateFormat,E=a.each,A=a.extend,B=a.format,h=a.isNumber,g=a.map,t=a.merge,r=a.pick,l=a.splat,f=a.syncTimeout,e=a.timeUnits;a.Tooltip=function(){this.init.apply(this,arguments)};a.Tooltip.prototype={init:function(a,e){this.chart=a;this.options=
e;this.crosshairs=[];this.now={x:0,y:0};this.isHidden=!0;this.split=e.split&&!a.inverted;this.shared=e.shared||this.split},cleanSplit:function(a){E(this.chart.series,function(e){var f=e&&e.tt;f&&(!f.isActive||a?e.tt=f.destroy():f.isActive=!1)})},getLabel:function(){var a=this.chart.renderer,e=this.options;this.label||(this.split?this.label=a.g("tooltip"):(this.label=a.label("",0,0,e.shape||"callout",null,null,e.useHTML,null,"tooltip").attr({padding:e.padding,r:e.borderRadius}),this.label.attr({fill:e.backgroundColor,
"stroke-width":e.borderWidth}).css(e.style).shadow(e.shadow)),this.label.attr({zIndex:8}).add());return this.label},update:function(a){this.destroy();this.init(this.chart,t(!0,this.options,a))},destroy:function(){this.label&&(this.label=this.label.destroy());this.split&&this.tt&&(this.cleanSplit(this.chart,!0),this.tt=this.tt.destroy());clearTimeout(this.hideTimer);clearTimeout(this.tooltipTimeout)},move:function(a,e,f,b){var m=this,c=m.now,d=!1!==m.options.animation&&!m.isHidden&&(1<Math.abs(a-c.x)||
1<Math.abs(e-c.y)),g=m.followPointer||1<m.len;A(c,{x:d?(2*c.x+a)/3:a,y:d?(c.y+e)/2:e,anchorX:g?void 0:d?(2*c.anchorX+f)/3:f,anchorY:g?void 0:d?(c.anchorY+b)/2:b});m.getLabel().attr(c);d&&(clearTimeout(this.tooltipTimeout),this.tooltipTimeout=setTimeout(function(){m&&m.move(a,e,f,b)},32))},hide:function(a){var e=this;clearTimeout(this.hideTimer);a=r(a,this.options.hideDelay,500);this.isHidden||(this.hideTimer=f(function(){e.getLabel()[a?"fadeOut":"hide"]();e.isHidden=!0},a))},getAnchor:function(a,
e){var f,b=this.chart,m=b.inverted,c=b.plotTop,d=b.plotLeft,h=0,u=0,q,n;a=l(a);f=a[0].tooltipPos;this.followPointer&&e&&(void 0===e.chartX&&(e=b.pointer.normalize(e)),f=[e.chartX-b.plotLeft,e.chartY-c]);f||(E(a,function(a){q=a.series.yAxis;n=a.series.xAxis;h+=a.plotX+(!m&&n?n.left-d:0);u+=(a.plotLow?(a.plotLow+a.plotHigh)/2:a.plotY)+(!m&&q?q.top-c:0)}),h/=a.length,u/=a.length,f=[m?b.plotWidth-u:h,this.shared&&!m&&1<a.length&&e?e.chartY-c:m?b.plotHeight-h:u]);return g(f,Math.round)},getPosition:function(a,
e,f){var b=this.chart,m=this.distance,c={},d=f.h||0,g,u=["y",b.chartHeight,e,f.plotY+b.plotTop,b.plotTop,b.plotTop+b.plotHeight],q=["x",b.chartWidth,a,f.plotX+b.plotLeft,b.plotLeft,b.plotLeft+b.plotWidth],h=!this.followPointer&&r(f.ttBelow,!b.inverted===!!f.negative),n=function(a,b,k,e,f,q){var p=k<e-m,u=e+m+k<b,g=e-m-k;e+=m;if(h&&u)c[a]=e;else if(!h&&p)c[a]=g;else if(p)c[a]=Math.min(q-k,0>g-d?g:g-d);else if(u)c[a]=Math.max(f,e+d+k>b?e:e+d);else return!1},l=function(a,b,d,k){var e;k<m||k>b-m?e=!1:
c[a]=k<d/2?1:k>b-d/2?b-d-2:k-d/2;return e},k=function(a){var b=u;u=q;q=b;g=a},C=function(){!1!==n.apply(0,u)?!1!==l.apply(0,q)||g||(k(!0),C()):g?c.x=c.y=0:(k(!0),C())};(b.inverted||1<this.len)&&k();C();return c},defaultFormatter:function(a){var e=this.points||l(this),f;f=[a.tooltipFooterHeaderFormatter(e[0])];f=f.concat(a.bodyFormatter(e));f.push(a.tooltipFooterHeaderFormatter(e[0],!0));return f},refresh:function(a,e){var f,b=this.options,m,c=a,d,g={},u=[];f=b.formatter||this.defaultFormatter;var g=
this.shared,q;clearTimeout(this.hideTimer);this.followPointer=l(c)[0].series.tooltipOptions.followPointer;d=this.getAnchor(c,e);e=d[0];m=d[1];!g||c.series&&c.series.noSharedTooltip?g=c.getLabelConfig():(E(c,function(a){a.setState("hover");u.push(a.getLabelConfig())}),g={x:c[0].category,y:c[0].y},g.points=u,c=c[0]);this.len=u.length;g=f.call(g,this);q=c.series;this.distance=r(q.tooltipOptions.distance,16);!1===g?this.hide():(f=this.getLabel(),this.isHidden&&f.attr({opacity:1}).show(),this.split?this.renderSplit(g,
a):(f.attr({text:g&&g.join?g.join(""):g}),f.removeClass(/highcharts-color-[\d]+/g).addClass("highcharts-color-"+r(c.colorIndex,q.colorIndex)),f.attr({stroke:b.borderColor||c.color||q.color||"#666666"}),this.updatePosition({plotX:e,plotY:m,negative:c.negative,ttBelow:c.ttBelow,h:d[2]||0})),this.isHidden=!1)},renderSplit:function(e,f){var g=this,b=[],m=this.chart,c=m.renderer,d=!0,h=this.options,u,q=this.getLabel();E(e.slice(0,f.length+1),function(a,e){e=f[e-1]||{isHeader:!0,plotX:f[0].plotX};var n=
e.series||g,k=n.tt,C=e.series||{},l="highcharts-color-"+r(e.colorIndex,C.colorIndex,"none");k||(n.tt=k=c.label(null,null,null,"callout").addClass("highcharts-tooltip-box "+l).attr({padding:h.padding,r:h.borderRadius,fill:h.backgroundColor,stroke:e.color||C.color||"#333333","stroke-width":h.borderWidth}).add(q));k.isActive=!0;k.attr({text:a});k.css(h.style);a=k.getBBox();C=a.width+k.strokeWidth();e.isHeader?(u=a.height,C=Math.max(0,Math.min(e.plotX+m.plotLeft-C/2,m.chartWidth-C))):C=e.plotX+m.plotLeft-
r(h.distance,16)-C;0>C&&(d=!1);a=(e.series&&e.series.yAxis&&e.series.yAxis.pos)+(e.plotY||0);a-=m.plotTop;b.push({target:e.isHeader?m.plotHeight+u:a,rank:e.isHeader?1:0,size:n.tt.getBBox().height+1,point:e,x:C,tt:k})});this.cleanSplit();a.distribute(b,m.plotHeight+u);E(b,function(a){var b=a.point,c=b.series;a.tt.attr({visibility:void 0===a.pos?"hidden":"inherit",x:d||b.isHeader?a.x:b.plotX+m.plotLeft+r(h.distance,16),y:a.pos+m.plotTop,anchorX:b.isHeader?b.plotX+m.plotLeft:b.plotX+c.xAxis.pos,anchorY:b.isHeader?
a.pos+m.plotTop-15:b.plotY+c.yAxis.pos})})},updatePosition:function(a){var e=this.chart,f=this.getLabel(),f=(this.options.positioner||this.getPosition).call(this,f.width,f.height,a);this.move(Math.round(f.x),Math.round(f.y||0),a.plotX+e.plotLeft,a.plotY+e.plotTop)},getDateFormat:function(a,f,g,b){var m=y("%m-%d %H:%M:%S.%L",f),c,d,h={millisecond:15,second:12,minute:9,hour:6,day:3},u="millisecond";for(d in e){if(a===e.week&&+y("%w",f)===g&&"00:00:00.000"===m.substr(6)){d="week";break}if(e[d]>a){d=
u;break}if(h[d]&&m.substr(h[d])!=="01-01 00:00:00.000".substr(h[d]))break;"week"!==d&&(u=d)}d&&(c=b[d]);return c},getXDateFormat:function(a,e,f){e=e.dateTimeLabelFormats;var b=f&&f.closestPointRange;return(b?this.getDateFormat(b,a.x,f.options.startOfWeek,e):e.day)||e.year},tooltipFooterHeaderFormatter:function(a,e){var f=e?"footer":"header";e=a.series;var b=e.tooltipOptions,m=b.xDateFormat,c=e.xAxis,d=c&&"datetime"===c.options.type&&h(a.key),f=b[f+"Format"];d&&!m&&(m=this.getXDateFormat(a,b,c));d&&
m&&(f=f.replace("{point.key}","{point.key:"+m+"}"));return B(f,{point:a,series:e})},bodyFormatter:function(a){return g(a,function(a){var e=a.series.tooltipOptions;return(e.pointFormatter||a.point.tooltipFormatter).call(a.point,e.pointFormat)})}}})(J);(function(a){var y=a.addEvent,E=a.attr,A=a.charts,B=a.color,h=a.css,g=a.defined,t=a.doc,r=a.each,l=a.extend,f=a.fireEvent,e=a.offset,n=a.pick,z=a.removeEvent,v=a.splat,b=a.Tooltip,m=a.win;a.Pointer=function(a,b){this.init(a,b)};a.Pointer.prototype={init:function(a,
d){this.options=d;this.chart=a;this.runChartClick=d.chart.events&&!!d.chart.events.click;this.pinchDown=[];this.lastValidTouch={};b&&d.tooltip.enabled&&(a.tooltip=new b(a,d.tooltip),this.followTouchMove=n(d.tooltip.followTouchMove,!0));this.setDOMEvents()},zoomOption:function(a){var b=this.chart,c=b.options.chart,e=c.zoomType||"",b=b.inverted;/touch/.test(a.type)&&(e=n(c.pinchType,e));this.zoomX=a=/x/.test(e);this.zoomY=e=/y/.test(e);this.zoomHor=a&&!b||e&&b;this.zoomVert=e&&!b||a&&b;this.hasZoom=
a||e},normalize:function(a,b){var c,d;a=a||m.event;a.target||(a.target=a.srcElement);d=a.touches?a.touches.length?a.touches.item(0):a.changedTouches[0]:a;b||(this.chartPosition=b=e(this.chart.container));void 0===d.pageX?(c=Math.max(a.x,a.clientX-b.left),b=a.y):(c=d.pageX-b.left,b=d.pageY-b.top);return l(a,{chartX:Math.round(c),chartY:Math.round(b)})},getCoordinates:function(a){var b={xAxis:[],yAxis:[]};r(this.chart.axes,function(c){b[c.isXAxis?"xAxis":"yAxis"].push({axis:c,value:c.toValue(a[c.horiz?
"chartX":"chartY"])})});return b},getKDPoints:function(a,b,e){var c=[],d,m,f;r(a,function(a){d=a.noSharedTooltip&&b;m=!b&&a.directTouch;a.visible&&!d&&!m&&n(a.options.enableMouseTracking,!0)&&(f=a.searchPoint(e,!d&&1===a.kdDimensions))&&f.series&&c.push(f)});c.sort(function(a,c){var d=a.distX-c.distX,k=a.dist-c.dist,e=(c.series.group&&c.series.group.zIndex)-(a.series.group&&a.series.group.zIndex);return 0!==d&&b?d:0!==k?k:0!==e?e:a.series.index>c.series.index?-1:1});if(b)for(a=c.length;a--;)(c[a].x!==
c[0].x||c[a].series.noSharedTooltip)&&c.splice(a,1);return c},getPointFromEvent:function(a){a=a.target;for(var b;a&&!b;)b=a.point,a=a.parentNode;return b},getHoverData:function(a,b,e,m,f,g){var c=a;a=b;var d;if(m)f?(d=[],r(e,function(a){var b=a.noSharedTooltip&&f,k=!f&&a.directTouch;a.visible&&!b&&!k&&n(a.options.enableMouseTracking,!0)&&(a=a.searchKDTree({clientX:c.clientX,plotY:c.plotY},!b&&1===a.kdDimensions))&&a.series&&d.push(a)}),0===d.length&&(d=[c])):d=[c];else{if(a&&!a.options.stickyTracking)d=
this.getKDPoints([a],f,g);else{if(!f)if(a)a.options.stickyTracking||(e=[a]);else for(m=0;m<e.length;m++)if(e[m].directTouch||!e[m].options.stickyTracking)e=[];d=this.getKDPoints(e,f,g)}a=(c=d[0])&&c.series}d.sort(function(a,b){return a.series.index-b.series.index});return{hoverPoint:c,hoverSeries:a,hoverPoints:d}},runPointActions:function(b,d){var c=this.chart,e=c.tooltip,m=e?e.shared:!1,f=d||c.hoverPoint,g=f&&f.series||c.hoverSeries;d=this.getHoverData(f,g,c.series,!!d||g&&g.directTouch,m,b);var h,
k,f=d.hoverPoint;h=(g=d.hoverSeries)&&g.tooltipOptions.followPointer;k=(m=m&&f&&!f.series.noSharedTooltip)?d.hoverPoints:[f];if(f&&(f!==c.hoverPoint||e&&e.isHidden)){r(c.hoverPoints||[],function(b){-1===a.inArray(b,k)&&b.setState()});r(k||[],function(a){a.setState("hover")});if(c.hoverSeries!==g)g.onMouseOver();g&&!g.directTouch&&(c.hoverPoint&&c.hoverPoint.firePointEvent("mouseOut"),f.firePointEvent("mouseOver"));c.hoverPoints=k;c.hoverPoint=f;e&&e.refresh(m?k:f,b)}else h&&e&&!e.isHidden&&(f=e.getAnchor([{}],
b),e.updatePosition({plotX:f[0],plotY:f[1]}));this.unDocMouseMove||(this.unDocMouseMove=y(t,"mousemove",function(b){var c=A[a.hoverChartIndex];if(c)c.pointer.onDocumentMouseMove(b)}));r(k,function(a){r(c.axes,function(c){(!a||a.series&&a.series[c.coll]===c)&&c.drawCrosshair(b,a)})})},reset:function(a,b){var c=this.chart,d=c.hoverSeries,e=c.hoverPoint,m=c.hoverPoints,f=c.tooltip,g=f&&f.shared?m:e;a&&g&&r(v(g),function(b){b.series.isCartesian&&void 0===b.plotX&&(a=!1)});if(a)f&&g&&(f.refresh(g),e&&
(e.setState(e.state,!0),r(c.axes,function(a){a.crosshair&&a.drawCrosshair(null,e)})));else{if(e)e.onMouseOut();m&&r(m,function(a){a.setState()});if(d)d.onMouseOut();f&&f.hide(b);this.unDocMouseMove&&(this.unDocMouseMove=this.unDocMouseMove());r(c.axes,function(a){a.hideCrosshair()});this.hoverX=c.hoverPoints=c.hoverPoint=null}},scaleGroups:function(a,b){var c=this.chart,d;r(c.series,function(e){d=a||e.getPlotBox();e.xAxis&&e.xAxis.zoomEnabled&&e.group&&(e.group.attr(d),e.markerGroup&&(e.markerGroup.attr(d),
e.markerGroup.clip(b?c.clipRect:null)),e.dataLabelsGroup&&e.dataLabelsGroup.attr(d))});c.clipRect.attr(b||c.clipBox)},dragStart:function(a){var b=this.chart;b.mouseIsDown=a.type;b.cancelClick=!1;b.mouseDownX=this.mouseDownX=a.chartX;b.mouseDownY=this.mouseDownY=a.chartY},drag:function(a){var b=this.chart,c=b.options.chart,e=a.chartX,m=a.chartY,f=this.zoomHor,g=this.zoomVert,h=b.plotLeft,k=b.plotTop,C=b.plotWidth,l=b.plotHeight,n,v=this.selectionMarker,p=this.mouseDownX,r=this.mouseDownY,t=c.panKey&&
a[c.panKey+"Key"];v&&v.touch||(e<h?e=h:e>h+C&&(e=h+C),m<k?m=k:m>k+l&&(m=k+l),this.hasDragged=Math.sqrt(Math.pow(p-e,2)+Math.pow(r-m,2)),10<this.hasDragged&&(n=b.isInsidePlot(p-h,r-k),b.hasCartesianSeries&&(this.zoomX||this.zoomY)&&n&&!t&&!v&&(this.selectionMarker=v=b.renderer.rect(h,k,f?1:C,g?1:l,0).attr({fill:c.selectionMarkerFill||B("#335cad").setOpacity(.25).get(),"class":"highcharts-selection-marker",zIndex:7}).add()),v&&f&&(e-=p,v.attr({width:Math.abs(e),x:(0<e?0:e)+p})),v&&g&&(e=m-r,v.attr({height:Math.abs(e),
y:(0<e?0:e)+r})),n&&!v&&c.panning&&b.pan(a,c.panning)))},drop:function(a){var b=this,c=this.chart,e=this.hasPinched;if(this.selectionMarker){var m={originalEvent:a,xAxis:[],yAxis:[]},n=this.selectionMarker,v=n.attr?n.attr("x"):n.x,t=n.attr?n.attr("y"):n.y,k=n.attr?n.attr("width"):n.width,C=n.attr?n.attr("height"):n.height,I;if(this.hasDragged||e)r(c.axes,function(c){if(c.zoomEnabled&&g(c.min)&&(e||b[{xAxis:"zoomX",yAxis:"zoomY"}[c.coll]])){var d=c.horiz,f="touchend"===a.type?c.minPixelPadding:0,q=
c.toValue((d?v:t)+f),d=c.toValue((d?v+k:t+C)-f);m[c.coll].push({axis:c,min:Math.min(q,d),max:Math.max(q,d)});I=!0}}),I&&f(c,"selection",m,function(a){c.zoom(l(a,e?{animation:!1}:null))});this.selectionMarker=this.selectionMarker.destroy();e&&this.scaleGroups()}c&&(h(c.container,{cursor:c._cursor}),c.cancelClick=10<this.hasDragged,c.mouseIsDown=this.hasDragged=this.hasPinched=!1,this.pinchDown=[])},onContainerMouseDown:function(a){a=this.normalize(a);this.zoomOption(a);a.preventDefault&&a.preventDefault();
this.dragStart(a)},onDocumentMouseUp:function(b){A[a.hoverChartIndex]&&A[a.hoverChartIndex].pointer.drop(b)},onDocumentMouseMove:function(a){var b=this.chart,c=this.chartPosition;a=this.normalize(a,c);!c||this.inClass(a.target,"highcharts-tracker")||b.isInsidePlot(a.chartX-b.plotLeft,a.chartY-b.plotTop)||this.reset()},onContainerMouseLeave:function(b){var c=A[a.hoverChartIndex];c&&(b.relatedTarget||b.toElement)&&(c.pointer.reset(),c.pointer.chartPosition=null)},onContainerMouseMove:function(b){var c=
this.chart;g(a.hoverChartIndex)&&A[a.hoverChartIndex]&&A[a.hoverChartIndex].mouseIsDown||(a.hoverChartIndex=c.index);b=this.normalize(b);b.returnValue=!1;"mousedown"===c.mouseIsDown&&this.drag(b);!this.inClass(b.target,"highcharts-tracker")&&!c.isInsidePlot(b.chartX-c.plotLeft,b.chartY-c.plotTop)||c.openMenu||this.runPointActions(b)},inClass:function(a,b){for(var c;a;){if(c=E(a,"class")){if(-1!==c.indexOf(b))return!0;if(-1!==c.indexOf("highcharts-container"))return!1}a=a.parentNode}},onTrackerMouseOut:function(a){var b=
this.chart.hoverSeries;a=a.relatedTarget||a.toElement;if(!(!b||!a||b.options.stickyTracking||this.inClass(a,"highcharts-tooltip")||this.inClass(a,"highcharts-series-"+b.index)&&this.inClass(a,"highcharts-tracker")))b.onMouseOut()},onContainerClick:function(a){var b=this.chart,c=b.hoverPoint,e=b.plotLeft,m=b.plotTop;a=this.normalize(a);b.cancelClick||(c&&this.inClass(a.target,"highcharts-tracker")?(f(c.series,"click",l(a,{point:c})),b.hoverPoint&&c.firePointEvent("click",a)):(l(a,this.getCoordinates(a)),
b.isInsidePlot(a.chartX-e,a.chartY-m)&&f(b,"click",a)))},setDOMEvents:function(){var b=this,d=b.chart.container;d.onmousedown=function(a){b.onContainerMouseDown(a)};d.onmousemove=function(a){b.onContainerMouseMove(a)};d.onclick=function(a){b.onContainerClick(a)};y(d,"mouseleave",b.onContainerMouseLeave);1===a.chartCount&&y(t,"mouseup",b.onDocumentMouseUp);a.hasTouch&&(d.ontouchstart=function(a){b.onContainerTouchStart(a)},d.ontouchmove=function(a){b.onContainerTouchMove(a)},1===a.chartCount&&y(t,
"touchend",b.onDocumentTouchEnd))},destroy:function(){var b;z(this.chart.container,"mouseleave",this.onContainerMouseLeave);a.chartCount||(z(t,"mouseup",this.onDocumentMouseUp),z(t,"touchend",this.onDocumentTouchEnd));clearInterval(this.tooltipTimeout);for(b in this)this[b]=null}}})(J);(function(a){var y=a.charts,E=a.each,A=a.extend,B=a.map,h=a.noop,g=a.pick;A(a.Pointer.prototype,{pinchTranslate:function(a,g,h,f,e,n){this.zoomHor&&this.pinchTranslateDirection(!0,a,g,h,f,e,n);this.zoomVert&&this.pinchTranslateDirection(!1,
a,g,h,f,e,n)},pinchTranslateDirection:function(a,g,h,f,e,n,z,v){var b=this.chart,m=a?"x":"y",c=a?"X":"Y",d="chart"+c,l=a?"width":"height",u=b["plot"+(a?"Left":"Top")],q,x,G=v||1,r=b.inverted,k=b.bounds[a?"h":"v"],C=1===g.length,I=g[0][d],t=h[0][d],K=!C&&g[1][d],p=!C&&h[1][d],F;h=function(){!C&&20<Math.abs(I-K)&&(G=v||Math.abs(t-p)/Math.abs(I-K));x=(u-t)/G+I;q=b["plot"+(a?"Width":"Height")]/G};h();g=x;g<k.min?(g=k.min,F=!0):g+q>k.max&&(g=k.max-q,F=!0);F?(t-=.8*(t-z[m][0]),C||(p-=.8*(p-z[m][1])),h()):
z[m]=[t,p];r||(n[m]=x-u,n[l]=q);n=r?1/G:G;e[l]=q;e[m]=g;f[r?a?"scaleY":"scaleX":"scale"+c]=G;f["translate"+c]=n*u+(t-n*I)},pinch:function(a){var r=this,l=r.chart,f=r.pinchDown,e=a.touches,n=e.length,t=r.lastValidTouch,v=r.hasZoom,b=r.selectionMarker,m={},c=1===n&&(r.inClass(a.target,"highcharts-tracker")&&l.runTrackerClick||r.runChartClick),d={};1<n&&(r.initiated=!0);v&&r.initiated&&!c&&a.preventDefault();B(e,function(a){return r.normalize(a)});"touchstart"===a.type?(E(e,function(a,b){f[b]={chartX:a.chartX,
chartY:a.chartY}}),t.x=[f[0].chartX,f[1]&&f[1].chartX],t.y=[f[0].chartY,f[1]&&f[1].chartY],E(l.axes,function(a){if(a.zoomEnabled){var b=l.bounds[a.horiz?"h":"v"],c=a.minPixelPadding,d=a.toPixels(g(a.options.min,a.dataMin)),e=a.toPixels(g(a.options.max,a.dataMax)),m=Math.max(d,e);b.min=Math.min(a.pos,Math.min(d,e)-c);b.max=Math.max(a.pos+a.len,m+c)}}),r.res=!0):r.followTouchMove&&1===n?this.runPointActions(r.normalize(a)):f.length&&(b||(r.selectionMarker=b=A({destroy:h,touch:!0},l.plotBox)),r.pinchTranslate(f,
e,m,b,d,t),r.hasPinched=v,r.scaleGroups(m,d),r.res&&(r.res=!1,this.reset(!1,0)))},touch:function(h,r){var l=this.chart,f,e;if(l.index!==a.hoverChartIndex)this.onContainerMouseLeave({relatedTarget:!0});a.hoverChartIndex=l.index;1===h.touches.length?(h=this.normalize(h),(e=l.isInsidePlot(h.chartX-l.plotLeft,h.chartY-l.plotTop))&&!l.openMenu?(r&&this.runPointActions(h),"touchmove"===h.type&&(r=this.pinchDown,f=r[0]?4<=Math.sqrt(Math.pow(r[0].chartX-h.chartX,2)+Math.pow(r[0].chartY-h.chartY,2)):!1),g(f,
!0)&&this.pinch(h)):r&&this.reset()):2===h.touches.length&&this.pinch(h)},onContainerTouchStart:function(a){this.zoomOption(a);this.touch(a,!0)},onContainerTouchMove:function(a){this.touch(a)},onDocumentTouchEnd:function(g){y[a.hoverChartIndex]&&y[a.hoverChartIndex].pointer.drop(g)}})})(J);(function(a){var y=a.addEvent,E=a.charts,A=a.css,B=a.doc,h=a.extend,g=a.noop,t=a.Pointer,r=a.removeEvent,l=a.win,f=a.wrap;if(l.PointerEvent||l.MSPointerEvent){var e={},n=!!l.PointerEvent,z=function(){var a,m=[];
m.item=function(a){return this[a]};for(a in e)e.hasOwnProperty(a)&&m.push({pageX:e[a].pageX,pageY:e[a].pageY,target:e[a].target});return m},v=function(b,e,c,d){"touch"!==b.pointerType&&b.pointerType!==b.MSPOINTER_TYPE_TOUCH||!E[a.hoverChartIndex]||(d(b),d=E[a.hoverChartIndex].pointer,d[e]({type:c,target:b.currentTarget,preventDefault:g,touches:z()}))};h(t.prototype,{onContainerPointerDown:function(a){v(a,"onContainerTouchStart","touchstart",function(a){e[a.pointerId]={pageX:a.pageX,pageY:a.pageY,
target:a.currentTarget}})},onContainerPointerMove:function(a){v(a,"onContainerTouchMove","touchmove",function(a){e[a.pointerId]={pageX:a.pageX,pageY:a.pageY};e[a.pointerId].target||(e[a.pointerId].target=a.currentTarget)})},onDocumentPointerUp:function(a){v(a,"onDocumentTouchEnd","touchend",function(a){delete e[a.pointerId]})},batchMSEvents:function(a){a(this.chart.container,n?"pointerdown":"MSPointerDown",this.onContainerPointerDown);a(this.chart.container,n?"pointermove":"MSPointerMove",this.onContainerPointerMove);
a(B,n?"pointerup":"MSPointerUp",this.onDocumentPointerUp)}});f(t.prototype,"init",function(a,e,c){a.call(this,e,c);this.hasZoom&&A(e.container,{"-ms-touch-action":"none","touch-action":"none"})});f(t.prototype,"setDOMEvents",function(a){a.apply(this);(this.hasZoom||this.followTouchMove)&&this.batchMSEvents(y)});f(t.prototype,"destroy",function(a){this.batchMSEvents(r);a.call(this)})}})(J);(function(a){var y,E=a.addEvent,A=a.css,B=a.discardElement,h=a.defined,g=a.each,t=a.isFirefox,r=a.marginNames,
l=a.merge,f=a.pick,e=a.setAnimation,n=a.stableSort,z=a.win,v=a.wrap;y=a.Legend=function(a,e){this.init(a,e)};y.prototype={init:function(a,e){this.chart=a;this.setOptions(e);e.enabled&&(this.render(),E(this.chart,"endResize",function(){this.legend.positionCheckboxes()}))},setOptions:function(a){var b=f(a.padding,8);this.options=a;this.itemStyle=a.itemStyle;this.itemHiddenStyle=l(this.itemStyle,a.itemHiddenStyle);this.itemMarginTop=a.itemMarginTop||0;this.initialItemX=this.padding=b;this.initialItemY=
b-5;this.itemHeight=this.maxItemWidth=0;this.symbolWidth=f(a.symbolWidth,16);this.pages=[]},update:function(a,e){var b=this.chart;this.setOptions(l(!0,this.options,a));this.destroy();b.isDirtyLegend=b.isDirtyBox=!0;f(e,!0)&&b.redraw()},colorizeItem:function(a,e){a.legendGroup[e?"removeClass":"addClass"]("highcharts-legend-item-hidden");var b=this.options,d=a.legendItem,f=a.legendLine,m=a.legendSymbol,g=this.itemHiddenStyle.color,b=e?b.itemStyle.color:g,h=e?a.color||g:g,n=a.options&&a.options.marker,
l={fill:h},k;d&&d.css({fill:b,color:b});f&&f.attr({stroke:h});if(m){if(n&&m.isMarker&&(l=a.pointAttribs(),!e))for(k in l)l[k]=g;m.attr(l)}},positionItem:function(a){var b=this.options,c=b.symbolPadding,b=!b.rtl,d=a._legendItemPos,e=d[0],d=d[1],f=a.checkbox;(a=a.legendGroup)&&a.element&&a.translate(b?e:this.legendWidth-e-2*c-4,d);f&&(f.x=e,f.y=d)},destroyItem:function(a){var b=a.checkbox;g(["legendItem","legendLine","legendSymbol","legendGroup"],function(b){a[b]&&(a[b]=a[b].destroy())});b&&B(a.checkbox)},
destroy:function(){function a(a){this[a]&&(this[a]=this[a].destroy())}g(this.getAllItems(),function(b){g(["legendItem","legendGroup"],a,b)});g("clipRect up down pager nav box title group".split(" "),a,this);this.display=null},positionCheckboxes:function(a){var b=this.group&&this.group.alignAttr,c,d=this.clipHeight||this.legendHeight,e=this.titleHeight;b&&(c=b.translateY,g(this.allItems,function(f){var m=f.checkbox,g;m&&(g=c+e+m.y+(a||0)+3,A(m,{left:b.translateX+f.checkboxOffset+m.x-20+"px",top:g+
"px",display:g>c-6&&g<c+d-6?"":"none"}))}))},renderTitle:function(){var a=this.padding,e=this.options.title,c=0;e.text&&(this.title||(this.title=this.chart.renderer.label(e.text,a-3,a-4,null,null,null,null,null,"legend-title").attr({zIndex:1}).css(e.style).add(this.group)),a=this.title.getBBox(),c=a.height,this.offsetWidth=a.width,this.contentGroup.attr({translateY:c}));this.titleHeight=c},setText:function(b){var e=this.options;b.legendItem.attr({text:e.labelFormat?a.format(e.labelFormat,b):e.labelFormatter.call(b)})},
renderItem:function(a){var b=this.chart,c=b.renderer,d=this.options,e="horizontal"===d.layout,g=this.symbolWidth,h=d.symbolPadding,n=this.itemStyle,v=this.itemHiddenStyle,r=this.padding,k=e?f(d.itemDistance,20):0,C=!d.rtl,I=d.width,z=d.itemMarginBottom||0,K=this.itemMarginTop,p=this.initialItemX,t=a.legendItem,M=!a.series,B=!M&&a.series.drawLegendSymbol?a.series:a,y=B.options,y=this.createCheckboxForItem&&y&&y.showCheckbox,A=d.useHTML;t||(a.legendGroup=c.g("legend-item").addClass("highcharts-"+B.type+
"-series highcharts-color-"+a.colorIndex+(a.options.className?" "+a.options.className:"")+(M?" highcharts-series-"+a.index:"")).attr({zIndex:1}).add(this.scrollGroup),a.legendItem=t=c.text("",C?g+h:-h,this.baseline||0,A).css(l(a.visible?n:v)).attr({align:C?"left":"right",zIndex:2}).add(a.legendGroup),this.baseline||(n=n.fontSize,this.fontMetrics=c.fontMetrics(n,t),this.baseline=this.fontMetrics.f+3+K,t.attr("y",this.baseline)),this.symbolHeight=d.symbolHeight||this.fontMetrics.f,B.drawLegendSymbol(this,
a),this.setItemEvents&&this.setItemEvents(a,t,A),y&&this.createCheckboxForItem(a));this.colorizeItem(a,a.visible);this.setText(a);c=t.getBBox();g=a.checkboxOffset=d.itemWidth||a.legendItemWidth||g+h+c.width+k+(y?20:0);this.itemHeight=h=Math.round(a.legendItemHeight||c.height);e&&this.itemX-p+g>(I||b.chartWidth-2*r-p-d.x)&&(this.itemX=p,this.itemY+=K+this.lastLineHeight+z,this.lastLineHeight=0);this.maxItemWidth=Math.max(this.maxItemWidth,g);this.lastItemY=K+this.itemY+z;this.lastLineHeight=Math.max(h,
this.lastLineHeight);a._legendItemPos=[this.itemX,this.itemY];e?this.itemX+=g:(this.itemY+=K+h+z,this.lastLineHeight=h);this.offsetWidth=I||Math.max((e?this.itemX-p-k:g)+r,this.offsetWidth)},getAllItems:function(){var a=[];g(this.chart.series,function(b){var c=b&&b.options;b&&f(c.showInLegend,h(c.linkedTo)?!1:void 0,!0)&&(a=a.concat(b.legendItems||("point"===c.legendType?b.data:b)))});return a},adjustMargins:function(a,e){var b=this.chart,d=this.options,m=d.align.charAt(0)+d.verticalAlign.charAt(0)+
d.layout.charAt(0);d.floating||g([/(lth|ct|rth)/,/(rtv|rm|rbv)/,/(rbh|cb|lbh)/,/(lbv|lm|ltv)/],function(c,g){c.test(m)&&!h(a[g])&&(b[r[g]]=Math.max(b[r[g]],b.legend[(g+1)%2?"legendHeight":"legendWidth"]+[1,-1,-1,1][g]*d[g%2?"x":"y"]+f(d.margin,12)+e[g]))})},render:function(){var a=this,e=a.chart,c=e.renderer,d=a.group,f,h,q,x,v=a.box,r=a.options,k=a.padding;a.itemX=a.initialItemX;a.itemY=a.initialItemY;a.offsetWidth=0;a.lastItemY=0;d||(a.group=d=c.g("legend").attr({zIndex:7}).add(),a.contentGroup=
c.g().attr({zIndex:1}).add(d),a.scrollGroup=c.g().add(a.contentGroup));a.renderTitle();f=a.getAllItems();n(f,function(a,b){return(a.options&&a.options.legendIndex||0)-(b.options&&b.options.legendIndex||0)});r.reversed&&f.reverse();a.allItems=f;a.display=h=!!f.length;a.lastLineHeight=0;g(f,function(b){a.renderItem(b)});q=(r.width||a.offsetWidth)+k;x=a.lastItemY+a.lastLineHeight+a.titleHeight;x=a.handleOverflow(x);x+=k;v||(a.box=v=c.rect().addClass("highcharts-legend-box").attr({r:r.borderRadius}).add(d),
v.isNew=!0);v.attr({stroke:r.borderColor,"stroke-width":r.borderWidth||0,fill:r.backgroundColor||"none"}).shadow(r.shadow);0<q&&0<x&&(v[v.isNew?"attr":"animate"](v.crisp({x:0,y:0,width:q,height:x},v.strokeWidth())),v.isNew=!1);v[h?"show":"hide"]();a.legendWidth=q;a.legendHeight=x;g(f,function(b){a.positionItem(b)});h&&d.align(l(r,{width:q,height:x}),!0,"spacingBox");e.isResizing||this.positionCheckboxes()},handleOverflow:function(a){var b=this,c=this.chart,d=c.renderer,e=this.options,h=e.y,c=c.spacingBox.height+
("top"===e.verticalAlign?-h:h)-this.padding,h=e.maxHeight,q,n=this.clipRect,l=e.navigation,v=f(l.animation,!0),k=l.arrowSize||12,C=this.nav,I=this.pages,r=this.padding,K,p=this.allItems,t=function(a){a?n.attr({height:a}):n&&(b.clipRect=n.destroy(),b.contentGroup.clip());b.contentGroup.div&&(b.contentGroup.div.style.clip=a?"rect("+r+"px,9999px,"+(r+a)+"px,0)":"auto")};"horizontal"!==e.layout||"middle"===e.verticalAlign||e.floating||(c/=2);h&&(c=Math.min(c,h));I.length=0;a>c&&!1!==l.enabled?(this.clipHeight=
q=Math.max(c-20-this.titleHeight-r,0),this.currentPage=f(this.currentPage,1),this.fullHeight=a,g(p,function(a,b){var c=a._legendItemPos[1];a=Math.round(a.legendItem.getBBox().height);var d=I.length;if(!d||c-I[d-1]>q&&(K||c)!==I[d-1])I.push(K||c),d++;b===p.length-1&&c+a-I[d-1]>q&&I.push(c);c!==K&&(K=c)}),n||(n=b.clipRect=d.clipRect(0,r,9999,0),b.contentGroup.clip(n)),t(q),C||(this.nav=C=d.g().attr({zIndex:1}).add(this.group),this.up=d.symbol("triangle",0,0,k,k).on("click",function(){b.scroll(-1,v)}).add(C),
this.pager=d.text("",15,10).addClass("highcharts-legend-navigation").css(l.style).add(C),this.down=d.symbol("triangle-down",0,0,k,k).on("click",function(){b.scroll(1,v)}).add(C)),b.scroll(0),a=c):C&&(t(),this.nav=C.destroy(),this.scrollGroup.attr({translateY:1}),this.clipHeight=0);return a},scroll:function(a,f){var b=this.pages,d=b.length;a=this.currentPage+a;var m=this.clipHeight,g=this.options.navigation,h=this.pager,n=this.padding;a>d&&(a=d);0<a&&(void 0!==f&&e(f,this.chart),this.nav.attr({translateX:n,
translateY:m+this.padding+7+this.titleHeight,visibility:"visible"}),this.up.attr({"class":1===a?"highcharts-legend-nav-inactive":"highcharts-legend-nav-active"}),h.attr({text:a+"/"+d}),this.down.attr({x:18+this.pager.getBBox().width,"class":a===d?"highcharts-legend-nav-inactive":"highcharts-legend-nav-active"}),this.up.attr({fill:1===a?g.inactiveColor:g.activeColor}).css({cursor:1===a?"default":"pointer"}),this.down.attr({fill:a===d?g.inactiveColor:g.activeColor}).css({cursor:a===d?"default":"pointer"}),
f=-b[a-1]+this.initialItemY,this.scrollGroup.animate({translateY:f}),this.currentPage=a,this.positionCheckboxes(f))}};a.LegendSymbolMixin={drawRectangle:function(a,e){var b=a.symbolHeight,d=a.options.squareSymbol;e.legendSymbol=this.chart.renderer.rect(d?(a.symbolWidth-b)/2:0,a.baseline-b+1,d?b:a.symbolWidth,b,f(a.options.symbolRadius,b/2)).addClass("highcharts-point").attr({zIndex:3}).add(e.legendGroup)},drawLineMarker:function(a){var b=this.options,c=b.marker,d=a.symbolWidth,e=a.symbolHeight,g=
e/2,h=this.chart.renderer,n=this.legendGroup;a=a.baseline-Math.round(.3*a.fontMetrics.b);var v;v={"stroke-width":b.lineWidth||0};b.dashStyle&&(v.dashstyle=b.dashStyle);this.legendLine=h.path(["M",0,a,"L",d,a]).addClass("highcharts-graph").attr(v).add(n);c&&!1!==c.enabled&&(b=Math.min(f(c.radius,g),g),0===this.symbol.indexOf("url")&&(c=l(c,{width:e,height:e}),b=0),this.legendSymbol=c=h.symbol(this.symbol,d/2-b,a-b,2*b,2*b,c).addClass("highcharts-point").add(n),c.isMarker=!0)}};(/Trident\/7\.0/.test(z.navigator.userAgent)||
t)&&v(y.prototype,"positionItem",function(a,e){var b=this,d=function(){e._legendItemPos&&a.call(b,e)};d();setTimeout(d)})})(J);(function(a){var y=a.addEvent,E=a.animate,A=a.animObject,B=a.attr,h=a.doc,g=a.Axis,t=a.createElement,r=a.defaultOptions,l=a.discardElement,f=a.charts,e=a.css,n=a.defined,z=a.each,v=a.extend,b=a.find,m=a.fireEvent,c=a.getStyle,d=a.grep,D=a.isNumber,u=a.isObject,q=a.isString,x=a.Legend,G=a.marginNames,H=a.merge,k=a.Pointer,C=a.pick,I=a.pInt,L=a.removeEvent,K=a.seriesTypes,p=
a.splat,F=a.svg,M=a.syncTimeout,N=a.win,S=a.Renderer,O=a.Chart=function(){this.getArgs.apply(this,arguments)};a.chart=function(a,b,c){return new O(a,b,c)};O.prototype={callbacks:[],getArgs:function(){var a=[].slice.call(arguments);if(q(a[0])||a[0].nodeName)this.renderTo=a.shift();this.init(a[0],a[1])},init:function(b,c){var d,e=b.series;b.series=null;d=H(r,b);d.series=b.series=e;this.userOptions=b;b=d.chart;e=b.events;this.margin=[];this.spacing=[];this.bounds={h:{},v:{}};this.callback=c;this.isResizing=
0;this.options=d;this.axes=[];this.series=[];this.hasCartesianSeries=b.showAxes;var k;this.index=f.length;f.push(this);a.chartCount++;if(e)for(k in e)y(this,k,e[k]);this.xAxis=[];this.yAxis=[];this.pointCount=this.colorCounter=this.symbolCounter=0;this.firstRender()},initSeries:function(b){var c=this.options.chart;(c=K[b.type||c.type||c.defaultSeriesType])||a.error(17,!0);c=new c;c.init(this,b);return c},orderSeries:function(a){var b=this.series;for(a=a||0;a<b.length;a++)b[a]&&(b[a].index=a,b[a].name=
b[a].name||"Series "+(b[a].index+1))},isInsidePlot:function(a,b,c){var d=c?b:a;a=c?a:b;return 0<=d&&d<=this.plotWidth&&0<=a&&a<=this.plotHeight},redraw:function(b){var c=this.axes,d=this.series,e=this.pointer,k=this.legend,f=this.isDirtyLegend,g,p,h=this.hasCartesianSeries,q=this.isDirtyBox,n,l=this.renderer,C=l.isHidden(),u=[];this.setResponsive&&this.setResponsive(!1);a.setAnimation(b,this);C&&this.cloneRenderTo();this.layOutTitles();for(b=d.length;b--;)if(n=d[b],n.options.stacking&&(g=!0,n.isDirty)){p=
!0;break}if(p)for(b=d.length;b--;)n=d[b],n.options.stacking&&(n.isDirty=!0);z(d,function(a){a.isDirty&&"point"===a.options.legendType&&(a.updateTotals&&a.updateTotals(),f=!0);a.isDirtyData&&m(a,"updatedData")});f&&k.options.enabled&&(k.render(),this.isDirtyLegend=!1);g&&this.getStacks();h&&z(c,function(a){a.updateNames();a.setScale()});this.getMargins();h&&(z(c,function(a){a.isDirty&&(q=!0)}),z(c,function(a){var b=a.min+","+a.max;a.extKey!==b&&(a.extKey=b,u.push(function(){m(a,"afterSetExtremes",
v(a.eventArgs,a.getExtremes()));delete a.eventArgs}));(q||g)&&a.redraw()}));q&&this.drawChartBox();m(this,"predraw");z(d,function(a){(q||a.isDirty)&&a.visible&&a.redraw();a.isDirtyData=!1});e&&e.reset(!0);l.draw();m(this,"redraw");m(this,"render");C&&this.cloneRenderTo(!0);z(u,function(a){a.call()})},get:function(a){function c(b){return b.id===a||b.options&&b.options.id===a}var d,e=this.series,k;d=b(this.axes,c)||b(this.series,c);for(k=0;!d&&k<e.length;k++)d=b(e[k].points||[],c);return d},getAxes:function(){var a=
this,b=this.options,c=b.xAxis=p(b.xAxis||{}),b=b.yAxis=p(b.yAxis||{});z(c,function(a,b){a.index=b;a.isX=!0});z(b,function(a,b){a.index=b});c=c.concat(b);z(c,function(b){new g(a,b)})},getSelectedPoints:function(){var a=[];z(this.series,function(b){a=a.concat(d(b.points||[],function(a){return a.selected}))});return a},getSelectedSeries:function(){return d(this.series,function(a){return a.selected})},setTitle:function(a,b,c){var d=this,e=d.options,k;k=e.title=H({style:{color:"#333333",fontSize:e.isStock?
"16px":"18px"}},e.title,a);e=e.subtitle=H({style:{color:"#666666"}},e.subtitle,b);z([["title",a,k],["subtitle",b,e]],function(a,b){var c=a[0],e=d[c],k=a[1];a=a[2];e&&k&&(d[c]=e=e.destroy());a&&a.text&&!e&&(d[c]=d.renderer.text(a.text,0,0,a.useHTML).attr({align:a.align,"class":"highcharts-"+c,zIndex:a.zIndex||4}).add(),d[c].update=function(a){d.setTitle(!b&&a,b&&a)},d[c].css(a.style))});d.layOutTitles(c)},layOutTitles:function(a){var b=0,c,d=this.renderer,e=this.spacingBox;z(["title","subtitle"],function(a){var c=
this[a],k=this.options[a],f;c&&(f=k.style.fontSize,f=d.fontMetrics(f,c).b,c.css({width:(k.width||e.width+k.widthAdjust)+"px"}).align(v({y:b+f+("title"===a?-3:2)},k),!1,"spacingBox"),k.floating||k.verticalAlign||(b=Math.ceil(b+c.getBBox().height)))},this);c=this.titleOffset!==b;this.titleOffset=b;!this.isDirtyBox&&c&&(this.isDirtyBox=c,this.hasRendered&&C(a,!0)&&this.isDirtyBox&&this.redraw())},getChartSize:function(){var b=this.options.chart,d=b.width,b=b.height,e=this.renderToClone||this.renderTo;
n(d)||(this.containerWidth=c(e,"width"));n(b)||(this.containerHeight=c(e,"height"));this.chartWidth=Math.max(0,d||this.containerWidth||600);this.chartHeight=Math.max(0,a.relativeLength(b,this.chartWidth)||this.containerHeight||400)},cloneRenderTo:function(a){var b=this.renderToClone,c=this.container;if(a){if(b){for(;b.childNodes.length;)this.renderTo.appendChild(b.firstChild);l(b);delete this.renderToClone}}else c&&c.parentNode===this.renderTo&&this.renderTo.removeChild(c),this.renderToClone=b=this.renderTo.cloneNode(0),
e(b,{position:"absolute",top:"-9999px",display:"block"}),b.style.setProperty&&b.style.setProperty("display","block","important"),h.body.appendChild(b),c&&b.appendChild(c)},setClassName:function(a){this.container.className="highcharts-container "+(a||"")},getContainer:function(){var b,c=this.options,d=c.chart,e,k;b=this.renderTo;var g=a.uniqueKey(),m;b||(this.renderTo=b=d.renderTo);q(b)&&(this.renderTo=b=h.getElementById(b));b||a.error(13,!0);e=I(B(b,"data-highcharts-chart"));D(e)&&f[e]&&f[e].hasRendered&&
f[e].destroy();B(b,"data-highcharts-chart",this.index);b.innerHTML="";d.skipClone||b.offsetWidth||this.cloneRenderTo();this.getChartSize();e=this.chartWidth;k=this.chartHeight;m=v({position:"relative",overflow:"hidden",width:e+"px",height:k+"px",textAlign:"left",lineHeight:"normal",zIndex:0,"-webkit-tap-highlight-color":"rgba(0,0,0,0)"},d.style);this.container=b=t("div",{id:g},m,this.renderToClone||b);this._cursor=b.style.cursor;this.renderer=new (a[d.renderer]||S)(b,e,k,null,d.forExport,c.exporting&&
c.exporting.allowHTML);this.setClassName(d.className);this.renderer.setStyle(d.style);this.renderer.chartIndex=this.index},getMargins:function(a){var b=this.spacing,c=this.margin,d=this.titleOffset;this.resetMargins();d&&!n(c[0])&&(this.plotTop=Math.max(this.plotTop,d+this.options.title.margin+b[0]));this.legend.display&&this.legend.adjustMargins(c,b);this.extraMargin&&(this[this.extraMargin.type]=(this[this.extraMargin.type]||0)+this.extraMargin.value);this.extraTopMargin&&(this.plotTop+=this.extraTopMargin);
a||this.getAxisMargins()},getAxisMargins:function(){var a=this,b=a.axisOffset=[0,0,0,0],c=a.margin;a.hasCartesianSeries&&z(a.axes,function(a){a.visible&&a.getOffset()});z(G,function(d,e){n(c[e])||(a[d]+=b[e])});a.setChartSize()},reflow:function(a){var b=this,d=b.options.chart,e=b.renderTo,k=n(d.width),f=d.width||c(e,"width"),d=d.height||c(e,"height"),e=a?a.target:N;if(!k&&!b.isPrinting&&f&&d&&(e===N||e===h)){if(f!==b.containerWidth||d!==b.containerHeight)clearTimeout(b.reflowTimeout),b.reflowTimeout=
M(function(){b.container&&b.setSize(void 0,void 0,!1)},a?100:0);b.containerWidth=f;b.containerHeight=d}},initReflow:function(){var a=this,b;b=y(N,"resize",function(b){a.reflow(b)});y(a,"destroy",b)},setSize:function(b,c,d){var k=this,f=k.renderer;k.isResizing+=1;a.setAnimation(d,k);k.oldChartHeight=k.chartHeight;k.oldChartWidth=k.chartWidth;void 0!==b&&(k.options.chart.width=b);void 0!==c&&(k.options.chart.height=c);k.getChartSize();b=f.globalAnimation;(b?E:e)(k.container,{width:k.chartWidth+"px",
height:k.chartHeight+"px"},b);k.setChartSize(!0);f.setSize(k.chartWidth,k.chartHeight,d);z(k.axes,function(a){a.isDirty=!0;a.setScale()});k.isDirtyLegend=!0;k.isDirtyBox=!0;k.layOutTitles();k.getMargins();k.redraw(d);k.oldChartHeight=null;m(k,"resize");M(function(){k&&m(k,"endResize",null,function(){--k.isResizing})},A(b).duration)},setChartSize:function(a){var b=this.inverted,c=this.renderer,d=this.chartWidth,e=this.chartHeight,k=this.options.chart,f=this.spacing,g=this.clipOffset,m,p,h,q;this.plotLeft=
m=Math.round(this.plotLeft);this.plotTop=p=Math.round(this.plotTop);this.plotWidth=h=Math.max(0,Math.round(d-m-this.marginRight));this.plotHeight=q=Math.max(0,Math.round(e-p-this.marginBottom));this.plotSizeX=b?q:h;this.plotSizeY=b?h:q;this.plotBorderWidth=k.plotBorderWidth||0;this.spacingBox=c.spacingBox={x:f[3],y:f[0],width:d-f[3]-f[1],height:e-f[0]-f[2]};this.plotBox=c.plotBox={x:m,y:p,width:h,height:q};d=2*Math.floor(this.plotBorderWidth/2);b=Math.ceil(Math.max(d,g[3])/2);c=Math.ceil(Math.max(d,
g[0])/2);this.clipBox={x:b,y:c,width:Math.floor(this.plotSizeX-Math.max(d,g[1])/2-b),height:Math.max(0,Math.floor(this.plotSizeY-Math.max(d,g[2])/2-c))};a||z(this.axes,function(a){a.setAxisSize();a.setAxisTranslation()})},resetMargins:function(){var a=this,b=a.options.chart;z(["margin","spacing"],function(c){var d=b[c],e=u(d)?d:[d,d,d,d];z(["Top","Right","Bottom","Left"],function(d,k){a[c][k]=C(b[c+d],e[k])})});z(G,function(b,c){a[b]=C(a.margin[c],a.spacing[c])});a.axisOffset=[0,0,0,0];a.clipOffset=
[0,0,0,0]},drawChartBox:function(){var a=this.options.chart,b=this.renderer,c=this.chartWidth,d=this.chartHeight,e=this.chartBackground,k=this.plotBackground,f=this.plotBorder,g,m=this.plotBGImage,p=a.backgroundColor,h=a.plotBackgroundColor,q=a.plotBackgroundImage,n,l=this.plotLeft,C=this.plotTop,u=this.plotWidth,x=this.plotHeight,v=this.plotBox,I=this.clipRect,r=this.clipBox,K="animate";e||(this.chartBackground=e=b.rect().addClass("highcharts-background").add(),K="attr");g=a.borderWidth||0;n=g+(a.shadow?
8:0);p={fill:p||"none"};if(g||e["stroke-width"])p.stroke=a.borderColor,p["stroke-width"]=g;e.attr(p).shadow(a.shadow);e[K]({x:n/2,y:n/2,width:c-n-g%2,height:d-n-g%2,r:a.borderRadius});K="animate";k||(K="attr",this.plotBackground=k=b.rect().addClass("highcharts-plot-background").add());k[K](v);k.attr({fill:h||"none"}).shadow(a.plotShadow);q&&(m?m.animate(v):this.plotBGImage=b.image(q,l,C,u,x).add());I?I.animate({width:r.width,height:r.height}):this.clipRect=b.clipRect(r);K="animate";f||(K="attr",this.plotBorder=
f=b.rect().addClass("highcharts-plot-border").attr({zIndex:1}).add());f.attr({stroke:a.plotBorderColor,"stroke-width":a.plotBorderWidth||0,fill:"none"});f[K](f.crisp({x:l,y:C,width:u,height:x},-f.strokeWidth()));this.isDirtyBox=!1},propFromSeries:function(){var a=this,b=a.options.chart,c,d=a.options.series,e,k;z(["inverted","angular","polar"],function(f){c=K[b.type||b.defaultSeriesType];k=b[f]||c&&c.prototype[f];for(e=d&&d.length;!k&&e--;)(c=K[d[e].type])&&c.prototype[f]&&(k=!0);a[f]=k})},linkSeries:function(){var a=
this,b=a.series;z(b,function(a){a.linkedSeries.length=0});z(b,function(b){var c=b.options.linkedTo;q(c)&&(c=":previous"===c?a.series[b.index-1]:a.get(c))&&c.linkedParent!==b&&(c.linkedSeries.push(b),b.linkedParent=c,b.visible=C(b.options.visible,c.options.visible,b.visible))})},renderSeries:function(){z(this.series,function(a){a.translate();a.render()})},renderLabels:function(){var a=this,b=a.options.labels;b.items&&z(b.items,function(c){var d=v(b.style,c.style),e=I(d.left)+a.plotLeft,k=I(d.top)+
a.plotTop+12;delete d.left;delete d.top;a.renderer.text(c.html,e,k).attr({zIndex:2}).css(d).add()})},render:function(){var a=this.axes,b=this.renderer,c=this.options,d,e,k;this.setTitle();this.legend=new x(this,c.legend);this.getStacks&&this.getStacks();this.getMargins(!0);this.setChartSize();c=this.plotWidth;d=this.plotHeight-=21;z(a,function(a){a.setScale()});this.getAxisMargins();e=1.1<c/this.plotWidth;k=1.05<d/this.plotHeight;if(e||k)z(a,function(a){(a.horiz&&e||!a.horiz&&k)&&a.setTickInterval(!0)}),
this.getMargins();this.drawChartBox();this.hasCartesianSeries&&z(a,function(a){a.visible&&a.render()});this.seriesGroup||(this.seriesGroup=b.g("series-group").attr({zIndex:3}).add());this.renderSeries();this.renderLabels();this.addCredits();this.setResponsive&&this.setResponsive();this.hasRendered=!0},addCredits:function(a){var b=this;a=H(!0,this.options.credits,a);a.enabled&&!this.credits&&(this.credits=this.renderer.text(a.text+(this.mapCredits||""),0,0).addClass("highcharts-credits").on("click",
function(){a.href&&(N.location.href=a.href)}).attr({align:a.position.align,zIndex:8}).css(a.style).add().align(a.position),this.credits.update=function(a){b.credits=b.credits.destroy();b.addCredits(a)})},destroy:function(){var b=this,c=b.axes,d=b.series,e=b.container,k,g=e&&e.parentNode;m(b,"destroy");f[b.index]=void 0;a.chartCount--;b.renderTo.removeAttribute("data-highcharts-chart");L(b);for(k=c.length;k--;)c[k]=c[k].destroy();this.scroller&&this.scroller.destroy&&this.scroller.destroy();for(k=
d.length;k--;)d[k]=d[k].destroy();z("title subtitle chartBackground plotBackground plotBGImage plotBorder seriesGroup clipRect credits pointer rangeSelector legend resetZoomButton tooltip renderer".split(" "),function(a){var c=b[a];c&&c.destroy&&(b[a]=c.destroy())});e&&(e.innerHTML="",L(e),g&&l(e));for(k in b)delete b[k]},isReadyToRender:function(){var a=this;return F||N!=N.top||"complete"===h.readyState?!0:(h.attachEvent("onreadystatechange",function(){h.detachEvent("onreadystatechange",a.firstRender);
"complete"===h.readyState&&a.firstRender()}),!1)},firstRender:function(){var a=this,b=a.options;if(a.isReadyToRender()){a.getContainer();m(a,"init");a.resetMargins();a.setChartSize();a.propFromSeries();a.getAxes();z(b.series||[],function(b){a.initSeries(b)});a.linkSeries();m(a,"beforeRender");k&&(a.pointer=new k(a,b));a.render();if(!a.renderer.imgCount&&a.onload)a.onload();a.cloneRenderTo(!0)}},onload:function(){z([this.callback].concat(this.callbacks),function(a){a&&void 0!==this.index&&a.apply(this,
[this])},this);m(this,"load");m(this,"render");n(this.index)&&!1!==this.options.chart.reflow&&this.initReflow();this.onload=null}}})(J);(function(a){var y,E=a.each,A=a.extend,B=a.erase,h=a.fireEvent,g=a.format,t=a.isArray,r=a.isNumber,l=a.pick,f=a.removeEvent;y=a.Point=function(){};y.prototype={init:function(a,f,g){this.series=a;this.color=a.color;this.applyOptions(f,g);a.options.colorByPoint?(f=a.options.colors||a.chart.options.colors,this.color=this.color||f[a.colorCounter],f=f.length,g=a.colorCounter,
a.colorCounter++,a.colorCounter===f&&(a.colorCounter=0)):g=a.colorIndex;this.colorIndex=l(this.colorIndex,g);a.chart.pointCount++;return this},applyOptions:function(a,f){var e=this.series,g=e.options.pointValKey||e.pointValKey;a=y.prototype.optionsToObject.call(this,a);A(this,a);this.options=this.options?A(this.options,a):a;a.group&&delete this.group;g&&(this.y=this[g]);this.isNull=l(this.isValid&&!this.isValid(),null===this.x||!r(this.y,!0));this.selected&&(this.state="select");"name"in this&&void 0===
f&&e.xAxis&&e.xAxis.hasNames&&(this.x=e.xAxis.nameToX(this));void 0===this.x&&e&&(this.x=void 0===f?e.autoIncrement(this):f);return this},optionsToObject:function(a){var e={},f=this.series,g=f.options.keys,b=g||f.pointArrayMap||["y"],m=b.length,c=0,d=0;if(r(a)||null===a)e[b[0]]=a;else if(t(a))for(!g&&a.length>m&&(f=typeof a[0],"string"===f?e.name=a[0]:"number"===f&&(e.x=a[0]),c++);d<m;)g&&void 0===a[c]||(e[b[d]]=a[c]),c++,d++;else"object"===typeof a&&(e=a,a.dataLabels&&(f._hasPointLabels=!0),a.marker&&
(f._hasPointMarkers=!0));return e},getClassName:function(){return"highcharts-point"+(this.selected?" highcharts-point-select":"")+(this.negative?" highcharts-negative":"")+(this.isNull?" highcharts-null-point":"")+(void 0!==this.colorIndex?" highcharts-color-"+this.colorIndex:"")+(this.options.className?" "+this.options.className:"")+(this.zone&&this.zone.className?" "+this.zone.className.replace("highcharts-negative",""):"")},getZone:function(){var a=this.series,f=a.zones,a=a.zoneAxis||"y",g=0,h;
for(h=f[g];this[a]>=h.value;)h=f[++g];h&&h.color&&!this.options.color&&(this.color=h.color);return h},destroy:function(){var a=this.series.chart,g=a.hoverPoints,h;a.pointCount--;g&&(this.setState(),B(g,this),g.length||(a.hoverPoints=null));if(this===a.hoverPoint)this.onMouseOut();if(this.graphic||this.dataLabel)f(this),this.destroyElements();this.legendItem&&a.legend.destroyItem(this);for(h in this)this[h]=null},destroyElements:function(){for(var a=["graphic","dataLabel","dataLabelUpper","connector",
"shadowGroup"],f,g=6;g--;)f=a[g],this[f]&&(this[f]=this[f].destroy())},getLabelConfig:function(){return{x:this.category,y:this.y,color:this.color,colorIndex:this.colorIndex,key:this.name||this.category,series:this.series,point:this,percentage:this.percentage,total:this.total||this.stackTotal}},tooltipFormatter:function(a){var e=this.series,f=e.tooltipOptions,h=l(f.valueDecimals,""),b=f.valuePrefix||"",m=f.valueSuffix||"";E(e.pointArrayMap||["y"],function(c){c="{point."+c;if(b||m)a=a.replace(c+"}",
b+c+"}"+m);a=a.replace(c+"}",c+":,."+h+"f}")});return g(a,{point:this,series:this.series})},firePointEvent:function(a,f,g){var e=this,b=this.series.options;(b.point.events[a]||e.options&&e.options.events&&e.options.events[a])&&this.importEvents();"click"===a&&b.allowPointSelect&&(g=function(a){e.select&&e.select(null,a.ctrlKey||a.metaKey||a.shiftKey)});h(this,a,f,g)},visible:!0}})(J);(function(a){var y=a.addEvent,E=a.animObject,A=a.arrayMax,B=a.arrayMin,h=a.correctFloat,g=a.Date,t=a.defaultOptions,
r=a.defaultPlotOptions,l=a.defined,f=a.each,e=a.erase,n=a.extend,z=a.fireEvent,v=a.grep,b=a.isArray,m=a.isNumber,c=a.isString,d=a.merge,D=a.pick,u=a.removeEvent,q=a.splat,x=a.SVGElement,G=a.syncTimeout,H=a.win;a.Series=a.seriesType("line",null,{lineWidth:2,allowPointSelect:!1,showCheckbox:!1,animation:{duration:1E3},events:{},marker:{lineWidth:0,lineColor:"#ffffff",radius:4,states:{hover:{animation:{duration:50},enabled:!0,radiusPlus:2,lineWidthPlus:1},select:{fillColor:"#cccccc",lineColor:"#000000",
lineWidth:2}}},point:{events:{}},dataLabels:{align:"center",formatter:function(){return null===this.y?"":a.numberFormat(this.y,-1)},style:{fontSize:"11px",fontWeight:"bold",color:"contrast",textOutline:"1px contrast"},verticalAlign:"bottom",x:0,y:0,padding:5},cropThreshold:300,pointRange:0,softThreshold:!0,states:{hover:{animation:{duration:50},lineWidthPlus:1,marker:{},halo:{size:10,opacity:.25}},select:{marker:{}}},stickyTracking:!0,turboThreshold:1E3},{isCartesian:!0,pointClass:a.Point,sorted:!0,
requireSorting:!0,directTouch:!1,axisTypes:["xAxis","yAxis"],colorCounter:0,parallelArrays:["x","y"],coll:"series",init:function(a,b){var c=this,d,e,k=a.series,g;c.chart=a;c.options=b=c.setOptions(b);c.linkedSeries=[];c.bindAxes();n(c,{name:b.name,state:"",visible:!1!==b.visible,selected:!0===b.selected});e=b.events;for(d in e)y(c,d,e[d]);if(e&&e.click||b.point&&b.point.events&&b.point.events.click||b.allowPointSelect)a.runTrackerClick=!0;c.getColor();c.getSymbol();f(c.parallelArrays,function(a){c[a+
"Data"]=[]});c.setData(b.data,!1);c.isCartesian&&(a.hasCartesianSeries=!0);k.length&&(g=k[k.length-1]);c._i=D(g&&g._i,-1)+1;a.orderSeries(this.insert(k))},insert:function(a){var b=this.options.index,c;if(m(b)){for(c=a.length;c--;)if(b>=D(a[c].options.index,a[c]._i)){a.splice(c+1,0,this);break}-1===c&&a.unshift(this);c+=1}else a.push(this);return D(c,a.length-1)},bindAxes:function(){var b=this,c=b.options,d=b.chart,e;f(b.axisTypes||[],function(k){f(d[k],function(a){e=a.options;if(c[k]===e.index||void 0!==
c[k]&&c[k]===e.id||void 0===c[k]&&0===e.index)b.insert(a.series),b[k]=a,a.isDirty=!0});b[k]||b.optionalAxis===k||a.error(18,!0)})},updateParallelArrays:function(a,b){var c=a.series,d=arguments,e=m(b)?function(d){var e="y"===d&&c.toYData?c.toYData(a):a[d];c[d+"Data"][b]=e}:function(a){Array.prototype[b].apply(c[a+"Data"],Array.prototype.slice.call(d,2))};f(c.parallelArrays,e)},autoIncrement:function(){var a=this.options,b=this.xIncrement,c,d=a.pointIntervalUnit,b=D(b,a.pointStart,0);this.pointInterval=
c=D(this.pointInterval,a.pointInterval,1);d&&(a=new g(b),"day"===d?a=+a[g.hcSetDate](a[g.hcGetDate]()+c):"month"===d?a=+a[g.hcSetMonth](a[g.hcGetMonth]()+c):"year"===d&&(a=+a[g.hcSetFullYear](a[g.hcGetFullYear]()+c)),c=a-b);this.xIncrement=b+c;return b},setOptions:function(a){var b=this.chart,c=b.options.plotOptions,b=b.userOptions||{},e=b.plotOptions||{},k=c[this.type];this.userOptions=a;c=d(k,c.series,a);this.tooltipOptions=d(t.tooltip,t.plotOptions[this.type].tooltip,b.tooltip,e.series&&e.series.tooltip,
e[this.type]&&e[this.type].tooltip,a.tooltip);null===k.marker&&delete c.marker;this.zoneAxis=c.zoneAxis;a=this.zones=(c.zones||[]).slice();!c.negativeColor&&!c.negativeFillColor||c.zones||a.push({value:c[this.zoneAxis+"Threshold"]||c.threshold||0,className:"highcharts-negative",color:c.negativeColor,fillColor:c.negativeFillColor});a.length&&l(a[a.length-1].value)&&a.push({color:this.color,fillColor:this.fillColor});return c},getCyclic:function(a,b,c){var d,e=this.chart,k=this.userOptions,f=a+"Index",
g=a+"Counter",m=c?c.length:D(e.options.chart[a+"Count"],e[a+"Count"]);b||(d=D(k[f],k["_"+f]),l(d)||(e.series.length||(e[g]=0),k["_"+f]=d=e[g]%m,e[g]+=1),c&&(b=c[d]));void 0!==d&&(this[f]=d);this[a]=b},getColor:function(){this.options.colorByPoint?this.options.color=null:this.getCyclic("color",this.options.color||r[this.type].color,this.chart.options.colors)},getSymbol:function(){this.getCyclic("symbol",this.options.marker.symbol,this.chart.options.symbols)},drawLegendSymbol:a.LegendSymbolMixin.drawLineMarker,
setData:function(d,e,g,h){var k=this,p=k.points,q=p&&p.length||0,l,n=k.options,u=k.chart,C=null,x=k.xAxis,v=n.turboThreshold,r=this.xData,t=this.yData,G=(l=k.pointArrayMap)&&l.length;d=d||[];l=d.length;e=D(e,!0);if(!1!==h&&l&&q===l&&!k.cropped&&!k.hasGroupedData&&k.visible)f(d,function(a,b){p[b].update&&a!==n.data[b]&&p[b].update(a,!1,null,!1)});else{k.xIncrement=null;k.colorCounter=0;f(this.parallelArrays,function(a){k[a+"Data"].length=0});if(v&&l>v){for(g=0;null===C&&g<l;)C=d[g],g++;if(m(C))for(g=
0;g<l;g++)r[g]=this.autoIncrement(),t[g]=d[g];else if(b(C))if(G)for(g=0;g<l;g++)C=d[g],r[g]=C[0],t[g]=C.slice(1,G+1);else for(g=0;g<l;g++)C=d[g],r[g]=C[0],t[g]=C[1];else a.error(12)}else for(g=0;g<l;g++)void 0!==d[g]&&(C={series:k},k.pointClass.prototype.applyOptions.apply(C,[d[g]]),k.updateParallelArrays(C,g));c(t[0])&&a.error(14,!0);k.data=[];k.options.data=k.userOptions.data=d;for(g=q;g--;)p[g]&&p[g].destroy&&p[g].destroy();x&&(x.minRange=x.userMinRange);k.isDirty=u.isDirtyBox=!0;k.isDirtyData=
!!p;g=!1}"point"===n.legendType&&(this.processData(),this.generatePoints());e&&u.redraw(g)},processData:function(b){var c=this.xData,d=this.yData,e=c.length,k;k=0;var f,g,m=this.xAxis,h,q=this.options;h=q.cropThreshold;var l=this.getExtremesFromAll||q.getExtremesFromAll,n=this.isCartesian,q=m&&m.val2lin,u=m&&m.isLog,x,v;if(n&&!this.isDirty&&!m.isDirty&&!this.yAxis.isDirty&&!b)return!1;m&&(b=m.getExtremes(),x=b.min,v=b.max);if(n&&this.sorted&&!l&&(!h||e>h||this.forceCrop))if(c[e-1]<x||c[0]>v)c=[],
d=[];else if(c[0]<x||c[e-1]>v)k=this.cropData(this.xData,this.yData,x,v),c=k.xData,d=k.yData,k=k.start,f=!0;for(h=c.length||1;--h;)e=u?q(c[h])-q(c[h-1]):c[h]-c[h-1],0<e&&(void 0===g||e<g)?g=e:0>e&&this.requireSorting&&a.error(15);this.cropped=f;this.cropStart=k;this.processedXData=c;this.processedYData=d;this.closestPointRange=g},cropData:function(a,b,c,d){var e=a.length,k=0,f=e,g=D(this.cropShoulder,1),m;for(m=0;m<e;m++)if(a[m]>=c){k=Math.max(0,m-g);break}for(c=m;c<e;c++)if(a[c]>d){f=c+g;break}return{xData:a.slice(k,
f),yData:b.slice(k,f),start:k,end:f}},generatePoints:function(){var a=this.options.data,b=this.data,c,d=this.processedXData,e=this.processedYData,f=this.pointClass,g=d.length,m=this.cropStart||0,h,l=this.hasGroupedData,n,u=[],x;b||l||(b=[],b.length=a.length,b=this.data=b);for(x=0;x<g;x++)h=m+x,l?(n=(new f).init(this,[d[x]].concat(q(e[x]))),n.dataGroup=this.groupMap[x]):(n=b[h])||void 0===a[h]||(b[h]=n=(new f).init(this,a[h],d[x])),n.index=h,u[x]=n;if(b&&(g!==(c=b.length)||l))for(x=0;x<c;x++)x!==m||
l||(x+=g),b[x]&&(b[x].destroyElements(),b[x].plotX=void 0);this.data=b;this.points=u},getExtremes:function(a){var c=this.yAxis,d=this.processedXData,e,k=[],f=0;e=this.xAxis.getExtremes();var g=e.min,h=e.max,q,l,n,u;a=a||this.stackedYData||this.processedYData||[];e=a.length;for(u=0;u<e;u++)if(l=d[u],n=a[u],q=(m(n,!0)||b(n))&&(!c.positiveValuesOnly||n.length||0<n),l=this.getExtremesFromAll||this.options.getExtremesFromAll||this.cropped||(d[u+1]||l)>=g&&(d[u-1]||l)<=h,q&&l)if(q=n.length)for(;q--;)null!==
n[q]&&(k[f++]=n[q]);else k[f++]=n;this.dataMin=B(k);this.dataMax=A(k)},translate:function(){this.processedXData||this.processData();this.generatePoints();var a=this.options,b=a.stacking,c=this.xAxis,d=c.categories,e=this.yAxis,f=this.points,g=f.length,q=!!this.modifyValue,n=a.pointPlacement,u="between"===n||m(n),x=a.threshold,v=a.startFromThreshold?x:0,r,t,G,H,z=Number.MAX_VALUE;"between"===n&&(n=.5);m(n)&&(n*=D(a.pointRange||c.pointRange));for(a=0;a<g;a++){var B=f[a],y=B.x,A=B.y;t=B.low;var E=b&&
e.stacks[(this.negStacks&&A<(v?0:x)?"-":"")+this.stackKey],J;e.positiveValuesOnly&&null!==A&&0>=A&&(B.isNull=!0);B.plotX=r=h(Math.min(Math.max(-1E5,c.translate(y,0,0,0,1,n,"flags"===this.type)),1E5));b&&this.visible&&!B.isNull&&E&&E[y]&&(H=this.getStackIndicator(H,y,this.index),J=E[y],A=J.points[H.key],t=A[0],A=A[1],t===v&&H.key===E[y].base&&(t=D(x,e.min)),e.positiveValuesOnly&&0>=t&&(t=null),B.total=B.stackTotal=J.total,B.percentage=J.total&&B.y/J.total*100,B.stackY=A,J.setOffset(this.pointXOffset||
0,this.barW||0));B.yBottom=l(t)?e.translate(t,0,1,0,1):null;q&&(A=this.modifyValue(A,B));B.plotY=t="number"===typeof A&&Infinity!==A?Math.min(Math.max(-1E5,e.translate(A,0,1,0,1)),1E5):void 0;B.isInside=void 0!==t&&0<=t&&t<=e.len&&0<=r&&r<=c.len;B.clientX=u?h(c.translate(y,0,0,0,1,n)):r;B.negative=B.y<(x||0);B.category=d&&void 0!==d[B.x]?d[B.x]:B.x;B.isNull||(void 0!==G&&(z=Math.min(z,Math.abs(r-G))),G=r);B.zone=this.zones.length&&B.getZone()}this.closestPointRangePx=z},getValidPoints:function(a,
b){var c=this.chart;return v(a||this.points||[],function(a){return b&&!c.isInsidePlot(a.plotX,a.plotY,c.inverted)?!1:!a.isNull})},setClip:function(a){var b=this.chart,c=this.options,d=b.renderer,e=b.inverted,k=this.clipBox,f=k||b.clipBox,g=this.sharedClipKey||["_sharedClip",a&&a.duration,a&&a.easing,f.height,c.xAxis,c.yAxis].join(),m=b[g],h=b[g+"m"];m||(a&&(f.width=0,b[g+"m"]=h=d.clipRect(-99,e?-b.plotLeft:-b.plotTop,99,e?b.chartWidth:b.chartHeight)),b[g]=m=d.clipRect(f),m.count={length:0});a&&!m.count[this.index]&&
(m.count[this.index]=!0,m.count.length+=1);!1!==c.clip&&(this.group.clip(a||k?m:b.clipRect),this.markerGroup.clip(h),this.sharedClipKey=g);a||(m.count[this.index]&&(delete m.count[this.index],--m.count.length),0===m.count.length&&g&&b[g]&&(k||(b[g]=b[g].destroy()),b[g+"m"]&&(this.markerGroup.clip(),b[g+"m"]=b[g+"m"].destroy())))},animate:function(a){var b=this.chart,c=E(this.options.animation),d;a?this.setClip(c):(d=this.sharedClipKey,(a=b[d])&&a.animate({width:b.plotSizeX},c),b[d+"m"]&&b[d+"m"].animate({width:b.plotSizeX+
99},c),this.animate=null)},afterAnimate:function(){this.setClip();z(this,"afterAnimate")},drawPoints:function(){var a=this.points,b=this.chart,c,d,e,f,g=this.options.marker,h,q,l,n,u=this.markerGroup,x=D(g.enabled,this.xAxis.isRadial?!0:null,this.closestPointRangePx>=2*g.radius);if(!1!==g.enabled||this._hasPointMarkers)for(d=0;d<a.length;d++)e=a[d],c=e.plotY,f=e.graphic,h=e.marker||{},q=!!e.marker,l=x&&void 0===h.enabled||h.enabled,n=e.isInside,l&&m(c)&&null!==e.y?(c=D(h.symbol,this.symbol),e.hasImage=
0===c.indexOf("url"),l=this.markerAttribs(e,e.selected&&"select"),f?f[n?"show":"hide"](!0).animate(l):n&&(0<l.width||e.hasImage)&&(e.graphic=f=b.renderer.symbol(c,l.x,l.y,l.width,l.height,q?h:g).add(u)),f&&f.attr(this.pointAttribs(e,e.selected&&"select")),f&&f.addClass(e.getClassName(),!0)):f&&(e.graphic=f.destroy())},markerAttribs:function(a,b){var c=this.options.marker,d=a.marker||{},e=D(d.radius,c.radius);b&&(c=c.states[b],b=d.states&&d.states[b],e=D(b&&b.radius,c&&c.radius,e+(c&&c.radiusPlus||
0)));a.hasImage&&(e=0);a={x:Math.floor(a.plotX)-e,y:a.plotY-e};e&&(a.width=a.height=2*e);return a},pointAttribs:function(a,b){var c=this.options.marker,d=a&&a.options,e=d&&d.marker||{},f=this.color,k=d&&d.color,g=a&&a.color,d=D(e.lineWidth,c.lineWidth);a=a&&a.zone&&a.zone.color;f=k||a||g||f;a=e.fillColor||c.fillColor||f;f=e.lineColor||c.lineColor||f;b&&(c=c.states[b],b=e.states&&e.states[b]||{},d=D(b.lineWidth,c.lineWidth,d+D(b.lineWidthPlus,c.lineWidthPlus,0)),a=b.fillColor||c.fillColor||a,f=b.lineColor||
c.lineColor||f);return{stroke:f,"stroke-width":d,fill:a}},destroy:function(){var a=this,b=a.chart,c=/AppleWebKit\/533/.test(H.navigator.userAgent),d,g=a.data||[],m,h,q;z(a,"destroy");u(a);f(a.axisTypes||[],function(b){(q=a[b])&&q.series&&(e(q.series,a),q.isDirty=q.forceRedraw=!0)});a.legendItem&&a.chart.legend.destroyItem(a);for(d=g.length;d--;)(m=g[d])&&m.destroy&&m.destroy();a.points=null;clearTimeout(a.animationTimeout);for(h in a)a[h]instanceof x&&!a[h].survive&&(d=c&&"group"===h?"hide":"destroy",
a[h][d]());b.hoverSeries===a&&(b.hoverSeries=null);e(b.series,a);b.orderSeries();for(h in a)delete a[h]},getGraphPath:function(a,b,c){var d=this,e=d.options,k=e.step,g,m=[],h=[],q;a=a||d.points;(g=a.reversed)&&a.reverse();(k={right:1,center:2}[k]||k&&3)&&g&&(k=4-k);!e.connectNulls||b||c||(a=this.getValidPoints(a));f(a,function(f,g){var p=f.plotX,n=f.plotY,u=a[g-1];(f.leftCliff||u&&u.rightCliff)&&!c&&(q=!0);f.isNull&&!l(b)&&0<g?q=!e.connectNulls:f.isNull&&!b?q=!0:(0===g||q?g=["M",f.plotX,f.plotY]:
d.getPointSpline?g=d.getPointSpline(a,f,g):k?(g=1===k?["L",u.plotX,n]:2===k?["L",(u.plotX+p)/2,u.plotY,"L",(u.plotX+p)/2,n]:["L",p,u.plotY],g.push("L",p,n)):g=["L",p,n],h.push(f.x),k&&h.push(f.x),m.push.apply(m,g),q=!1)});m.xMap=h;return d.graphPath=m},drawGraph:function(){var a=this,b=this.options,c=(this.gappedPath||this.getGraphPath).call(this),d=[["graph","highcharts-graph",b.lineColor||this.color,b.dashStyle]];f(this.zones,function(c,e){d.push(["zone-graph-"+e,"highcharts-graph highcharts-zone-graph-"+
e+" "+(c.className||""),c.color||a.color,c.dashStyle||b.dashStyle])});f(d,function(d,e){var f=d[0],g=a[f];g?(g.endX=c.xMap,g.animate({d:c})):c.length&&(a[f]=a.chart.renderer.path(c).addClass(d[1]).attr({zIndex:1}).add(a.group),g={stroke:d[2],"stroke-width":b.lineWidth,fill:a.fillGraph&&a.color||"none"},d[3]?g.dashstyle=d[3]:"square"!==b.linecap&&(g["stroke-linecap"]=g["stroke-linejoin"]="round"),g=a[f].attr(g).shadow(2>e&&b.shadow));g&&(g.startX=c.xMap,g.isArea=c.isArea)})},applyZones:function(){var a=
this,b=this.chart,c=b.renderer,d=this.zones,e,g,m=this.clips||[],h,q=this.graph,l=this.area,n=Math.max(b.chartWidth,b.chartHeight),u=this[(this.zoneAxis||"y")+"Axis"],x,v,r=b.inverted,t,G,H,z,B=!1;d.length&&(q||l)&&u&&void 0!==u.min&&(v=u.reversed,t=u.horiz,q&&q.hide(),l&&l.hide(),x=u.getExtremes(),f(d,function(d,f){e=v?t?b.plotWidth:0:t?0:u.toPixels(x.min);e=Math.min(Math.max(D(g,e),0),n);g=Math.min(Math.max(Math.round(u.toPixels(D(d.value,x.max),!0)),0),n);B&&(e=g=u.toPixels(x.max));G=Math.abs(e-
g);H=Math.min(e,g);z=Math.max(e,g);u.isXAxis?(h={x:r?z:H,y:0,width:G,height:n},t||(h.x=b.plotHeight-h.x)):(h={x:0,y:r?z:H,width:n,height:G},t&&(h.y=b.plotWidth-h.y));r&&c.isVML&&(h=u.isXAxis?{x:0,y:v?H:z,height:h.width,width:b.chartWidth}:{x:h.y-b.plotLeft-b.spacingBox.x,y:0,width:h.height,height:b.chartHeight});m[f]?m[f].animate(h):(m[f]=c.clipRect(h),q&&a["zone-graph-"+f].clip(m[f]),l&&a["zone-area-"+f].clip(m[f]));B=d.value>x.max}),this.clips=m)},invertGroups:function(a){function b(){f(["group",
"markerGroup"],function(b){c[b]&&(d.renderer.isVML&&c[b].attr({width:c.yAxis.len,height:c.xAxis.len}),c[b].width=c.yAxis.len,c[b].height=c.xAxis.len,c[b].invert(a))})}var c=this,d=c.chart,e;c.xAxis&&(e=y(d,"resize",b),y(c,"destroy",e),b(a),c.invertGroups=b)},plotGroup:function(a,b,c,d,e){var f=this[a],g=!f;g&&(this[a]=f=this.chart.renderer.g(b).attr({zIndex:d||.1}).add(e),f.addClass("highcharts-series-"+this.index+" highcharts-"+this.type+"-series highcharts-color-"+this.colorIndex+" "+(this.options.className||
"")));f.attr({visibility:c})[g?"attr":"animate"](this.getPlotBox());return f},getPlotBox:function(){var a=this.chart,b=this.xAxis,c=this.yAxis;a.inverted&&(b=c,c=this.xAxis);return{translateX:b?b.left:a.plotLeft,translateY:c?c.top:a.plotTop,scaleX:1,scaleY:1}},render:function(){var a=this,b=a.chart,c,d=a.options,e=!!a.animate&&b.renderer.isSVG&&E(d.animation).duration,f=a.visible?"inherit":"hidden",g=d.zIndex,h=a.hasRendered,m=b.seriesGroup,q=b.inverted;c=a.plotGroup("group","series",f,g,m);a.markerGroup=
a.plotGroup("markerGroup","markers",f,g,m);e&&a.animate(!0);c.inverted=a.isCartesian?q:!1;a.drawGraph&&(a.drawGraph(),a.applyZones());a.drawDataLabels&&a.drawDataLabels();a.visible&&a.drawPoints();a.drawTracker&&!1!==a.options.enableMouseTracking&&a.drawTracker();a.invertGroups(q);!1===d.clip||a.sharedClipKey||h||c.clip(b.clipRect);e&&a.animate();h||(a.animationTimeout=G(function(){a.afterAnimate()},e));a.isDirty=!1;a.hasRendered=!0},redraw:function(){var a=this.chart,b=this.isDirty||this.isDirtyData,
c=this.group,d=this.xAxis,e=this.yAxis;c&&(a.inverted&&c.attr({width:a.plotWidth,height:a.plotHeight}),c.animate({translateX:D(d&&d.left,a.plotLeft),translateY:D(e&&e.top,a.plotTop)}));this.translate();this.render();b&&delete this.kdTree},kdDimensions:1,kdAxisArray:["clientX","plotY"],searchPoint:function(a,b){var c=this.xAxis,d=this.yAxis,e=this.chart.inverted;return this.searchKDTree({clientX:e?c.len-a.chartY+c.pos:a.chartX-c.pos,plotY:e?d.len-a.chartX+d.pos:a.chartY-d.pos},b)},buildKDTree:function(){function a(c,
d,e){var f,g;if(g=c&&c.length)return f=b.kdAxisArray[d%e],c.sort(function(a,b){return a[f]-b[f]}),g=Math.floor(g/2),{point:c[g],left:a(c.slice(0,g),d+1,e),right:a(c.slice(g+1),d+1,e)}}this.buildingKdTree=!0;var b=this,c=b.kdDimensions;delete b.kdTree;G(function(){b.kdTree=a(b.getValidPoints(null,!b.directTouch),c,c);b.buildingKdTree=!1},b.options.kdNow?0:1)},searchKDTree:function(a,b){function c(a,b,k,h){var m=b.point,q=d.kdAxisArray[k%h],n,p,u=m;p=l(a[e])&&l(m[e])?Math.pow(a[e]-m[e],2):null;n=l(a[f])&&
l(m[f])?Math.pow(a[f]-m[f],2):null;n=(p||0)+(n||0);m.dist=l(n)?Math.sqrt(n):Number.MAX_VALUE;m.distX=l(p)?Math.sqrt(p):Number.MAX_VALUE;q=a[q]-m[q];n=0>q?"left":"right";p=0>q?"right":"left";b[n]&&(n=c(a,b[n],k+1,h),u=n[g]<u[g]?n:m);b[p]&&Math.sqrt(q*q)<u[g]&&(a=c(a,b[p],k+1,h),u=a[g]<u[g]?a:u);return u}var d=this,e=this.kdAxisArray[0],f=this.kdAxisArray[1],g=b?"distX":"dist";this.kdTree||this.buildingKdTree||this.buildKDTree();if(this.kdTree)return c(a,this.kdTree,this.kdDimensions,this.kdDimensions)}})})(J);
(function(a){var y=a.addEvent,E=a.animate,A=a.Axis,B=a.createElement,h=a.css,g=a.defined,t=a.each,r=a.erase,l=a.extend,f=a.fireEvent,e=a.inArray,n=a.isNumber,z=a.isObject,v=a.merge,b=a.pick,m=a.Point,c=a.Series,d=a.seriesTypes,D=a.setAnimation,u=a.splat;l(a.Chart.prototype,{addSeries:function(a,c,d){var e,g=this;a&&(c=b(c,!0),f(g,"addSeries",{options:a},function(){e=g.initSeries(a);g.isDirtyLegend=!0;g.linkSeries();c&&g.redraw(d)}));return e},addAxis:function(a,c,d,e){var f=c?"xAxis":"yAxis",g=this.options;
a=v(a,{index:this[f].length,isX:c});new A(this,a);g[f]=u(g[f]||{});g[f].push(a);b(d,!0)&&this.redraw(e)},showLoading:function(a){var b=this,c=b.options,d=b.loadingDiv,e=c.loading,f=function(){d&&h(d,{left:b.plotLeft+"px",top:b.plotTop+"px",width:b.plotWidth+"px",height:b.plotHeight+"px"})};d||(b.loadingDiv=d=B("div",{className:"highcharts-loading highcharts-loading-hidden"},null,b.container),b.loadingSpan=B("span",{className:"highcharts-loading-inner"},null,d),y(b,"redraw",f));d.className="highcharts-loading";
b.loadingSpan.innerHTML=a||c.lang.loading;h(d,l(e.style,{zIndex:10}));h(b.loadingSpan,e.labelStyle);b.loadingShown||(h(d,{opacity:0,display:""}),E(d,{opacity:e.style.opacity||.5},{duration:e.showDuration||0}));b.loadingShown=!0;f()},hideLoading:function(){var a=this.options,b=this.loadingDiv;b&&(b.className="highcharts-loading highcharts-loading-hidden",E(b,{opacity:0},{duration:a.loading.hideDuration||100,complete:function(){h(b,{display:"none"})}}));this.loadingShown=!1},propsRequireDirtyBox:"backgroundColor borderColor borderWidth margin marginTop marginRight marginBottom marginLeft spacing spacingTop spacingRight spacingBottom spacingLeft borderRadius plotBackgroundColor plotBackgroundImage plotBorderColor plotBorderWidth plotShadow shadow".split(" "),
propsRequireUpdateSeries:"chart.inverted chart.polar chart.ignoreHiddenSeries chart.type colors plotOptions".split(" "),update:function(a,c){var d,f={credits:"addCredits",title:"setTitle",subtitle:"setSubtitle"},k=a.chart,m,h;if(k){v(!0,this.options.chart,k);"className"in k&&this.setClassName(k.className);if("inverted"in k||"polar"in k)this.propFromSeries(),m=!0;for(d in k)k.hasOwnProperty(d)&&(-1!==e("chart."+d,this.propsRequireUpdateSeries)&&(h=!0),-1!==e(d,this.propsRequireDirtyBox)&&(this.isDirtyBox=
!0));"style"in k&&this.renderer.setStyle(k.style)}for(d in a){if(this[d]&&"function"===typeof this[d].update)this[d].update(a[d],!1);else if("function"===typeof this[f[d]])this[f[d]](a[d]);"chart"!==d&&-1!==e(d,this.propsRequireUpdateSeries)&&(h=!0)}a.colors&&(this.options.colors=a.colors);a.plotOptions&&v(!0,this.options.plotOptions,a.plotOptions);t(["xAxis","yAxis","series"],function(b){a[b]&&t(u(a[b]),function(a,c){(c=g(a.id)&&this.get(a.id)||this[b][c])&&c.coll===b&&c.update(a,!1)},this)},this);
m&&t(this.axes,function(a){a.update({},!1)});h&&t(this.series,function(a){a.update({},!1)});a.loading&&v(!0,this.options.loading,a.loading);d=k&&k.width;k=k&&k.height;n(d)&&d!==this.chartWidth||n(k)&&k!==this.chartHeight?this.setSize(d,k):b(c,!0)&&this.redraw()},setSubtitle:function(a){this.setTitle(void 0,a)}});l(m.prototype,{update:function(a,c,d,e){function f(){g.applyOptions(a);null===g.y&&h&&(g.graphic=h.destroy());z(a,!0)&&(h&&h.element&&a&&a.marker&&a.marker.symbol&&(g.graphic=h.destroy()),
a&&a.dataLabels&&g.dataLabel&&(g.dataLabel=g.dataLabel.destroy()));n=g.index;m.updateParallelArrays(g,n);l.data[n]=z(l.data[n],!0)||z(a,!0)?g.options:a;m.isDirty=m.isDirtyData=!0;!m.fixedBox&&m.hasCartesianSeries&&(q.isDirtyBox=!0);"point"===l.legendType&&(q.isDirtyLegend=!0);c&&q.redraw(d)}var g=this,m=g.series,h=g.graphic,n,q=m.chart,l=m.options;c=b(c,!0);!1===e?f():g.firePointEvent("update",{options:a},f)},remove:function(a,b){this.series.removePoint(e(this,this.series.data),a,b)}});l(c.prototype,
{addPoint:function(a,c,d,e){var f=this.options,g=this.data,m=this.chart,h=this.xAxis,h=h&&h.hasNames&&h.names,n=f.data,l,q,u=this.xData,x,v;c=b(c,!0);l={series:this};this.pointClass.prototype.applyOptions.apply(l,[a]);v=l.x;x=u.length;if(this.requireSorting&&v<u[x-1])for(q=!0;x&&u[x-1]>v;)x--;this.updateParallelArrays(l,"splice",x,0,0);this.updateParallelArrays(l,x);h&&l.name&&(h[v]=l.name);n.splice(x,0,a);q&&(this.data.splice(x,0,null),this.processData());"point"===f.legendType&&this.generatePoints();
d&&(g[0]&&g[0].remove?g[0].remove(!1):(g.shift(),this.updateParallelArrays(l,"shift"),n.shift()));this.isDirtyData=this.isDirty=!0;c&&m.redraw(e)},removePoint:function(a,c,d){var e=this,f=e.data,g=f[a],m=e.points,h=e.chart,n=function(){m&&m.length===f.length&&m.splice(a,1);f.splice(a,1);e.options.data.splice(a,1);e.updateParallelArrays(g||{series:e},"splice",a,1);g&&g.destroy();e.isDirty=!0;e.isDirtyData=!0;c&&h.redraw()};D(d,h);c=b(c,!0);g?g.firePointEvent("remove",null,n):n()},remove:function(a,
c,d){function e(){g.destroy();m.isDirtyLegend=m.isDirtyBox=!0;m.linkSeries();b(a,!0)&&m.redraw(c)}var g=this,m=g.chart;!1!==d?f(g,"remove",null,e):e()},update:function(a,c){var e=this,f=this.chart,g=this.userOptions,m=this.oldType||this.type,h=a.type||g.type||f.options.chart.type,n=d[m].prototype,q=["group","markerGroup","dataLabelsGroup"],u;if(h&&h!==m||void 0!==a.zIndex)q.length=0;t(q,function(a){q[a]=e[a];delete e[a]});a=v(g,{animation:!1,index:this.index,pointStart:this.xData[0]},{data:this.options.data},
a);this.remove(!1,null,!1);for(u in n)this[u]=void 0;l(this,d[h||m].prototype);t(q,function(a){e[a]=q[a]});this.init(f,a);this.oldType=m;f.linkSeries();b(c,!0)&&f.redraw(!1)}});l(A.prototype,{update:function(a,c){var d=this.chart;a=d.options[this.coll][this.options.index]=v(this.userOptions,a);this.destroy(!0);this.init(d,l(a,{events:void 0}));d.isDirtyBox=!0;b(c,!0)&&d.redraw()},remove:function(a){for(var c=this.chart,d=this.coll,e=this.series,f=e.length;f--;)e[f]&&e[f].remove(!1);r(c.axes,this);
r(c[d],this);c.options[d].splice(this.options.index,1);t(c[d],function(a,b){a.options.index=b});this.destroy();c.isDirtyBox=!0;b(a,!0)&&c.redraw()},setTitle:function(a,b){this.update({title:a},b)},setCategories:function(a,b){this.update({categories:a},b)}})})(J);(function(a){var y=a.animObject,E=a.color,A=a.each,B=a.extend,h=a.isNumber,g=a.merge,t=a.pick,r=a.Series,l=a.seriesType,f=a.svg;l("column","line",{borderRadius:0,groupPadding:.2,marker:null,pointPadding:.1,minPointLength:0,cropThreshold:50,
pointRange:null,states:{hover:{halo:!1,brightness:.1,shadow:!1},select:{color:"#cccccc",borderColor:"#000000",shadow:!1}},dataLabels:{align:null,verticalAlign:null,y:null},softThreshold:!1,startFromThreshold:!0,stickyTracking:!1,tooltip:{distance:6},threshold:0,borderColor:"#ffffff"},{cropShoulder:0,directTouch:!0,trackerGroups:["group","dataLabelsGroup"],negStacks:!0,init:function(){r.prototype.init.apply(this,arguments);var a=this,f=a.chart;f.hasRendered&&A(f.series,function(e){e.type===a.type&&
(e.isDirty=!0)})},getColumnMetrics:function(){var a=this,f=a.options,g=a.xAxis,h=a.yAxis,b=g.reversed,m,c={},d=0;!1===f.grouping?d=1:A(a.chart.series,function(b){var e=b.options,f=b.yAxis,g;b.type===a.type&&b.visible&&h.len===f.len&&h.pos===f.pos&&(e.stacking?(m=b.stackKey,void 0===c[m]&&(c[m]=d++),g=c[m]):!1!==e.grouping&&(g=d++),b.columnIndex=g)});var l=Math.min(Math.abs(g.transA)*(g.ordinalSlope||f.pointRange||g.closestPointRange||g.tickInterval||1),g.len),u=l*f.groupPadding,q=(l-2*u)/(d||1),f=
Math.min(f.maxPointWidth||g.len,t(f.pointWidth,q*(1-2*f.pointPadding)));a.columnMetrics={width:f,offset:(q-f)/2+(u+((a.columnIndex||0)+(b?1:0))*q-l/2)*(b?-1:1)};return a.columnMetrics},crispCol:function(a,f,g,h){var b=this.chart,e=this.borderWidth,c=-(e%2?.5:0),e=e%2?.5:1;b.inverted&&b.renderer.isVML&&(e+=1);g=Math.round(a+g)+c;a=Math.round(a)+c;h=Math.round(f+h)+e;c=.5>=Math.abs(f)&&.5<h;f=Math.round(f)+e;h-=f;c&&h&&(--f,h+=1);return{x:a,y:f,width:g-a,height:h}},translate:function(){var a=this,f=
a.chart,g=a.options,h=a.dense=2>a.closestPointRange*a.xAxis.transA,h=a.borderWidth=t(g.borderWidth,h?0:1),b=a.yAxis,m=a.translatedThreshold=b.getThreshold(g.threshold),c=t(g.minPointLength,5),d=a.getColumnMetrics(),l=d.width,u=a.barW=Math.max(l,1+2*h),q=a.pointXOffset=d.offset;f.inverted&&(m-=.5);g.pointPadding&&(u=Math.ceil(u));r.prototype.translate.apply(a);A(a.points,function(d){var e=t(d.yBottom,m),g=999+Math.abs(e),g=Math.min(Math.max(-g,d.plotY),b.len+g),h=d.plotX+q,n=u,v=Math.min(g,e),r,x=
Math.max(g,e)-v;Math.abs(x)<c&&c&&(x=c,r=!b.reversed&&!d.negative||b.reversed&&d.negative,v=Math.abs(v-m)>c?e-c:m-(r?c:0));d.barX=h;d.pointWidth=l;d.tooltipPos=f.inverted?[b.len+b.pos-f.plotLeft-g,a.xAxis.len-h-n/2,x]:[h+n/2,g+b.pos-f.plotTop,x];d.shapeType="rect";d.shapeArgs=a.crispCol.apply(a,d.isNull?[d.plotX,b.len/2,0,0]:[h,v,n,x])})},getSymbol:a.noop,drawLegendSymbol:a.LegendSymbolMixin.drawRectangle,drawGraph:function(){this.group[this.dense?"addClass":"removeClass"]("highcharts-dense-data")},
pointAttribs:function(a,f){var e=this.options,h,b=this.pointAttrToOptions||{};h=b.stroke||"borderColor";var m=b["stroke-width"]||"borderWidth",c=a&&a.color||this.color,d=a[h]||e[h]||this.color||c,l=a[m]||e[m]||this[m]||0,b=e.dashStyle;a&&this.zones.length&&(c=(c=a.getZone())&&c.color||a.options.color||this.color);f&&(a=g(e.states[f],a.options.states&&a.options.states[f]||{}),f=a.brightness,c=a.color||void 0!==f&&E(c).brighten(a.brightness).get()||c,d=a[h]||d,l=a[m]||l,b=a.dashStyle||b);h={fill:c,
stroke:d,"stroke-width":l};e.borderRadius&&(h.r=e.borderRadius);b&&(h.dashstyle=b);return h},drawPoints:function(){var a=this,f=this.chart,l=a.options,r=f.renderer,b=l.animationLimit||250,m;A(a.points,function(c){var d=c.graphic;if(h(c.plotY)&&null!==c.y){m=c.shapeArgs;if(d)d[f.pointCount<b?"animate":"attr"](g(m));else c.graphic=d=r[c.shapeType](m).add(c.group||a.group);d.attr(a.pointAttribs(c,c.selected&&"select")).shadow(l.shadow,null,l.stacking&&!l.borderRadius);d.addClass(c.getClassName(),!0)}else d&&
(c.graphic=d.destroy())})},animate:function(a){var e=this,g=this.yAxis,h=e.options,b=this.chart.inverted,m={};f&&(a?(m.scaleY=.001,a=Math.min(g.pos+g.len,Math.max(g.pos,g.toPixels(h.threshold))),b?m.translateX=a-g.len:m.translateY=a,e.group.attr(m)):(m[b?"translateX":"translateY"]=g.pos,e.group.animate(m,B(y(e.options.animation),{step:function(a,b){e.group.attr({scaleY:Math.max(.001,b.pos)})}})),e.animate=null))},remove:function(){var a=this,f=a.chart;f.hasRendered&&A(f.series,function(e){e.type===
a.type&&(e.isDirty=!0)});r.prototype.remove.apply(a,arguments)}})})(J);(function(a){var y=a.Series;a=a.seriesType;a("scatter","line",{lineWidth:0,marker:{enabled:!0},tooltip:{headerFormat:'\x3cspan style\x3d"color:{point.color}"\x3e\u25cf\x3c/span\x3e \x3cspan style\x3d"font-size: 0.85em"\x3e {series.name}\x3c/span\x3e\x3cbr/\x3e',pointFormat:"x: \x3cb\x3e{point.x}\x3c/b\x3e\x3cbr/\x3ey: \x3cb\x3e{point.y}\x3c/b\x3e\x3cbr/\x3e"}},{sorted:!1,requireSorting:!1,noSharedTooltip:!0,trackerGroups:["group",
"markerGroup","dataLabelsGroup"],takeOrdinalPosition:!1,kdDimensions:2,drawGraph:function(){this.options.lineWidth&&y.prototype.drawGraph.call(this)}})})(J);(function(a){var y=a.addEvent,E=a.arrayMax,A=a.defined,B=a.each,h=a.extend,g=a.format,t=a.map,r=a.merge,l=a.noop,f=a.pick,e=a.relativeLength,n=a.Series,z=a.seriesTypes,v=a.stableSort;a.distribute=function(a,e){function b(a,b){return a.target-b.target}var d,f=!0,g=a,h=[],m;m=0;for(d=a.length;d--;)m+=a[d].size;if(m>e){v(a,function(a,b){return(b.rank||
0)-(a.rank||0)});for(m=d=0;m<=e;)m+=a[d].size,d++;h=a.splice(d-1,a.length)}v(a,b);for(a=t(a,function(a){return{size:a.size,targets:[a.target]}});f;){for(d=a.length;d--;)f=a[d],m=(Math.min.apply(0,f.targets)+Math.max.apply(0,f.targets))/2,f.pos=Math.min(Math.max(0,m-f.size/2),e-f.size);d=a.length;for(f=!1;d--;)0<d&&a[d-1].pos+a[d-1].size>a[d].pos&&(a[d-1].size+=a[d].size,a[d-1].targets=a[d-1].targets.concat(a[d].targets),a[d-1].pos+a[d-1].size>e&&(a[d-1].pos=e-a[d-1].size),a.splice(d,1),f=!0)}d=0;
B(a,function(a){var b=0;B(a.targets,function(){g[d].pos=a.pos+b;b+=g[d].size;d++})});g.push.apply(g,h);v(g,b)};n.prototype.drawDataLabels=function(){var a=this,e=a.options,c=e.dataLabels,d=a.points,h,l,q=a.hasRendered||0,n,v,t=f(c.defer,!0),k=a.chart.renderer;if(c.enabled||a._hasPointLabels)a.dlProcessOptions&&a.dlProcessOptions(c),v=a.plotGroup("dataLabelsGroup","data-labels",t&&!q?"hidden":"visible",c.zIndex||6),t&&(v.attr({opacity:+q}),q||y(a,"afterAnimate",function(){a.visible&&v.show(!0);v[e.animation?
"animate":"attr"]({opacity:1},{duration:200})})),l=c,B(d,function(b){var d,m=b.dataLabel,q,u,x,t=b.connector,D=!m,C;h=b.dlOptions||b.options&&b.options.dataLabels;if(d=f(h&&h.enabled,l.enabled)&&null!==b.y)for(u in c=r(l,h),q=b.getLabelConfig(),n=c.format?g(c.format,q):c.formatter.call(q,c),C=c.style,x=c.rotation,C.color=f(c.color,C.color,a.color,"#000000"),"contrast"===C.color&&(C.color=c.inside||0>c.distance||e.stacking?k.getContrast(b.color||a.color):"#000000"),e.cursor&&(C.cursor=e.cursor),q=
{fill:c.backgroundColor,stroke:c.borderColor,"stroke-width":c.borderWidth,r:c.borderRadius||0,rotation:x,padding:c.padding,zIndex:1},q)void 0===q[u]&&delete q[u];!m||d&&A(n)?d&&A(n)&&(m?q.text=n:(m=b.dataLabel=k[x?"text":"label"](n,0,-9999,c.shape,null,null,c.useHTML,null,"data-label"),m.addClass("highcharts-data-label-color-"+b.colorIndex+" "+(c.className||"")+(c.useHTML?"highcharts-tracker":""))),m.attr(q),m.css(C).shadow(c.shadow),m.added||m.add(v),a.alignDataLabel(b,m,c,null,D)):(b.dataLabel=
m.destroy(),t&&(b.connector=t.destroy()))})};n.prototype.alignDataLabel=function(a,e,c,d,g){var b=this.chart,m=b.inverted,l=f(a.plotX,-9999),n=f(a.plotY,-9999),r=e.getBBox(),k,v=c.rotation,t=c.align,D=this.visible&&(a.series.forceDL||b.isInsidePlot(l,Math.round(n),m)||d&&b.isInsidePlot(l,m?d.x+1:d.y+d.height-1,m)),z="justify"===f(c.overflow,"justify");D&&(k=c.style.fontSize,k=b.renderer.fontMetrics(k,e).b,d=h({x:m?b.plotWidth-n:l,y:Math.round(m?b.plotHeight-l:n),width:0,height:0},d),h(c,{width:r.width,
height:r.height}),v?(z=!1,m=b.renderer.rotCorr(k,v),m={x:d.x+c.x+d.width/2+m.x,y:d.y+c.y+{top:0,middle:.5,bottom:1}[c.verticalAlign]*d.height},e[g?"attr":"animate"](m).attr({align:t}),l=(v+720)%360,l=180<l&&360>l,"left"===t?m.y-=l?r.height:0:"center"===t?(m.x-=r.width/2,m.y-=r.height/2):"right"===t&&(m.x-=r.width,m.y-=l?0:r.height)):(e.align(c,null,d),m=e.alignAttr),z?this.justifyDataLabel(e,c,m,r,d,g):f(c.crop,!0)&&(D=b.isInsidePlot(m.x,m.y)&&b.isInsidePlot(m.x+r.width,m.y+r.height)),c.shape&&!v&&
e.attr({anchorX:a.plotX,anchorY:a.plotY}));D||(e.attr({y:-9999}),e.placed=!1)};n.prototype.justifyDataLabel=function(a,e,c,d,f,g){var b=this.chart,h=e.align,m=e.verticalAlign,l,k,n=a.box?0:a.padding||0;l=c.x+n;0>l&&("right"===h?e.align="left":e.x=-l,k=!0);l=c.x+d.width-n;l>b.plotWidth&&("left"===h?e.align="right":e.x=b.plotWidth-l,k=!0);l=c.y+n;0>l&&("bottom"===m?e.verticalAlign="top":e.y=-l,k=!0);l=c.y+d.height-n;l>b.plotHeight&&("top"===m?e.verticalAlign="bottom":e.y=b.plotHeight-l,k=!0);k&&(a.placed=
!g,a.align(e,null,f))};z.pie&&(z.pie.prototype.drawDataLabels=function(){var b=this,e=b.data,c,d=b.chart,g=b.options.dataLabels,h=f(g.connectorPadding,10),l=f(g.connectorWidth,1),r=d.plotWidth,v=d.plotHeight,z,k=g.distance,C=b.center,y=C[2]/2,A=C[1],K=0<k,p,F,M,N,J=[[],[]],O,w,Q,R,P=[0,0,0,0];b.visible&&(g.enabled||b._hasPointLabels)&&(B(e,function(a){a.dataLabel&&a.visible&&a.dataLabel.shortened&&(a.dataLabel.attr({width:"auto"}).css({width:"auto",textOverflow:"clip"}),a.dataLabel.shortened=!1)}),
n.prototype.drawDataLabels.apply(b),B(e,function(a){a.dataLabel&&a.visible&&(J[a.half].push(a),a.dataLabel._pos=null)}),B(J,function(e,f){var m,l,n=e.length,q,u,x;if(n)for(b.sortByAngle(e,f-.5),0<k&&(m=Math.max(0,A-y-k),l=Math.min(A+y+k,d.plotHeight),q=t(e,function(a){if(a.dataLabel)return x=a.dataLabel.getBBox().height||21,{target:a.labelPos[1]-m+x/2,size:x,rank:a.y}}),a.distribute(q,l+x-m)),R=0;R<n;R++)c=e[R],M=c.labelPos,p=c.dataLabel,Q=!1===c.visible?"hidden":"inherit",u=M[1],q?void 0===q[R].pos?
Q="hidden":(N=q[R].size,w=m+q[R].pos):w=u,O=g.justify?C[0]+(f?-1:1)*(y+k):b.getX(w<m+2||w>l-2?u:w,f),p._attr={visibility:Q,align:M[6]},p._pos={x:O+g.x+({left:h,right:-h}[M[6]]||0),y:w+g.y-10},M.x=O,M.y=w,null===b.options.size&&(F=p.getBBox().width,u=null,O-F<h?(u=Math.round(F-O+h),P[3]=Math.max(u,P[3])):O+F>r-h&&(u=Math.round(O+F-r+h),P[1]=Math.max(u,P[1])),0>w-N/2?P[0]=Math.max(Math.round(-w+N/2),P[0]):w+N/2>v&&(P[2]=Math.max(Math.round(w+N/2-v),P[2])),p.sideOverflow=u)}),0===E(P)||this.verifyDataLabelOverflow(P))&&
(this.placeDataLabels(),K&&l&&B(this.points,function(a){var c;z=a.connector;if((p=a.dataLabel)&&p._pos&&a.visible){Q=p._attr.visibility;if(c=!z)a.connector=z=d.renderer.path().addClass("highcharts-data-label-connector highcharts-color-"+a.colorIndex).add(b.dataLabelsGroup),z.attr({"stroke-width":l,stroke:g.connectorColor||a.color||"#666666"});z[c?"attr":"animate"]({d:b.connectorPath(a.labelPos)});z.attr("visibility",Q)}else z&&(a.connector=z.destroy())}))},z.pie.prototype.connectorPath=function(a){var b=
a.x,c=a.y;return f(this.options.dataLabels.softConnector,!0)?["M",b+("left"===a[6]?5:-5),c,"C",b,c,2*a[2]-a[4],2*a[3]-a[5],a[2],a[3],"L",a[4],a[5]]:["M",b+("left"===a[6]?5:-5),c,"L",a[2],a[3],"L",a[4],a[5]]},z.pie.prototype.placeDataLabels=function(){B(this.points,function(a){var b=a.dataLabel;b&&a.visible&&((a=b._pos)?(b.sideOverflow&&(b._attr.width=b.getBBox().width-b.sideOverflow,b.css({width:b._attr.width+"px",textOverflow:"ellipsis"}),b.shortened=!0),b.attr(b._attr),b[b.moved?"animate":"attr"](a),
b.moved=!0):b&&b.attr({y:-9999}))},this)},z.pie.prototype.alignDataLabel=l,z.pie.prototype.verifyDataLabelOverflow=function(a){var b=this.center,c=this.options,d=c.center,f=c.minSize||80,g,h;null!==d[0]?g=Math.max(b[2]-Math.max(a[1],a[3]),f):(g=Math.max(b[2]-a[1]-a[3],f),b[0]+=(a[3]-a[1])/2);null!==d[1]?g=Math.max(Math.min(g,b[2]-Math.max(a[0],a[2])),f):(g=Math.max(Math.min(g,b[2]-a[0]-a[2]),f),b[1]+=(a[0]-a[2])/2);g<b[2]?(b[2]=g,b[3]=Math.min(e(c.innerSize||0,g),g),this.translate(b),this.drawDataLabels&&
this.drawDataLabels()):h=!0;return h});z.column&&(z.column.prototype.alignDataLabel=function(a,e,c,d,g){var b=this.chart.inverted,h=a.series,m=a.dlBox||a.shapeArgs,l=f(a.below,a.plotY>f(this.translatedThreshold,h.yAxis.len)),v=f(c.inside,!!this.options.stacking);m&&(d=r(m),0>d.y&&(d.height+=d.y,d.y=0),m=d.y+d.height-h.yAxis.len,0<m&&(d.height-=m),b&&(d={x:h.yAxis.len-d.y-d.height,y:h.xAxis.len-d.x-d.width,width:d.height,height:d.width}),v||(b?(d.x+=l?0:d.width,d.width=0):(d.y+=l?d.height:0,d.height=
0)));c.align=f(c.align,!b||v?"center":l?"right":"left");c.verticalAlign=f(c.verticalAlign,b||v?"middle":l?"top":"bottom");n.prototype.alignDataLabel.call(this,a,e,c,d,g)})})(J);(function(a){var y=a.Chart,E=a.each,A=a.pick,B=a.addEvent;y.prototype.callbacks.push(function(a){function g(){var g=[];E(a.series||[],function(a){var h=a.options.dataLabels,f=a.dataLabelCollections||["dataLabel"];(h.enabled||a._hasPointLabels)&&!h.allowOverlap&&a.visible&&E(f,function(e){E(a.points,function(a){a[e]&&(a[e].labelrank=
A(a.labelrank,a.shapeArgs&&a.shapeArgs.height),g.push(a[e]))})})});a.hideOverlappingLabels(g)}g();B(a,"redraw",g)});y.prototype.hideOverlappingLabels=function(a){var g=a.length,h,r,l,f,e,n,z,v,b,m=function(a,b,e,f,g,h,m,l){return!(g>a+e||g+m<a||h>b+f||h+l<b)};for(r=0;r<g;r++)if(h=a[r])h.oldOpacity=h.opacity,h.newOpacity=1;a.sort(function(a,b){return(b.labelrank||0)-(a.labelrank||0)});for(r=0;r<g;r++)for(l=a[r],h=r+1;h<g;++h)if(f=a[h],l&&f&&l.placed&&f.placed&&0!==l.newOpacity&&0!==f.newOpacity&&(e=
l.alignAttr,n=f.alignAttr,z=l.parentGroup,v=f.parentGroup,b=2*(l.box?0:l.padding),e=m(e.x+z.translateX,e.y+z.translateY,l.width-b,l.height-b,n.x+v.translateX,n.y+v.translateY,f.width-b,f.height-b)))(l.labelrank<f.labelrank?l:f).newOpacity=0;E(a,function(a){var b,c;a&&(c=a.newOpacity,a.oldOpacity!==c&&a.placed&&(c?a.show(!0):b=function(){a.hide()},a.alignAttr.opacity=c,a[a.isOld?"animate":"attr"](a.alignAttr,null,b)),a.isOld=!0)})}})(J);(function(a){var y=a.addEvent,E=a.Chart,A=a.createElement,B=a.css,
h=a.defaultOptions,g=a.defaultPlotOptions,t=a.each,r=a.extend,l=a.fireEvent,f=a.hasTouch,e=a.inArray,n=a.isObject,z=a.Legend,v=a.merge,b=a.pick,m=a.Point,c=a.Series,d=a.seriesTypes,D=a.svg;a=a.TrackerMixin={drawTrackerPoint:function(){var a=this,b=a.chart.pointer,c=function(a){var c=b.getPointFromEvent(a);if(void 0!==c)c.onMouseOver(a)};t(a.points,function(a){a.graphic&&(a.graphic.element.point=a);a.dataLabel&&(a.dataLabel.div?a.dataLabel.div.point=a:a.dataLabel.element.point=a)});a._hasTracking||
(t(a.trackerGroups,function(d){if(a[d]){a[d].addClass("highcharts-tracker").on("mouseover",c).on("mouseout",function(a){b.onTrackerMouseOut(a)});if(f)a[d].on("touchstart",c);a.options.cursor&&a[d].css(B).css({cursor:a.options.cursor})}}),a._hasTracking=!0)},drawTrackerGraph:function(){var a=this,b=a.options,c=b.trackByArea,d=[].concat(c?a.areaPath:a.graphPath),e=d.length,g=a.chart,h=g.pointer,m=g.renderer,l=g.options.tooltip.snap,n=a.tracker,p,r=function(){if(g.hoverSeries!==a)a.onMouseOver()},v=
"rgba(192,192,192,"+(D?.0001:.002)+")";if(e&&!c)for(p=e+1;p--;)"M"===d[p]&&d.splice(p+1,0,d[p+1]-l,d[p+2],"L"),(p&&"M"===d[p]||p===e)&&d.splice(p,0,"L",d[p-2]+l,d[p-1]);n?n.attr({d:d}):a.graph&&(a.tracker=m.path(d).attr({"stroke-linejoin":"round",visibility:a.visible?"visible":"hidden",stroke:v,fill:c?v:"none","stroke-width":a.graph.strokeWidth()+(c?0:2*l),zIndex:2}).add(a.group),t([a.tracker,a.markerGroup],function(a){a.addClass("highcharts-tracker").on("mouseover",r).on("mouseout",function(a){h.onTrackerMouseOut(a)});
b.cursor&&a.css({cursor:b.cursor});if(f)a.on("touchstart",r)}))}};d.column&&(d.column.prototype.drawTracker=a.drawTrackerPoint);d.pie&&(d.pie.prototype.drawTracker=a.drawTrackerPoint);d.scatter&&(d.scatter.prototype.drawTracker=a.drawTrackerPoint);r(z.prototype,{setItemEvents:function(a,b,c){var d=this,e=d.chart.renderer.boxWrapper,f="highcharts-legend-"+(a.series?"point":"series")+"-active";(c?b:a.legendGroup).on("mouseover",function(){a.setState("hover");e.addClass(f);b.css(d.options.itemHoverStyle)}).on("mouseout",
function(){b.css(a.visible?d.itemStyle:d.itemHiddenStyle);e.removeClass(f);a.setState()}).on("click",function(b){var c=function(){a.setVisible&&a.setVisible()};b={browserEvent:b};a.firePointEvent?a.firePointEvent("legendItemClick",b,c):l(a,"legendItemClick",b,c)})},createCheckboxForItem:function(a){a.checkbox=A("input",{type:"checkbox",checked:a.selected,defaultChecked:a.selected},this.options.itemCheckboxStyle,this.chart.container);y(a.checkbox,"click",function(b){l(a.series||a,"checkboxClick",{checked:b.target.checked,
item:a},function(){a.select()})})}});h.legend.itemStyle.cursor="pointer";r(E.prototype,{showResetZoom:function(){var a=this,b=h.lang,c=a.options.chart.resetZoomButton,d=c.theme,e=d.states,f="chart"===c.relativeTo?null:"plotBox";this.resetZoomButton=a.renderer.button(b.resetZoom,null,null,function(){a.zoomOut()},d,e&&e.hover).attr({align:c.position.align,title:b.resetZoomTitle}).addClass("highcharts-reset-zoom").add().align(c.position,!1,f)},zoomOut:function(){var a=this;l(a,"selection",{resetSelection:!0},
function(){a.zoom()})},zoom:function(a){var c,d=this.pointer,e=!1,f;!a||a.resetSelection?t(this.axes,function(a){c=a.zoom()}):t(a.xAxis.concat(a.yAxis),function(a){var b=a.axis;d[b.isXAxis?"zoomX":"zoomY"]&&(c=b.zoom(a.min,a.max),b.displayBtn&&(e=!0))});f=this.resetZoomButton;e&&!f?this.showResetZoom():!e&&n(f)&&(this.resetZoomButton=f.destroy());c&&this.redraw(b(this.options.chart.animation,a&&a.animation,100>this.pointCount))},pan:function(a,b){var c=this,d=c.hoverPoints,e;d&&t(d,function(a){a.setState()});
t("xy"===b?[1,0]:[1],function(b){b=c[b?"xAxis":"yAxis"][0];var d=b.horiz,f=a[d?"chartX":"chartY"],d=d?"mouseDownX":"mouseDownY",g=c[d],h=(b.pointRange||0)/2,m=b.getExtremes(),k=b.toValue(g-f,!0)+h,h=b.toValue(g+b.len-f,!0)-h,l=h<k,g=l?h:k,k=l?k:h,l=b.toValue(b.toPixels(m.min)-b.minPixelPadding),h=b.toValue(b.toPixels(m.max)+b.minPixelPadding),l=Math.min(m.dataMin,l)-g,m=k-Math.max(m.dataMax,h);b.series.length&&0>l&&0>m&&(b.setExtremes(g,k,!1,!1,{trigger:"pan"}),e=!0);c[d]=f});e&&c.redraw(!1);B(c.container,
{cursor:"move"})}});r(m.prototype,{select:function(a,c){var d=this,f=d.series,g=f.chart;a=b(a,!d.selected);d.firePointEvent(a?"select":"unselect",{accumulate:c},function(){d.selected=d.options.selected=a;f.options.data[e(d,f.data)]=d.options;d.setState(a&&"select");c||t(g.getSelectedPoints(),function(a){a.selected&&a!==d&&(a.selected=a.options.selected=!1,f.options.data[e(a,f.data)]=a.options,a.setState(""),a.firePointEvent("unselect"))})})},onMouseOver:function(a){var b=this.series.chart.pointer;
this.firePointEvent("mouseOver");b.runPointActions(a,this)},onMouseOut:function(){var a=this.series.chart;this.firePointEvent("mouseOut");t(a.hoverPoints||[],function(a){a.setState()});a.hoverPoints=a.hoverPoint=null},importEvents:function(){if(!this.hasImportedEvents){var a=v(this.series.options.point,this.options).events,b;this.events=a;for(b in a)y(this,b,a[b]);this.hasImportedEvents=!0}},setState:function(a,c){var d=Math.floor(this.plotX),e=this.plotY,f=this.series,h=f.options.states[a]||{},m=
g[f.type].marker&&f.options.marker,l=m&&!1===m.enabled,n=m&&m.states&&m.states[a]||{},q=!1===n.enabled,p=f.stateMarkerGraphic,u=this.marker||{},v=f.chart,t=f.halo,z,D=m&&f.markerAttribs;a=a||"";if(!(a===this.state&&!c||this.selected&&"select"!==a||!1===h.enabled||a&&(q||l&&!1===n.enabled)||a&&u.states&&u.states[a]&&!1===u.states[a].enabled)){D&&(z=f.markerAttribs(this,a));if(this.graphic)this.state&&this.graphic.removeClass("highcharts-point-"+this.state),a&&this.graphic.addClass("highcharts-point-"+
a),this.graphic.attr(f.pointAttribs(this,a)),z&&this.graphic.animate(z,b(v.options.chart.animation,n.animation,m.animation)),p&&p.hide();else{if(a&&n){m=u.symbol||f.symbol;p&&p.currentSymbol!==m&&(p=p.destroy());if(p)p[c?"animate":"attr"]({x:z.x,y:z.y});else m&&(f.stateMarkerGraphic=p=v.renderer.symbol(m,z.x,z.y,z.width,z.height).add(f.markerGroup),p.currentSymbol=m);p&&p.attr(f.pointAttribs(this,a))}p&&(p[a&&v.isInsidePlot(d,e,v.inverted)?"show":"hide"](),p.element.point=this)}(d=h.halo)&&d.size?
(t||(f.halo=t=v.renderer.path().add(D?f.markerGroup:f.group)),t[c?"animate":"attr"]({d:this.haloPath(d.size)}),t.attr({"class":"highcharts-halo highcharts-color-"+b(this.colorIndex,f.colorIndex)}),t.point=this,t.attr(r({fill:this.color||f.color,"fill-opacity":d.opacity,zIndex:-1},d.attributes))):t&&t.point&&t.point.haloPath&&t.animate({d:t.point.haloPath(0)});this.state=a}},haloPath:function(a){return this.series.chart.renderer.symbols.circle(Math.floor(this.plotX)-a,this.plotY-a,2*a,2*a)}});r(c.prototype,
{onMouseOver:function(){var a=this.chart,b=a.hoverSeries;if(b&&b!==this)b.onMouseOut();this.options.events.mouseOver&&l(this,"mouseOver");this.setState("hover");a.hoverSeries=this},onMouseOut:function(){var a=this.options,b=this.chart,c=b.tooltip,d=b.hoverPoint;b.hoverSeries=null;if(d)d.onMouseOut();this&&a.events.mouseOut&&l(this,"mouseOut");!c||a.stickyTracking||c.shared&&!this.noSharedTooltip||c.hide();this.setState()},setState:function(a){var c=this,d=c.options,e=c.graph,f=d.states,g=d.lineWidth,
d=0;a=a||"";if(c.state!==a&&(t([c.group,c.markerGroup,c.dataLabelsGroup],function(b){b&&(c.state&&b.removeClass("highcharts-series-"+c.state),a&&b.addClass("highcharts-series-"+a))}),c.state=a,!f[a]||!1!==f[a].enabled)&&(a&&(g=f[a].lineWidth||g+(f[a].lineWidthPlus||0)),e&&!e.dashstyle))for(g={"stroke-width":g},e.animate(g,b(c.chart.options.chart.animation,f[a]&&f[a].animation));c["zone-graph-"+d];)c["zone-graph-"+d].attr(g),d+=1},setVisible:function(a,b){var c=this,d=c.chart,e=c.legendItem,f,g=d.options.chart.ignoreHiddenSeries,
h=c.visible;f=(c.visible=a=c.options.visible=c.userOptions.visible=void 0===a?!h:a)?"show":"hide";t(["group","dataLabelsGroup","markerGroup","tracker","tt"],function(a){if(c[a])c[a][f]()});if(d.hoverSeries===c||(d.hoverPoint&&d.hoverPoint.series)===c)c.onMouseOut();e&&d.legend.colorizeItem(c,a);c.isDirty=!0;c.options.stacking&&t(d.series,function(a){a.options.stacking&&a.visible&&(a.isDirty=!0)});t(c.linkedSeries,function(b){b.setVisible(a,!1)});g&&(d.isDirtyBox=!0);!1!==b&&d.redraw();l(c,f)},show:function(){this.setVisible(!0)},
hide:function(){this.setVisible(!1)},select:function(a){this.selected=a=void 0===a?!this.selected:a;this.checkbox&&(this.checkbox.checked=a);l(this,a?"select":"unselect")},drawTracker:a.drawTrackerGraph})})(J);(function(a){var y=a.Chart,E=a.each,A=a.inArray,B=a.isArray,h=a.isObject,g=a.pick,t=a.splat;y.prototype.setResponsive=function(g){var h=this.options.responsive,f=[],e=this.currentResponsive;h&&h.rules&&E(h.rules,function(e){void 0===e._id&&(e._id=a.uniqueKey());this.matchResponsiveRule(e,f,
g)},this);var n=a.merge.apply(0,a.map(f,function(e){return a.find(h.rules,function(a){return a._id===e}).chartOptions})),f=f.toString()||void 0;f!==(e&&e.ruleIds)&&(e&&this.update(e.undoOptions,g),f?(this.currentResponsive={ruleIds:f,mergedOptions:n,undoOptions:this.currentOptions(n)},this.update(n,g)):this.currentResponsive=void 0)};y.prototype.matchResponsiveRule=function(a,h){var f=a.condition;(f.callback||function(){return this.chartWidth<=g(f.maxWidth,Number.MAX_VALUE)&&this.chartHeight<=g(f.maxHeight,
Number.MAX_VALUE)&&this.chartWidth>=g(f.minWidth,0)&&this.chartHeight>=g(f.minHeight,0)}).call(this)&&h.push(a._id)};y.prototype.currentOptions=function(a){function g(a,f,l,v){var b,e;for(b in a)if(!v&&-1<A(b,["series","xAxis","yAxis"]))for(a[b]=t(a[b]),l[b]=[],e=0;e<a[b].length;e++)f[b][e]&&(l[b][e]={},g(a[b][e],f[b][e],l[b][e],v+1));else h(a[b])?(l[b]=B(a[b])?[]:{},g(a[b],f[b]||{},l[b],v+1)):l[b]=f[b]||null}var f={};g(a,this.options,f,0);return f}})(J);(function(a){var y=a.Axis,E=a.each,A=a.pick;
a=a.wrap;a(y.prototype,"getSeriesExtremes",function(a){var h=this.isXAxis,g,t,r=[],l;h&&E(this.series,function(a,e){a.useMapGeometry&&(r[e]=a.xData,a.xData=[])});a.call(this);h&&(g=A(this.dataMin,Number.MAX_VALUE),t=A(this.dataMax,-Number.MAX_VALUE),E(this.series,function(a,e){a.useMapGeometry&&(g=Math.min(g,A(a.minX,g)),t=Math.max(t,A(a.maxX,g)),a.xData=r[e],l=!0)}),l&&(this.dataMin=g,this.dataMax=t))});a(y.prototype,"setAxisTranslation",function(a){var h=this.chart,g=h.plotWidth/h.plotHeight,h=
h.xAxis[0],t;a.call(this);"yAxis"===this.coll&&void 0!==h.transA&&E(this.series,function(a){a.preserveAspectRatio&&(t=!0)});if(t&&(this.transA=h.transA=Math.min(this.transA,h.transA),a=g/((h.max-h.min)/(this.max-this.min)),a=1>a?this:h,g=(a.max-a.min)*a.transA,a.pixelPadding=a.len-g,a.minPixelPadding=a.pixelPadding/2,g=a.fixTo)){g=g[1]-a.toValue(g[0],!0);g*=a.transA;if(Math.abs(g)>a.minPixelPadding||a.min===a.dataMin&&a.max===a.dataMax)g=0;a.minPixelPadding-=g}});a(y.prototype,"render",function(a){a.call(this);
this.fixTo=null})})(J);(function(a){var y=a.Axis,E=a.Chart,A=a.color,B,h=a.each,g=a.extend,t=a.isNumber,r=a.Legend,l=a.LegendSymbolMixin,f=a.noop,e=a.merge,n=a.pick,z=a.wrap;B=a.ColorAxis=function(){this.init.apply(this,arguments)};g(B.prototype,y.prototype);g(B.prototype,{defaultColorAxisOptions:{lineWidth:0,minPadding:0,maxPadding:0,gridLineWidth:1,tickPixelInterval:72,startOnTick:!0,endOnTick:!0,offset:0,marker:{animation:{duration:50},width:.01,color:"#999999"},labels:{overflow:"justify",rotation:0},
minColor:"#e6ebf5",maxColor:"#003399",tickLength:5,showInLegend:!0},keepProps:["legendGroup","legendItemHeight","legendItemWidth","legendItem","legendSymbol"].concat(y.prototype.keepProps),init:function(a,b){var f="vertical"!==a.options.legend.layout,c;this.coll="colorAxis";c=e(this.defaultColorAxisOptions,{side:f?2:1,reversed:!f},b,{opposite:!f,showEmpty:!1,title:null});y.prototype.init.call(this,a,c);b.dataClasses&&this.initDataClasses(b);this.initStops(b);this.horiz=f;this.zoomEnabled=!1;this.defaultLegendLength=
200},tweenColors:function(a,b,e){var c;b.rgba.length&&a.rgba.length?(a=a.rgba,b=b.rgba,c=1!==b[3]||1!==a[3],a=(c?"rgba(":"rgb(")+Math.round(b[0]+(a[0]-b[0])*(1-e))+","+Math.round(b[1]+(a[1]-b[1])*(1-e))+","+Math.round(b[2]+(a[2]-b[2])*(1-e))+(c?","+(b[3]+(a[3]-b[3])*(1-e)):"")+")"):a=b.input||"none";return a},initDataClasses:function(a){var b=this,f=this.chart,c,d=0,g=f.options.chart.colorCount,l=this.options,n=a.dataClasses.length;this.dataClasses=c=[];this.legendItems=[];h(a.dataClasses,function(a,
h){a=e(a);c.push(a);a.color||("category"===l.dataClassColor?(h=f.options.colors,g=h.length,a.color=h[d],a.colorIndex=d,d++,d===g&&(d=0)):a.color=b.tweenColors(A(l.minColor),A(l.maxColor),2>n?.5:h/(n-1)))})},initStops:function(a){this.stops=a.stops||[[0,this.options.minColor],[1,this.options.maxColor]];h(this.stops,function(a){a.color=A(a[1])})},setOptions:function(a){y.prototype.setOptions.call(this,a);this.options.crosshair=this.options.marker},setAxisSize:function(){var a=this.legendSymbol,b=this.chart,
e=b.options.legend||{},c,d;a?(this.left=e=a.attr("x"),this.top=c=a.attr("y"),this.width=d=a.attr("width"),this.height=a=a.attr("height"),this.right=b.chartWidth-e-d,this.bottom=b.chartHeight-c-a,this.len=this.horiz?d:a,this.pos=this.horiz?e:c):this.len=(this.horiz?e.symbolWidth:e.symbolHeight)||this.defaultLegendLength},toColor:function(a,b){var e=this.stops,c,d,f=this.dataClasses,g,h;if(f)for(h=f.length;h--;){if(g=f[h],c=g.from,e=g.to,(void 0===c||a>=c)&&(void 0===e||a<=e)){d=g.color;b&&(b.dataClass=
h,b.colorIndex=g.colorIndex);break}}else{this.isLog&&(a=this.val2lin(a));a=1-(this.max-a)/(this.max-this.min||1);for(h=e.length;h--&&!(a>e[h][0]););c=e[h]||e[h+1];e=e[h+1]||c;a=1-(e[0]-a)/(e[0]-c[0]||1);d=this.tweenColors(c.color,e.color,a)}return d},getOffset:function(){var a=this.legendGroup,b=this.chart.axisOffset[this.side];a&&(this.axisParent=a,y.prototype.getOffset.call(this),this.added||(this.added=!0,this.labelLeft=0,this.labelRight=this.width),this.chart.axisOffset[this.side]=b)},setLegendColor:function(){var a,
b=this.options,e=this.reversed;a=e?1:0;e=e?0:1;a=this.horiz?[a,0,e,0]:[0,e,0,a];this.legendColor={linearGradient:{x1:a[0],y1:a[1],x2:a[2],y2:a[3]},stops:b.stops||[[0,b.minColor],[1,b.maxColor]]}},drawLegendSymbol:function(a,b){var e=a.padding,c=a.options,d=this.horiz,f=n(c.symbolWidth,d?this.defaultLegendLength:12),g=n(c.symbolHeight,d?12:this.defaultLegendLength),h=n(c.labelPadding,d?16:30),c=n(c.itemDistance,10);this.setLegendColor();b.legendSymbol=this.chart.renderer.rect(0,a.baseline-11,f,g).attr({zIndex:1}).add(b.legendGroup);
this.legendItemWidth=f+e+(d?c:h);this.legendItemHeight=g+e+(d?h:0)},setState:f,visible:!0,setVisible:f,getSeriesExtremes:function(){var a=this.series,b=a.length;this.dataMin=Infinity;for(this.dataMax=-Infinity;b--;)void 0!==a[b].valueMin&&(this.dataMin=Math.min(this.dataMin,a[b].valueMin),this.dataMax=Math.max(this.dataMax,a[b].valueMax))},drawCrosshair:function(a,b){var e=b&&b.plotX,c=b&&b.plotY,d,f=this.pos,g=this.len;b&&(d=this.toPixels(b[b.series.colorKey]),d<f?d=f-2:d>f+g&&(d=f+g+2),b.plotX=
d,b.plotY=this.len-d,y.prototype.drawCrosshair.call(this,a,b),b.plotX=e,b.plotY=c,this.cross&&(this.cross.addClass("highcharts-coloraxis-marker").add(this.legendGroup),this.cross.attr({fill:this.crosshair.color})))},getPlotLinePath:function(a,b,e,c,d){return t(d)?this.horiz?["M",d-4,this.top-6,"L",d+4,this.top-6,d,this.top,"Z"]:["M",this.left,d,"L",this.left-6,d+6,this.left-6,d-6,"Z"]:y.prototype.getPlotLinePath.call(this,a,b,e,c)},update:function(a,b){var f=this.chart,c=f.legend;h(this.series,function(a){a.isDirtyData=
!0});a.dataClasses&&c.allItems&&(h(c.allItems,function(a){a.isDataClass&&a.legendGroup.destroy()}),f.isDirtyLegend=!0);f.options[this.coll]=e(this.userOptions,a);y.prototype.update.call(this,a,b);this.legendItem&&(this.setLegendColor(),c.colorizeItem(this,!0))},getDataClassLegendSymbols:function(){var e=this,b=this.chart,m=this.legendItems,c=b.options.legend,d=c.valueDecimals,n=c.valueSuffix||"",r;m.length||h(this.dataClasses,function(c,t){var q=!0,u=c.from,k=c.to;r="";void 0===u?r="\x3c ":void 0===
k&&(r="\x3e ");void 0!==u&&(r+=a.numberFormat(u,d)+n);void 0!==u&&void 0!==k&&(r+=" - ");void 0!==k&&(r+=a.numberFormat(k,d)+n);m.push(g({chart:b,name:r,options:{},drawLegendSymbol:l.drawRectangle,visible:!0,setState:f,isDataClass:!0,setVisible:function(){q=this.visible=!q;h(e.series,function(a){h(a.points,function(a){a.dataClass===t&&a.setVisible(q)})});b.legend.colorizeItem(this,q)}},c))});return m},name:""});h(["fill","stroke"],function(e){a.Fx.prototype[e+"Setter"]=function(){this.elem.attr(e,
B.prototype.tweenColors(A(this.start),A(this.end),this.pos),null,!0)}});z(E.prototype,"getAxes",function(a){var b=this.options.colorAxis;a.call(this);this.colorAxis=[];b&&new B(this,b)});z(r.prototype,"getAllItems",function(a){var b=[],e=this.chart.colorAxis[0];e&&e.options&&(e.options.showInLegend&&(e.options.dataClasses?b=b.concat(e.getDataClassLegendSymbols()):b.push(e)),h(e.series,function(a){a.options.showInLegend=!1}));return b.concat(a.call(this))});z(r.prototype,"colorizeItem",function(a,
b,e){a.call(this,b,e);e&&b.legendColor&&b.legendSymbol.attr({fill:b.legendColor})})})(J);(function(a){var y=a.defined,E=a.each,A=a.noop,B=a.seriesTypes;a.colorPointMixin={isValid:function(){return null!==this.value},setVisible:function(a){var g=this,h=a?"show":"hide";E(["graphic","dataLabel"],function(a){if(g[a])g[a][h]()})},setState:function(h){a.Point.prototype.setState.call(this,h);this.graphic&&this.graphic.attr({zIndex:"hover"===h?1:0})}};a.colorSeriesMixin={pointArrayMap:["value"],axisTypes:["xAxis",
"yAxis","colorAxis"],optionalAxis:"colorAxis",trackerGroups:["group","markerGroup","dataLabelsGroup"],getSymbol:A,parallelArrays:["x","y","value"],colorKey:"value",pointAttribs:B.column.prototype.pointAttribs,translateColors:function(){var a=this,g=this.options.nullColor,t=this.colorAxis,r=this.colorKey;E(this.data,function(h){var f=h[r];if(f=h.options.color||(h.isNull?g:t&&void 0!==f?t.toColor(f,h):h.color||a.color))h.color=f})},colorAttribs:function(a){var g={};y(a.color)&&(g[this.colorProp||"fill"]=
a.color);return g}}})(J);(function(a){function y(a){a&&(a.preventDefault&&a.preventDefault(),a.stopPropagation&&a.stopPropagation(),a.cancelBubble=!0)}function E(a){this.init(a)}var A=a.addEvent,B=a.Chart,h=a.doc,g=a.each,t=a.extend,r=a.merge,l=a.pick;a=a.wrap;E.prototype.init=function(a){this.chart=a;a.mapNavButtons=[]};E.prototype.update=function(a){var e=this.chart,f=e.options.mapNavigation,g,h,b,m,c,d=function(a){this.handler.call(e,a);y(a)},D=e.mapNavButtons;a&&(f=e.options.mapNavigation=r(e.options.mapNavigation,
a));for(;D.length;)D.pop().destroy();if(l(f.enableButtons,f.enabled)&&!e.renderer.forExport)for(g in a=f.buttons,a)a.hasOwnProperty(g)&&(b=r(f.buttonOptions,a[g]),h=b.theme,h.style=r(b.theme.style,b.style),c=(m=h.states)&&m.hover,m=m&&m.select,h=e.renderer.button(b.text,0,0,d,h,c,m,0,"zoomIn"===g?"topbutton":"bottombutton").addClass("highcharts-map-navigation").attr({width:b.width,height:b.height,title:e.options.lang[g],padding:b.padding,zIndex:5}).add(),h.handler=b.onclick,h.align(t(b,{width:h.width,
height:2*h.height}),null,b.alignTo),A(h.element,"dblclick",y),D.push(h));this.updateEvents(f)};E.prototype.updateEvents=function(a){var e=this.chart;l(a.enableDoubleClickZoom,a.enabled)||a.enableDoubleClickZoomTo?this.unbindDblClick=this.unbindDblClick||A(e.container,"dblclick",function(a){e.pointer.onContainerDblClick(a)}):this.unbindDblClick&&(this.unbindDblClick=this.unbindDblClick());l(a.enableMouseWheelZoom,a.enabled)?this.unbindMouseWheel=this.unbindMouseWheel||A(e.container,void 0===h.onmousewheel?
"DOMMouseScroll":"mousewheel",function(a){e.pointer.onContainerMouseWheel(a);y(a);return!1}):this.unbindMouseWheel&&(this.unbindMouseWheel=this.unbindMouseWheel())};t(B.prototype,{fitToBox:function(a,e){g([["x","width"],["y","height"]],function(f){var g=f[0];f=f[1];a[g]+a[f]>e[g]+e[f]&&(a[f]>e[f]?(a[f]=e[f],a[g]=e[g]):a[g]=e[g]+e[f]-a[f]);a[f]>e[f]&&(a[f]=e[f]);a[g]<e[g]&&(a[g]=e[g])});return a},mapZoom:function(a,e,g,h,r){var b=this.xAxis[0],f=b.max-b.min,c=l(e,b.min+f/2),d=f*a,f=this.yAxis[0],n=
f.max-f.min,t=l(g,f.min+n/2),n=n*a,c=this.fitToBox({x:c-d*(h?(h-b.pos)/b.len:.5),y:t-n*(r?(r-f.pos)/f.len:.5),width:d,height:n},{x:b.dataMin,y:f.dataMin,width:b.dataMax-b.dataMin,height:f.dataMax-f.dataMin}),d=c.x<=b.dataMin&&c.width>=b.dataMax-b.dataMin&&c.y<=f.dataMin&&c.height>=f.dataMax-f.dataMin;h&&(b.fixTo=[h-b.pos,e]);r&&(f.fixTo=[r-f.pos,g]);void 0===a||d?(b.setExtremes(void 0,void 0,!1),f.setExtremes(void 0,void 0,!1)):(b.setExtremes(c.x,c.x+c.width,!1),f.setExtremes(c.y,c.y+c.height,!1));
this.redraw()}});a(B.prototype,"render",function(a){this.mapNavigation=new E(this);this.mapNavigation.update();a.call(this)})})(J);(function(a){var y=a.extend,E=a.pick,A=a.Pointer;a=a.wrap;y(A.prototype,{onContainerDblClick:function(a){var h=this.chart;a=this.normalize(a);h.options.mapNavigation.enableDoubleClickZoomTo?h.pointer.inClass(a.target,"highcharts-tracker")&&h.hoverPoint&&h.hoverPoint.zoomTo():h.isInsidePlot(a.chartX-h.plotLeft,a.chartY-h.plotTop)&&h.mapZoom(.5,h.xAxis[0].toValue(a.chartX),
h.yAxis[0].toValue(a.chartY),a.chartX,a.chartY)},onContainerMouseWheel:function(a){var h=this.chart,g;a=this.normalize(a);g=a.detail||-(a.wheelDelta/120);h.isInsidePlot(a.chartX-h.plotLeft,a.chartY-h.plotTop)&&h.mapZoom(Math.pow(h.options.mapNavigation.mouseWheelSensitivity,g),h.xAxis[0].toValue(a.chartX),h.yAxis[0].toValue(a.chartY),a.chartX,a.chartY)}});a(A.prototype,"zoomOption",function(a){var h=this.chart.options.mapNavigation;E(h.enableTouchZoom,h.enabled)&&(this.chart.options.chart.pinchType=
"xy");a.apply(this,[].slice.call(arguments,1))});a(A.prototype,"pinchTranslate",function(a,h,g,t,r,l,f){a.call(this,h,g,t,r,l,f);"map"===this.chart.options.chart.type&&this.hasZoom&&(a=t.scaleX>t.scaleY,this.pinchTranslateDirection(!a,h,g,t,r,l,f,a?t.scaleX:t.scaleY))})})(J);(function(a){var y=a.color,E=a.ColorAxis,A=a.colorPointMixin,B=a.each,h=a.extend,g=a.isNumber,t=a.map,r=a.merge,l=a.noop,f=a.pick,e=a.isArray,n=a.Point,z=a.Series,v=a.seriesType,b=a.seriesTypes,m=a.splat,c=void 0!==a.doc.documentElement.style.vectorEffect;
v("map","scatter",{allAreas:!0,animation:!1,nullColor:"#f7f7f7",borderColor:"#cccccc",borderWidth:1,marker:null,stickyTracking:!1,joinBy:"hc-key",dataLabels:{formatter:function(){return this.point.value},inside:!0,verticalAlign:"middle",crop:!1,overflow:!1,padding:0},turboThreshold:0,tooltip:{followPointer:!0,pointFormat:"{point.name}: {point.value}\x3cbr/\x3e"},states:{normal:{animation:!0},hover:{brightness:.2,halo:null},select:{color:"#cccccc"}}},r(a.colorSeriesMixin,{type:"map",supportsDrilldown:!0,
getExtremesFromAll:!0,useMapGeometry:!0,forceDL:!0,searchPoint:l,directTouch:!0,preserveAspectRatio:!0,pointArrayMap:["value"],getBox:function(b){var c=Number.MAX_VALUE,d=-c,e=c,h=-c,m=c,l=c,k=this.xAxis,n=this.yAxis,r;B(b||[],function(b){if(b.path){"string"===typeof b.path&&(b.path=a.splitPath(b.path));var k=b.path||[],n=k.length,q=!1,t=-c,u=c,v=-c,x=c,w=b.properties;if(!b._foundBox){for(;n--;)g(k[n])&&(q?(t=Math.max(t,k[n]),u=Math.min(u,k[n])):(v=Math.max(v,k[n]),x=Math.min(x,k[n])),q=!q);b._midX=
u+(t-u)*(b.middleX||w&&w["hc-middle-x"]||.5);b._midY=x+(v-x)*(b.middleY||w&&w["hc-middle-y"]||.5);b._maxX=t;b._minX=u;b._maxY=v;b._minY=x;b.labelrank=f(b.labelrank,(t-u)*(v-x));b._foundBox=!0}d=Math.max(d,b._maxX);e=Math.min(e,b._minX);h=Math.max(h,b._maxY);m=Math.min(m,b._minY);l=Math.min(b._maxX-b._minX,b._maxY-b._minY,l);r=!0}});r&&(this.minY=Math.min(m,f(this.minY,c)),this.maxY=Math.max(h,f(this.maxY,-c)),this.minX=Math.min(e,f(this.minX,c)),this.maxX=Math.max(d,f(this.maxX,-c)),k&&void 0===k.options.minRange&&
(k.minRange=Math.min(5*l,(this.maxX-this.minX)/5,k.minRange||c)),n&&void 0===n.options.minRange&&(n.minRange=Math.min(5*l,(this.maxY-this.minY)/5,n.minRange||c)))},getExtremes:function(){z.prototype.getExtremes.call(this,this.valueData);this.chart.hasRendered&&this.isDirtyData&&this.getBox(this.options.data);this.valueMin=this.dataMin;this.valueMax=this.dataMax;this.dataMin=this.minY;this.dataMax=this.maxY},translatePath:function(a){var b=!1,c=this.xAxis,d=this.yAxis,e=c.min,f=c.transA,c=c.minPixelPadding,
h=d.min,m=d.transA,d=d.minPixelPadding,l,n=[];if(a)for(l=a.length;l--;)g(a[l])?(n[l]=b?(a[l]-e)*f+c:(a[l]-h)*m+d,b=!b):n[l]=a[l];return n},setData:function(b,c,f,h){var d=this.options,l=this.chart.options.chart,n=l&&l.map,k=d.mapData,q=d.joinBy,u=null===q,v=d.keys||this.pointArrayMap,D=[],p={},y,A=this.chart.mapTransforms;!k&&n&&(k="string"===typeof n?a.maps[n]:n);u&&(q="_i");q=this.joinBy=m(q);q[1]||(q[1]=q[0]);b&&B(b,function(a,c){var f=0;if(g(a))b[c]={value:a};else if(e(a)){b[c]={};!d.keys&&a.length>
v.length&&"string"===typeof a[0]&&(b[c]["hc-key"]=a[0],++f);for(var h=0;h<v.length;++h,++f)v[h]&&(b[c][v[h]]=a[f])}u&&(b[c]._i=c)});this.getBox(b);if(this.chart.mapTransforms=A=l&&l.mapTransforms||k&&k["hc-transform"]||A)for(y in A)A.hasOwnProperty(y)&&y.rotation&&(y.cosAngle=Math.cos(y.rotation),y.sinAngle=Math.sin(y.rotation));if(k){"FeatureCollection"===k.type&&(this.mapTitle=k.title,k=a.geojson(k,this.type,this));this.mapData=k;this.mapMap={};for(y=0;y<k.length;y++)l=k[y],n=l.properties,l._i=
y,q[0]&&n&&n[q[0]]&&(l[q[0]]=n[q[0]]),p[l[q[0]]]=l;this.mapMap=p;b&&q[1]&&B(b,function(a){p[a[q[1]]]&&D.push(p[a[q[1]]])});d.allAreas?(this.getBox(k),b=b||[],q[1]&&B(b,function(a){D.push(a[q[1]])}),D="|"+t(D,function(a){return a&&a[q[0]]}).join("|")+"|",B(k,function(a){q[0]&&-1!==D.indexOf("|"+a[q[0]]+"|")||(b.push(r(a,{value:null})),h=!1)})):this.getBox(D)}z.prototype.setData.call(this,b,c,f,h)},drawGraph:l,drawDataLabels:l,doFullTranslate:function(){return this.isDirtyData||this.chart.isResizing||
this.chart.renderer.isVML||!this.baseTrans},translate:function(){var a=this,b=a.xAxis,c=a.yAxis,e=a.doFullTranslate();a.generatePoints();B(a.data,function(d){d.plotX=b.toPixels(d._midX,!0);d.plotY=c.toPixels(d._midY,!0);e&&(d.shapeType="path",d.shapeArgs={d:a.translatePath(d.path)})});a.translateColors()},pointAttribs:function(a,e){e=b.column.prototype.pointAttribs.call(this,a,e);a.isFading&&delete e.fill;c?e["vector-effect"]="non-scaling-stroke":e["stroke-width"]="inherit";return e},drawPoints:function(){var a=
this,e=a.xAxis,f=a.yAxis,g=a.group,h=a.chart,m=h.renderer,l,k,n,r,t=this.baseTrans,v,p,z,y,A;a.transformGroup||(a.transformGroup=m.g().attr({scaleX:1,scaleY:1}).add(g),a.transformGroup.survive=!0);a.doFullTranslate()?(h.hasRendered&&B(a.points,function(b){b.shapeArgs&&(b.shapeArgs.fill=a.pointAttribs(b,b.state).fill)}),a.group=a.transformGroup,b.column.prototype.drawPoints.apply(a),a.group=g,B(a.points,function(a){a.graphic&&(a.name&&a.graphic.addClass("highcharts-name-"+a.name.replace(/ /g,"-").toLowerCase()),
a.properties&&a.properties["hc-key"]&&a.graphic.addClass("highcharts-key-"+a.properties["hc-key"].toLowerCase()))}),this.baseTrans={originX:e.min-e.minPixelPadding/e.transA,originY:f.min-f.minPixelPadding/f.transA+(f.reversed?0:f.len/f.transA),transAX:e.transA,transAY:f.transA},this.transformGroup.animate({translateX:0,translateY:0,scaleX:1,scaleY:1})):(l=e.transA/t.transAX,k=f.transA/t.transAY,n=e.toPixels(t.originX,!0),r=f.toPixels(t.originY,!0),.99<l&&1.01>l&&.99<k&&1.01>k&&(k=l=1,n=Math.round(n),
r=Math.round(r)),v=this.transformGroup,h.renderer.globalAnimation?(p=v.attr("translateX"),z=v.attr("translateY"),y=v.attr("scaleX"),A=v.attr("scaleY"),v.attr({animator:0}).animate({animator:1},{step:function(a,b){v.attr({translateX:p+(n-p)*b.pos,translateY:z+(r-z)*b.pos,scaleX:y+(l-y)*b.pos,scaleY:A+(k-A)*b.pos})}})):v.attr({translateX:n,translateY:r,scaleX:l,scaleY:k}));c||a.group.element.setAttribute("stroke-width",a.options[a.pointAttrToOptions&&a.pointAttrToOptions["stroke-width"]||"borderWidth"]/
(l||1));this.drawMapDataLabels()},drawMapDataLabels:function(){z.prototype.drawDataLabels.call(this);this.dataLabelsGroup&&this.dataLabelsGroup.clip(this.chart.clipRect)},render:function(){var a=this,b=z.prototype.render;a.chart.renderer.isVML&&3E3<a.data.length?setTimeout(function(){b.call(a)}):b.call(a)},animate:function(a){var b=this.options.animation,c=this.group,d=this.xAxis,e=this.yAxis,f=d.pos,g=e.pos;this.chart.renderer.isSVG&&(!0===b&&(b={duration:1E3}),a?c.attr({translateX:f+d.len/2,translateY:g+
e.len/2,scaleX:.001,scaleY:.001}):(c.animate({translateX:f,translateY:g,scaleX:1,scaleY:1},b),this.animate=null))},animateDrilldown:function(a){var b=this.chart.plotBox,c=this.chart.drilldownLevels[this.chart.drilldownLevels.length-1],d=c.bBox,e=this.chart.options.drilldown.animation;a||(a=Math.min(d.width/b.width,d.height/b.height),c.shapeArgs={scaleX:a,scaleY:a,translateX:d.x,translateY:d.y},B(this.points,function(a){a.graphic&&a.graphic.attr(c.shapeArgs).animate({scaleX:1,scaleY:1,translateX:0,
translateY:0},e)}),this.animate=null)},drawLegendSymbol:a.LegendSymbolMixin.drawRectangle,animateDrillupFrom:function(a){b.column.prototype.animateDrillupFrom.call(this,a)},animateDrillupTo:function(a){b.column.prototype.animateDrillupTo.call(this,a)}}),h({applyOptions:function(a,b){a=n.prototype.applyOptions.call(this,a,b);b=this.series;var c=b.joinBy;b.mapData&&((c=void 0!==a[c[1]]&&b.mapMap[a[c[1]]])?(b.xyFromShape&&(a.x=c._midX,a.y=c._midY),h(a,c)):a.value=a.value||null);return a},onMouseOver:function(a){clearTimeout(this.colorInterval);
if(null!==this.value)n.prototype.onMouseOver.call(this,a);else this.series.onMouseOut(a)},onMouseOut:function(){var a=this,b=+new Date,c=y(this.series.pointAttribs(a).fill),e=y(this.series.pointAttribs(a,"hover").fill),f=a.series.options.states.normal.animation,g=f&&(f.duration||500);g&&4===c.rgba.length&&4===e.rgba.length&&"select"!==a.state&&(clearTimeout(a.colorInterval),a.colorInterval=setInterval(function(){var d=(new Date-b)/g,f=a.graphic;1<d&&(d=1);f&&f.attr("fill",E.prototype.tweenColors.call(0,
e,c,d));1<=d&&clearTimeout(a.colorInterval)},13),a.isFading=!0);n.prototype.onMouseOut.call(a);a.isFading=null},zoomTo:function(){var a=this.series;a.xAxis.setExtremes(this._minX,this._maxX,!1);a.yAxis.setExtremes(this._minY,this._maxY,!1);a.chart.redraw()}},A))})(J);(function(a){var y=a.seriesType,E=a.seriesTypes;y("mapline","map",{lineWidth:1,fillColor:"none"},{type:"mapline",colorProp:"stroke",pointAttrToOptions:{stroke:"color","stroke-width":"lineWidth"},pointAttribs:function(a,y){a=E.map.prototype.pointAttribs.call(this,
a,y);a.fill=this.options.fillColor;return a},drawLegendSymbol:E.line.prototype.drawLegendSymbol})})(J);(function(a){var y=a.merge,E=a.Point;a=a.seriesType;a("mappoint","scatter",{dataLabels:{enabled:!0,formatter:function(){return this.point.name},crop:!1,defer:!1,overflow:!1,style:{color:"#000000"}}},{type:"mappoint",forceDL:!0},{applyOptions:function(a,B){a=void 0!==a.lat&&void 0!==a.lon?y(a,this.series.chart.fromLatLonToPoint(a)):a;return E.prototype.applyOptions.call(this,a,B)}})})(J);(function(a){var y=
a.arrayMax,E=a.arrayMin,A=a.Axis,B=a.color,h=a.each,g=a.isNumber,t=a.noop,r=a.pick,l=a.pInt,f=a.Point,e=a.Series,n=a.seriesType,z=a.seriesTypes;n("bubble","scatter",{dataLabels:{formatter:function(){return this.point.z},inside:!0,verticalAlign:"middle"},marker:{lineColor:null,lineWidth:1,radius:null,states:{hover:{radiusPlus:0}},symbol:"circle"},minSize:8,maxSize:"20%",softThreshold:!1,states:{hover:{halo:{size:5}}},tooltip:{pointFormat:"({point.x}, {point.y}), Size: {point.z}"},turboThreshold:0,
zThreshold:0,zoneAxis:"z"},{pointArrayMap:["y","z"],parallelArrays:["x","y","z"],trackerGroups:["markerGroup","dataLabelsGroup"],bubblePadding:!0,zoneAxis:"z",pointAttribs:function(a,b){var f=r(this.options.marker.fillOpacity,.5);a=e.prototype.pointAttribs.call(this,a,b);1!==f&&(a.fill=B(a.fill).setOpacity(f).get("rgba"));return a},getRadii:function(a,b,e,c){var d,f,g,h=this.zData,l=[],m=this.options,n="width"!==m.sizeBy,k=m.zThreshold,r=b-a;f=0;for(d=h.length;f<d;f++)g=h[f],m.sizeByAbsoluteValue&&
null!==g&&(g=Math.abs(g-k),b=Math.max(b-k,Math.abs(a-k)),a=0),null===g?g=null:g<a?g=e/2-1:(g=0<r?(g-a)/r:.5,n&&0<=g&&(g=Math.sqrt(g)),g=Math.ceil(e+g*(c-e))/2),l.push(g);this.radii=l},animate:function(a){var b=this.options.animation;a||(h(this.points,function(a){var c=a.graphic,d;c&&c.width&&(d={x:c.x,y:c.y,width:c.width,height:c.height},c.attr({x:a.plotX,y:a.plotY,width:1,height:1}),c.animate(d,b))}),this.animate=null)},translate:function(){var e,b=this.data,f,c,d=this.radii;z.scatter.prototype.translate.call(this);
for(e=b.length;e--;)f=b[e],c=d?d[e]:0,g(c)&&c>=this.minPxSize/2?(f.marker=a.extend(f.marker,{radius:c,width:2*c,height:2*c}),f.dlBox={x:f.plotX-c,y:f.plotY-c,width:2*c,height:2*c}):f.shapeArgs=f.plotY=f.dlBox=void 0},alignDataLabel:z.column.prototype.alignDataLabel,buildKDTree:t,applyZones:t},{haloPath:function(a){return f.prototype.haloPath.call(this,0===a?0:(this.marker?this.marker.radius||0:0)+a)},ttBelow:!1});A.prototype.beforePadding=function(){var a=this,b=this.len,e=this.chart,c=0,d=b,f=this.isXAxis,
n=f?"xData":"yData",q=this.min,t={},z=Math.min(e.plotWidth,e.plotHeight),B=Number.MAX_VALUE,k=-Number.MAX_VALUE,C=this.max-q,A=b/C,J=[];h(this.series,function(b){var c=b.options;!b.bubblePadding||!b.visible&&e.options.chart.ignoreHiddenSeries||(a.allowZoomOutside=!0,J.push(b),f&&(h(["minSize","maxSize"],function(a){var b=c[a],d=/%$/.test(b),b=l(b);t[a]=d?z*b/100:b}),b.minPxSize=t.minSize,b.maxPxSize=Math.max(t.maxSize,t.minSize),b=b.zData,b.length&&(B=r(c.zMin,Math.min(B,Math.max(E(b),!1===c.displayNegative?
c.zThreshold:-Number.MAX_VALUE))),k=r(c.zMax,Math.max(k,y(b))))))});h(J,function(b){var e=b[n],h=e.length,l;f&&b.getRadii(B,k,b.minPxSize,b.maxPxSize);if(0<C)for(;h--;)g(e[h])&&a.dataMin<=e[h]&&e[h]<=a.dataMax&&(l=b.radii[h],c=Math.min((e[h]-q)*A-l,c),d=Math.max((e[h]-q)*A+l,d))});J.length&&0<C&&!this.isLog&&(d-=b,A*=(b+c-d)/b,h([["min","userMin",c],["max","userMax",d]],function(b){void 0===r(a.options[b[0]],a[b[1]])&&(a[b[0]]+=b[2]/A)}))}})(J);(function(a){var y=a.merge,E=a.Point,A=a.seriesType,
B=a.seriesTypes;B.bubble&&A("mapbubble","bubble",{animationLimit:500,tooltip:{pointFormat:"{point.name}: {point.z}"}},{xyFromShape:!0,type:"mapbubble",pointArrayMap:["z"],getMapData:B.map.prototype.getMapData,getBox:B.map.prototype.getBox,setData:B.map.prototype.setData},{applyOptions:function(a,g){return a&&void 0!==a.lat&&void 0!==a.lon?E.prototype.applyOptions.call(this,y(a,this.series.chart.fromLatLonToPoint(a)),g):B.map.prototype.pointClass.prototype.applyOptions.call(this,a,g)},ttBelow:!1})})(J);
(function(a){var y=a.colorPointMixin,E=a.each,A=a.merge,B=a.noop,h=a.pick,g=a.Series,t=a.seriesType,r=a.seriesTypes;t("heatmap","scatter",{animation:!1,borderWidth:0,nullColor:"#f7f7f7",dataLabels:{formatter:function(){return this.point.value},inside:!0,verticalAlign:"middle",crop:!1,overflow:!1,padding:0},marker:null,pointRange:null,tooltip:{pointFormat:"{point.x}, {point.y}: {point.value}\x3cbr/\x3e"},states:{normal:{animation:!0},hover:{halo:!1,brightness:.2}}},A(a.colorSeriesMixin,{pointArrayMap:["y",
"value"],hasPointSpecificOptions:!0,supportsDrilldown:!0,getExtremesFromAll:!0,directTouch:!0,init:function(){var a;r.scatter.prototype.init.apply(this,arguments);a=this.options;a.pointRange=h(a.pointRange,a.colsize||1);this.yAxis.axisPointRange=a.rowsize||1},translate:function(){var a=this.options,f=this.xAxis,e=this.yAxis,g=function(a,e,b){return Math.min(Math.max(e,a),b)};this.generatePoints();E(this.points,function(h){var l=(a.colsize||1)/2,b=(a.rowsize||1)/2,m=g(Math.round(f.len-f.translate(h.x-
l,0,1,0,1)),-f.len,2*f.len),l=g(Math.round(f.len-f.translate(h.x+l,0,1,0,1)),-f.len,2*f.len),c=g(Math.round(e.translate(h.y-b,0,1,0,1)),-e.len,2*e.len),b=g(Math.round(e.translate(h.y+b,0,1,0,1)),-e.len,2*e.len);h.plotX=h.clientX=(m+l)/2;h.plotY=(c+b)/2;h.shapeType="rect";h.shapeArgs={x:Math.min(m,l),y:Math.min(c,b),width:Math.abs(l-m),height:Math.abs(b-c)}});this.translateColors()},drawPoints:function(){r.column.prototype.drawPoints.call(this);E(this.points,function(a){a.graphic.attr(this.colorAttribs(a))},
this)},animate:B,getBox:B,drawLegendSymbol:a.LegendSymbolMixin.drawRectangle,alignDataLabel:r.column.prototype.alignDataLabel,getExtremes:function(){g.prototype.getExtremes.call(this,this.valueData);this.valueMin=this.dataMin;this.valueMax=this.dataMax;g.prototype.getExtremes.call(this)}}),y)})(J);(function(a){function y(a,f){var e,g,h,l=!1,b=a.x,m=a.y;a=0;for(e=f.length-1;a<f.length;e=a++)g=f[a][1]>m,h=f[e][1]>m,g!==h&&b<(f[e][0]-f[a][0])*(m-f[a][1])/(f[e][1]-f[a][1])+f[a][0]&&(l=!l);return l}var E=
a.Chart,A=a.each,B=a.extend,h=a.format,g=a.merge,t=a.win,r=a.wrap;E.prototype.transformFromLatLon=function(g,f){if(void 0===t.proj4)return a.error(21),{x:0,y:null};g=t.proj4(f.crs,[g.lon,g.lat]);var e=f.cosAngle||f.rotation&&Math.cos(f.rotation),h=f.sinAngle||f.rotation&&Math.sin(f.rotation);g=f.rotation?[g[0]*e+g[1]*h,-g[0]*h+g[1]*e]:g;return{x:((g[0]-(f.xoffset||0))*(f.scale||1)+(f.xpan||0))*(f.jsonres||1)+(f.jsonmarginX||0),y:(((f.yoffset||0)-g[1])*(f.scale||1)+(f.ypan||0))*(f.jsonres||1)-(f.jsonmarginY||
0)}};E.prototype.transformToLatLon=function(g,f){if(void 0===t.proj4)a.error(21);else{g={x:((g.x-(f.jsonmarginX||0))/(f.jsonres||1)-(f.xpan||0))/(f.scale||1)+(f.xoffset||0),y:((-g.y-(f.jsonmarginY||0))/(f.jsonres||1)+(f.ypan||0))/(f.scale||1)+(f.yoffset||0)};var e=f.cosAngle||f.rotation&&Math.cos(f.rotation),h=f.sinAngle||f.rotation&&Math.sin(f.rotation);f=t.proj4(f.crs,"WGS84",f.rotation?{x:g.x*e+g.y*-h,y:g.x*h+g.y*e}:g);return{lat:f.y,lon:f.x}}};E.prototype.fromPointToLatLon=function(g){var f=this.mapTransforms,
e;if(f){for(e in f)if(f.hasOwnProperty(e)&&f[e].hitZone&&y({x:g.x,y:-g.y},f[e].hitZone.coordinates[0]))return this.transformToLatLon(g,f[e]);return this.transformToLatLon(g,f["default"])}a.error(22)};E.prototype.fromLatLonToPoint=function(g){var f=this.mapTransforms,e,h;if(!f)return a.error(22),{x:0,y:null};for(e in f)if(f.hasOwnProperty(e)&&f[e].hitZone&&(h=this.transformFromLatLon(g,f[e]),y({x:h.x,y:-h.y},f[e].hitZone.coordinates[0])))return h;return this.transformFromLatLon(g,f["default"])};a.geojson=
function(a,f,e){var g=[],l=[],r=function(a){var b,c=a.length;l.push("M");for(b=0;b<c;b++)1===b&&l.push("L"),l.push(a[b][0],-a[b][1])};f=f||"map";A(a.features,function(a){var b=a.geometry,c=b.type,b=b.coordinates;a=a.properties;var d;l=[];"map"===f||"mapbubble"===f?("Polygon"===c?(A(b,r),l.push("Z")):"MultiPolygon"===c&&(A(b,function(a){A(a,r)}),l.push("Z")),l.length&&(d={path:l})):"mapline"===f?("LineString"===c?r(b):"MultiLineString"===c&&A(b,r),l.length&&(d={path:l})):"mappoint"===f&&"Point"===
c&&(d={x:b[0],y:-b[1]});d&&g.push(B(d,{name:a.name||a.NAME,properties:a}))});e&&a.copyrightShort&&(e.chart.mapCredits=h(e.chart.options.credits.mapText,{geojson:a}),e.chart.mapCreditsFull=h(e.chart.options.credits.mapTextFull,{geojson:a}));return g};r(E.prototype,"addCredits",function(a,f){f=g(!0,this.options.credits,f);this.mapCredits&&(f.href=null);a.call(this,f);this.credits&&this.mapCreditsFull&&this.credits.attr({title:this.mapCreditsFull})})})(J);(function(a){function y(a,f,g,h,b,l,c,d){return["M",
a+b,f,"L",a+g-l,f,"C",a+g-l/2,f,a+g,f+l/2,a+g,f+l,"L",a+g,f+h-c,"C",a+g,f+h-c/2,a+g-c/2,f+h,a+g-c,f+h,"L",a+d,f+h,"C",a+d/2,f+h,a,f+h-d/2,a,f+h-d,"L",a,f+b,"C",a,f+b/2,a+b/2,f,a+b,f,"Z"]}var E=a.Chart,A=a.defaultOptions,B=a.each,h=a.extend,g=a.merge,t=a.pick,r=a.Renderer,l=a.SVGRenderer,f=a.VMLRenderer;h(A.lang,{zoomIn:"Zoom in",zoomOut:"Zoom out"});A.mapNavigation={buttonOptions:{alignTo:"plotBox",align:"left",verticalAlign:"top",x:0,width:18,height:18,padding:5,style:{fontSize:"15px",fontWeight:"bold"},
theme:{"stroke-width":1,"text-align":"center"}},buttons:{zoomIn:{onclick:function(){this.mapZoom(.5)},text:"+",y:0},zoomOut:{onclick:function(){this.mapZoom(2)},text:"-",y:28}},mouseWheelSensitivity:1.1};a.splitPath=function(a){var e;a=a.replace(/([A-Za-z])/g," $1 ");a=a.replace(/^\s*/,"").replace(/\s*$/,"");a=a.split(/[ ,]+/);for(e=0;e<a.length;e++)/[a-zA-Z]/.test(a[e])||(a[e]=parseFloat(a[e]));return a};a.maps={};l.prototype.symbols.topbutton=function(a,f,g,h,b){return y(a-1,f-1,g,h,b.r,b.r,0,0)};
l.prototype.symbols.bottombutton=function(a,f,g,h,b){return y(a-1,f-1,g,h,0,0,b.r,b.r)};r===f&&B(["topbutton","bottombutton"],function(a){f.prototype.symbols[a]=l.prototype.symbols[a]});a.Map=a.mapChart=function(e,f,h){var l="string"===typeof e||e.nodeName,b=arguments[l?1:0],m={endOnTick:!1,visible:!1,minPadding:0,maxPadding:0,startOnTick:!1},c,d=a.getOptions().credits;c=b.series;b.series=null;b=g({chart:{panning:"xy",type:"map"},credits:{mapText:t(d.mapText,' \u00a9 \x3ca href\x3d"{geojson.copyrightUrl}"\x3e{geojson.copyrightShort}\x3c/a\x3e'),
mapTextFull:t(d.mapTextFull,"{geojson.copyright}")},tooltip:{followTouchMove:!1},xAxis:m,yAxis:g(m,{reversed:!0})},b,{chart:{inverted:!1,alignTicks:!1}});b.series=c;return l?new E(e,b,h):new E(b,f)}})(J);return J});